;;; Cf. [DELAHAYE, J.-P., 2008 : "Dconcertantes conjectures", Pour la science :
;;; 367 : 92 sq.]

;;   Procd de "renversement-addition" d'un nombre. Il suffit en gnral de
;; quelques tapes (de 0  4 environ) pour aboutir  un rsultat palindromique.
;; Deux exceptions notables cependant :
;;     89 / 98  (24 tapes)
;;    689 / 986 (pas de rsultat etc. ????)
;;    196 / 691 (pas de rsultat palindromique trouv  ce jour, malgr des
;;               millions d'tapes calcules)

;;   Prdicat de "palindromit" pour les squences. Rend son argument d'appel
;; <seq> s'il est un palindrome, ou nil sinon.

(defun palindromep (seq)
  (when (equal seq (reverse seq)) seq))

;;   Applique le procd de renversement-addition  <n> jusqu' l'obtention d'un
;; rsultat palindromique. RevAdd n'effectue aucun calcul si <n> est dj un
;; palindrome. Exemple de mise en oeuvre :
;;    (defparameter xx 0)
;;    (pasapas (format t "~D : => ~15T~D~%" (incf xx) (RevAdd xx)))  
;; N.B. : ne pas utiliser pressebouton avec 196, car on n'y a pas la possibilit
;; d'arrter manuellement le processus infini sans tuer LISP !!!

(defun RevAdd (n)
  (do ((etp 0) r)
      ((palindromep (format nil "~D" n)) n)
    (format t "~20T~D : (+ ~D ~D) => ~D~%"
      (incf etp)
      n
      (setq r (read-from-string (reverse (format nil "~D" n))))
      (setq n (+ n r)))))