;;; Ralisation du mot de Thue-Morse, objet ubiquitaire
;$$ Umsetzung der Thue-Morse-Folge, Universelles Objekt               [d..H.B.]
;++ Realization of the Thue-Morse Word, Ubiquitous Object

;; Dclaration de variables globales
;$ Deklaration der globalen Variablen
;+ Declaration of global variables

(defparameter *mot-thue-morse* nil)
(defparameter *longueur-thue-morse* nil)
(defparameter *indice-thue-morse* nil)

;; Initialisation des variables globales intgres dans une fonction
;$ Initialisierung der globalen Variablen, eingebunden in eine Funktion
;+ Initialization of the global variables integrated in a function

(defun zero-thue-morse ()
  (setq *mot-thue-morse* '(0))                       ;dbut du mot de Thue-Morse
  (setq *longueur-thue-morse* 1)                     ;longueur actuelle du mot
  (setq *indice-thue-morse* -1))                     ;initialisation indice

;; Fonction servant  doubler au besoin la longueur de la liste <li>. Elle
;; contiendra un tat du mot de Thue-Morse
;$ Funktion um, wenn ntig, die Lnge der <li>-Liste zu verdoppeln. Sie wird
;$ einen Zustand der Thue-Morse-Folge enthalten
;+ Function used to double, when necessary, the length of the <li> list. It will
;+ contain a state of the Thue-Morse word

(defun etape-thue-morse (li)
  (append li (inverser li)))

;; Auxiliaire  de la prcdente : dans une liste de 0 et de 1, remplece tous les
;; 0 par des 1 et vice-versa
;$ Hilfsfunktion fr das Vorangegangene: ersetze in einer Liste mit 0en und 1en
;$ jede 0 durch 1 und umgekehrt
;+ Auxiliairy for the preceding: in a list of 0 and 1, replaces all 0 by 1 and
;+ vice versa

(defun inverser (li)
  ;   1- Ralisation par rcursion non-terminale
  ;(if (endp li) nil
  ;(cons (mod (1+ (car li)) 2) (inverser (cdr li)))))
  ;   2- Ralisation par (mapcar (lambda... ) ... )
  (mapcar (lambda (x) (mod (1+ x) 2)) li))

;; Gestion de l'ensemble... Rend la lettre suivante du mot de Thue-Morse
;$ Allgemeines Management... Gibt das nchste Glied der Thue-Morse-Folge aus
;+ General managemnt... Outputs the next letter of the Thue-Morse word

(defun lettre-thue-morse ()
  (incf *indice-thue-morse*)                                 ;incrmenter indice
  (when (= *indice-thue-morse* *longueur-thue-morse*)        ;faut-il allonger ?
    (setq *mot-thue-morse* (etape-thue-morse *mot-thue-morse*));doubl. si oui et
    (setq *longueur-thue-morse* (* 2 *longueur-thue-morse*)))   ;noter nlle lngr
  (nth *indice-thue-morse* *mot-thue-morse*))               ;donner lettre suiv.

;; Initialisation au dbut du mot de Thue-Morse par (re)dfinition des variables
;; globales. Sera value au chargement du code LISP. Rinitialisation possible
;;  tout moment. Cette valuation rendra -1, valeur elle-mme rendue par
;; l'objet [maxlisp] lors de la dfinition de son code LISP, pour satisfaire la
;; contrainte de la sortie de l'objet [maxlisp] vers un objet Max *number box*
;$ Initialisierung am Anfang der Thue-Morse-Folge durch (Re)Definition der
;$ globalen Variablen. Wird beim Laden des LISP-Codes ausgefhrt.
;$ Reinitialisierung ist jederzeit mglich. Diese Evaluation wird -1 
;$ zurckgeben, dasselbe wie als Ausgabe des [maxlisp]-Objekts gesendet wird
;$ wenn dessen LISP-Code definiert wird, um eine kompatible Nachricht zu einer
;$ nachfolgenden Max *number box* zu senden.
;+ Initialization at the beginning of the Thue-Morse word by (re)definition of
;+ the global variables. Will be evaluated when loading the LISP code.
;+ Reinitialization possible at any time. This evaluation will return -1, also
;+ sent as output of the [maxlisp] object when its LISP code will be defined, in
;+ order to send a compatible message to a subsequent Max *number box* object

(zero-thue-morse)

;; valuer pour peler le mot de Thue-Morse...
;$ Evaluieren um die Thue-Morse-Folge auszugeben...
;+ Evaluate to spell the Thue-Morse word...
;-    (pasapas (print (lettre-thue-morse)))
;; ... ou imprimer les 1024 premires lettres :
;$ ... oder um die 1024 erste Zeichen auszudrucken:
;+ ... or to print the first 1024 letters:
;-    (dotimes (n 1024) (format t "~A" (lettre-thue-morse))) ; <===
;; ... ou pour rinitialiser le mot  son dbut
;$ ... oder um die Folge an ihren Anfang zurckzusetzen
;+ ... or to reinitialize the word at its beginning
;-    (zero-thue-morse)
