;===============================================================================
;;; 10 --> EXEMPLES NUMRIQUES
;$$ 10 --> NUMERISCHE BEISPIELE                                       [d..H.B.]
;++ 10 --> NUMERICAL EXAMPLES
;===============================================================================

;;; Nombres entiers
;$$ Ganzzahlen
;++ Integer Numbers

;;   Nombres figurs pythagoriciens (VIe sicle avant J.-C.) -- cf. fichier
;; ped.11.1.
;$   Pythagoreische figurale Zahlen (4. Jahrhundert v. Chr.) -- vgl. Datei
;$ ped.11.1.
;+   Pythagorean figured numbers (VIth Century BC)--cf. file ped.11.1.

;;   Quelques nombres premiers -- cf. fichier ped.16.1c, mthode du Crible
;; d'ratosthne (IIIe sicle avant J.-C.).
;$   Einige Primzahlen -- Vgl. Datei ped.16.1c, Methode des Sieb des
;$ Eratosthenes (3. Jahrhundert v. Chr.).
;+   A few prime numbers--cf. file ped.16.1c, method of Eratosthenes's Sieve
;+ (IIIrd Century BC).

(defparameter NbrPrems '(3 5 7 11 13 17 19 23 29 31 37 41 43 47 53 59 61 67 71 73 79 83 89 97 101 103 107 109 113 127 131 137 139 149 151 157 163 167 173 179 181 191 193 197 199 211 223 227 229 233 239 241 251 257 263 269 271 277 281 283 293 307 311 313 317 331 337 347 349 353 359 367 373 379 383 389 397 401 409 419 421 431 433 439 443 449 457 461 463 467 479 487 491 499 503 509 521 523 541 547 557 563 569 571 577 587 593 599 601 607 613 617 619 631 641 643 647 653 659 661 673 677 683 691 701 709 719 727 733 739 743 751 757 761 769 773 787 797 809 811 821 823 827 829 839 853 857 859 863 877 881))

;;   Les nombres de Mersenne sont des nombres premiers dfinis comme
;$   Mersennezahlen sind Primzahlen die bestimmt werden
;+   Mersenne numbers are prime numbers defined
;-    (- (expt 2 <n>) 1)
;;   valuations calculant une squence de (- (expt 2 <n>) 1). Certains sont
;; des nombres premiers de Mersenne et les autres non :
;$   Evaluationen zur Berechnung einer Reihe von (- (expt 2 <n>) 1). Einige
;$ Mersenne-Primzahlen sind und die andere nicht:
;+   Evaluations to compute a sequence of (- (expt 2 <n>) 1). Some are Mersenne
;+ prime numbers and the others not:
;-    (let ((n 0)) (pasapas (print (- (expt 2 (incf n)) 1))))
;; Le plus grand nombre de Mersenne connu est
;$ Die grte bekannte Mersennezahl ist
;+ The largest known Mersenne number is
;-    (- (expt 2 3021377) 1) cf. fichier / Vgl. Datei / cf. file
;-        ped.10.Mersenne.txt
;-            => ca. 2 min. computing time (LispWorks, Intel Core 2 Duo 2,6 GHz)

;;   Petit thorme de Fermat (XVIIe sicle). "Pour tout nombre premier <p>, et
;; tout nombre <a> non divisible par <p>, on a
;$   Kleiner Fermatscher Satz (17. Jahrhundert). "Fr jede Primzahl <p> und jede
;$ nicht durch <p> teilbare Zahl <a>, gilt
;+   Small Fermat theorem (XVIIth Century). "For any prime number <p>, and any
;+ number <a> not divisible by <p>, one has
;-    (<a>**(<p>-1)) := 1 (modulo <p>)"
;- [DAHAN-DALMEDICO, Amy, & PEIFFER, Jeanne, 1986 : Une histoire des
;- mathmatiques, Seuil, Collection Points sciences : 112].
;;   La fonction suivante montre cette proprit pour un nombre premier <p>
;; donn, et <a> croissant de 1  <lim>.
;$   Die folgende Funktion zeigt diese Eigenschaft fr eine gegebene Primzahl
;$ <p> und <a> im Bereich von 1 bis <lim>.
;+   The following function shows this property for a given prime number <p>,
;+ and <a> ranging from 1 to <lim>.

(defun PetitFermat (p lim &aux a q)
  (dotimes (n lim 'ok)
    (setq a (1+ n))
    (when (not (= 0 (mod a p)))
      (format t "~&a=~D, (expt ~D ~D) = ~D = (+ (* ~D ~D) 1) := ~D (mod ~D)~%"
        a a (1- p) (setq q (expt a (1- p))) p (floor (/ q p)) (mod q p) p))))

;;; Nombres rationnels
;$$ Rationale Zahlen
;++ Rational Numbers

;;   Proportion musicale pythagoricienne. "Outre les nombres entiers, les
;; pythagoriciens ont tudi les rapports de nombres entiers et ont cr une
;; thorie des proportions. Parmi celles-ci, ils ont retenu surtout les plus
;; belles, la proportion musicale en particulier :
;$   Pythagoreische musikalische Proportionen. "Neben Ganzzahlen studierten die
;$ Pythagoreer auch Ganzzahlenverhltnisse und schufen eine Theorie der
;$ Proportionen. Unter diesen whlten sie besonders die schnste, die
;$ musikalische Proportion:
;+   Pythagorean musical proportion. "Aside from integer numbers, the
;+ pythagoreans studied the ratios of integer numbers and created a theory of
;+ proportions. Among these, they primarily selected the most beautiful, the
;+ musical proportion in particular:
;-    <p>/<a> = <h>/<q>
;; o <p> et <q> sont deux nombres, <a> tant leur moyenne arithmtique
;$ wobei <p> und <q> zwei Zahlen sind, <a> ist deren arithmetisches Mittel
;+ where <p> and <q> are two numbers, <a> being their arithmetic mean
;-    (/ (+ <p> <q>) 2)
;; et <h> leur moyenne harmonique
;$ und <h> deren harmonisches Mittel
;+ and <h> their harmonic mean
;-    (/ (* 2 <p> <q>) (+ <p> <q>))
;;   La musique tait trs  l'honneur, les connaissances musicales faisant
;; partie du domaine des mathmatiques et, plus particulirement, de la thorie
;; des nombres. Les sons harmonieux, selon Pythagore, sont produits par des
;; rapports exprims en nombres entiers, et, plus le rapport est simple (c'est-
;; -dire plus sa valeur numrique est petite), plus le son est beau"
;; [ibid. : 48].
;$   Musik war hochgeschtzt, musikalisches Wissen war Teil der Mathematik und,
;$ im Besonderen, der Zahlentheorie. Nach Pythagoras ergeben sich harmonische
;$ Klnge aus Ganzzahlenverhltnissen, und je einfacher das Verhltnis (also
;$ je kleiner dessen Zahlenwerte), desto schner der Klang" [ibid.: 48].
;+   Music was very honored, musical knowledge being part of mathematics and,
;+ more particularly, of the theory of numbers. Harmonious sounds, according to
;+ Pythagoras, result from ratios of integer numbers, and, the simpler the ratio
;+ (namely the smaller its numerical value), the more beautiful the sound"
;+ [ibid.: 48].

(defun PropMus (p q)
  (let ((a (/ (+ p q) 2))                                  ;moyenne arithmtique
        (h (/ (* 2 p q) (+ p q))))                         ;moyenne harmonique
    (list (list '/ p a)
          '=
          (list '/ h q)
          '==>
          (list (/ p a) '= (/ h q)))))

;;   Moyenne arithmtique.
;$   Arithmetisches Mittel.
;+   Arithmetic mean.

(defun ma (a b)
  (/ (+ a b) 2))

;;   Moyenne harmonique.
;$   Harmonisches Mittel.
;+   Harmonic mean.

(defun mh (a b)
  (/ (* 2 a b) (+ a b)))

;;   La moyenne gomtrique entre <a> et <b> est <x>, tel que
;$   Das geometrische Mittel zwischen <a> und <b> sei <x>, wobei gilt
;+   The geometric mean between <a> and <b> is <x> such as
;-    a/x = x/b => x = (sqrt (* a b))
;; Cf., par exemple l'enchanement des formats de papier A<n>, tels que
;$ Vgl. z. B. die Folge der A<n>-Papierformate bei denen gilt
;+ Cf., for instance, the sequence of A<n> paper formats such as
;-    a/b = b/(a/2) = (sqrt 2)
;; C'est  dire (hauteur * largeur) :
;$ Das heit also (Hhe * Breite):
;+ Namely (heigth * width):
;-    ...
;-    format A3 := a * b    , tels que a/b         = (sqrt 2)
;-    format A4 := b * a/2  , tels que b/(a/2)     = (sqrt 2)
;-    format A5 := a/2 * b/2, tels que (a/2)/(b/2) = (sqrt 2)
;-    ...
;; La racine carre implique l'apparition quasi infaillible d'irrationnels...
;$ Die Quadratwurzel bedeutet das fast unausweichliche Auftauchen
;$ irrationaler Zahlen...
;+ The square root implies the nearly infallible appearance of irrationals...
;- Exemples : / Beispiele: / Examples:
;-    (mg 1 4)  => 2,  (mg 4 16) => 8,  (mg 2 8)  => 4, (mg 666 2664) => 1332
;- mais... / aber... / but...
;-    (mg 2 4) => 2.828427, (mg 2 5) => 3.1622777, (mg 43 29) => 35.312886
;-    Format A4 : (float (/ 297 210)) => 1.4142857 ~ (sqrt 2)

(defun mg (a b)
  (sqrt (* a b)))

;;   Platon, cf. ped.11.1.
;$   Platon, Vgl. ped.11.1
;$   Plato, cf. ped.11.1.

(defun timee (n)
  (if (<= n 3) n
  (* (timee (- n 2)) (if (evenp n) 2 3))))

(defparameter ser25 (let ((rez nil))
                      (dotimes (n 25)
                        (push (timee (1+ n)) rez))
                      (reverse rez)))

(defparameter ser7 (let ((rez nil))
                      (dotimes (n 7)
                        (push (timee (1+ n)) rez))
                      (reverse rez)))

;-   Serendipity...

;- (mh 1 2) => 4/3
;- (mh 1 (mh 1 (mh 1 (mh 1 (mh 1 (mh 1 (mh 1 2))))))) => 256/255

;- (mh 1 3) => 3/2
;- (mh 1 (mh 1 (mh 1 (mh 1 (mh 1 (mh 1 (mh 1 (mh 1 (mh 1 3))))))))) => 768/767

;- (ma 1 2) => 3/2
;- (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 2)))))))) => 257/256

;- (ma 1 3) => 2
;- (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 3)))))))) => 129/128
;- (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 (ma 1 3))))))))) => 257/256

;- 256 = (expt 2 8)
;- 243 = (expt 3 5)

;- 256/243 = (/ (expt 2 8) (expt 3 5))
;- (float 256/243) => 1.0534979423868314

;- (expt 4/3 5) => 1024/243
;- (expt 3/2 5) => 243/32
;- (expt 9/8 2) => 81/64
;- (expt 9/8 3) => 729/512
;- (log 256/243 9/8) => 0.44247459618086
;- (expt 9/8 0.44247459618086) => 1.0534979423868314

;;   O trouver 9/8 ?
;$   Wo findet man 9/8?
;+   Where is 9/8 to be found?
;-    - (/ (timee 5) (timee 6)) => 9/8
;-    - (/  (ma <n> 2<n>)  (mh <n> 2<n>)) => 9/8 par ex. / zum B. / e.g.
;-        - (/  (ma 1 2)  (mh 1 2)) = (/ 3/2 4/3) => 9/8
;-        - (/  (ma 3 6)  (mh 3 6)) = (/ 9/2   4) => 9/8

;;; Nombres irrationnels
;$$ Irrationale Zahlen
;++ Irrational Numbers

;;   "[...] les pythagoriciens interprtaient les grandeurs gomtriques comme
;; collections discrtes d'units. Ils ne pouvaient donc comparer deux grandeurs
;; que lorsque celles-ci avaient une unit de mesure commune, de sorte que
;; chacune d'elles tait un multiple entier de l'unit commune. [...] Or,
;; voulant rapporter la diagonale d'une carr au ct, les pythagoriciens ont
;; constat avec effroi que ces deux grandeurs n'avaient pas de mesure commune.
;; [...] Cette dcouverte sanctionne l'chec de l'adquation pythagoricienne du
;; monde aux nombres entiers et provoque une premire crise de l'histoire des
;; mathmatiques" [ibid. : 57-58, 48-49].
;$   "[...] die Pythagoreer betrachteten geometrische Gren als eine
;$ Ansammlung diskreter Einheiten. Sie konnten daher zwei Gren nur
;$ vergleichen, wenn diese eine gemeinsame Maeinheit hatten, sodass beide ein
;$ ganzzahliges Vielfaches einer gemeinsamen Einheit waren.  [...] Als sie
;$ jedoch die Diagonale eines Quadrats mit dessen Seite vergleichen wollten,
;$ stellten die Pythagoreer mit Entsetzen fest, dass diese zwei Gren kein
;$ gemeinsames Ma besaen. [...] Diese Entdeckung markiert das Scheitern der
;$ pythagoreischen Weltformel mit Ganzzahlen und lst die erste Krise
;$ in der Geschichte der Mathematik aus" [ibid.: 57-58, 48-49].
;+   "[...] the pythagoreans considered geometric quantities as collections of
;+ discrete units. They could thus compare two quantities only when they
;+ possessed a common measuring unit, such as each of them was an integer
;+ multiple of the common unit. [...] And yet, wanting to compare the diagonal
;+ of a square to the side, the pythagoreans discovered with awe that these two
;+ quantities did not possess a common measure. [...] This discovery marks the
;+ failure of the pythagorean equation of the world to the integer numbers and
;+ initiates a first crisis of the history of mathematics" [ibid.: 57-58, 48-
;+ 49].

;- gaux : / ...: / Equal:
;-    (mg 1 2) = (mg 4/3 3/2) = (mg (mh 1 2) (ma 1 2)) = (sqrt 2) ~ (sqrt 2.0D0)

;;   Approximations de pi, nombre irrationnel et transcendant [ibid. : 51, 68,
;; 120, 121, 125, 126].
;$   Annherungen an pi, die irrationale und transzendentale Zahl  [ibid.: 51,
;$ 68, 120, 121, 125, 126].
;+   Approximations of pi, irrational and transcendental number [ibid.: 51, 68,
;+ 120, 121, 125, 126].

pi                                                 ;variable Common LISP

(setq piEgyp (* 16/9 16/9))                        ;Egypte
(float piEgyp)

(setq piBaby (+ 3 1/8))                            ;Babylone
(float piBaby)

(setq piArchMin (+ 3 10/71))                       ;Archimde, bas de fourchette
(float piArchMin)
(setq piArchMax (+ 3 1/7))                         ;   "  "   haut de fourchette
(float piArchMax)

;;   Al-Khwarizmi (IXe sicle) : approximation de la racine carre. "[...] donne
;; comme rgle d'approximation de la racine carre d'un nombre
;$   Al-Khwarizmi (9. Jahrhundert): Annherung an die Quadratwurzel. "[...] gibt
;$ als Regel zur Annherung an die Quadratwurzel einer Zahl
;+   Al-Khwarizmi (IXth Century): approximation of the square root. "[...] gives
;+ as rule to approximate the square root of a number
;-    <N> = (<a>**2)+<r>
;; la quantit
;$ den Wert
;+ the value
;-    <a>+(<r>/2<a>)
;; Notons que le mot <algorithme>, qui a dsign le systme de l'arithmtique
;; dcimale jusqu' l'aube de l'poque moderne, provient du surnom latinis
;; d'Al-Khwarizmi" [ibid. : 84].
;$ Hinweis: Das Wort <Algorithmus>, welches bis zum Anfang der Moderne das
;$ dezimale arithmetische System bezeichnete, stammt vom latinisierten Nachnamen
;$ Al-Khwarizmi's" [ibid.: 84].
;+ Note that the word <algorithm>, which has designated the decimal arithmetic
;+ system until the dawn of the Modern era, comes from the latinized surname of
;+ Al-Khwarizmi" [ibid.: 84].
;- Exemples : / Beispiele: / Examples:
;-    (sqrt 5)?    5=4+1     = (2**2)+1  => (+ 2 (/ 1 (* 2 2))),   (float 9/4)
;-    (sqrt 17)?   17=16+1   = (4**2)+1  => (+ 4 (/ 1 (* 2 4))),   (float 33/8)
;-    (sqrt 57)?   57=49+8   = (7**2)+8  => (+ 7 (/ 8 (* 2 7))),   (float 53/7)
;-    (sqrt 333)?  333=324+9 = (18**2)+9 => (+ 18 (/ 9 (* 2 18))), (float 73/4)
;; Mmes exemples avec la formule de Ul Uqlidisi (Xe sicle)  [ibid. : 88] :
;$ Dasselbe Beispiel mit Ul Uqlidisi's Formel (10. Jhdt.) [ibid.: 88]:
;+ Same examples with Ul Uqlidisi's formula (Xth Century) [ibid.: 88]:
;-    (sqrt 5)?    ...  => (+ 2 (/ 1 (1+ (* 2 2)))),   (float 11/5)
;-    (sqrt 17)?   ...  => (+ 4 (/ 1 (1+ (* 2 4)))),   (float 37/9)
;-    (sqrt 57)?   ...  => (+ 7 (/ 8 (1+ (* 2 7)))),   (float 113/15)
;-    (sqrt 333)?  ...  => (+ 18 (/ 9 (1+ (* 2 18)))), (float 675/37)

;;; Limite et infini
;$$ Limes und Unendlichkeit
;++ Limit and Infinity

;;   Apories de Znon d'le [v.-490,?], disciple de Parmnide [v.-544,v.-450] :
;; Achille et la tortue, la Dichotomie, le Paradoxe de la flche -- cf. [ibid. :
;; 169-170].
;$   Aporias von Zeno aus Elea [ca. 490 v. Chr., (?)], Schler des Parmenides
;$ [ca. 544 - ca. 450 v. Chr.]: Achilles und die Schildkrte, die Dichotomie,
;$ das Paradox des Pfeils -- Vgl. [ibid.: 169-170].
;+   Aporias of Zeno of Elea [ca.-490,?], pupil of Parmenides [ca.-544,ca.-450]:
;+ Achilles and the Turtle, The Dichotomy, The Paradox of the Arrow--cf. [ibid.:
;+ 169-170].
;;   Aporie de la Dichotomie : approximation en nombres floats, double floats ou
;; rationnels de la srie
;$   Aporie der Dichotomie: Annherung in float, double float oder rationale
;$ Zahlen an die Folge
;+   Aporia of the Dichotomy: float, double float or rational number
;+ approximation of the series
;-    1/2 + 1/4 + 1/8 + 1/16 ... = 1
;- Utilisation : / Anwendung: / Usage:
;-    (Dichotomie <n> &key num) => 1/2 + 1/4 + 1/8 + 1/16 + ... + 1/(2**<n>)
;; Le calcul est effectu en nombres rationnels sauf si le mot-clef :num est
;; 'float ou 'double.
;$ Die Berechnung wird mit rationale Bruchzahlen durchgefhrt falls das
;$ Schlsselwort :num ist 'float oder 'double.
;+ The computation is performed in rational numbers unless the key-word :num is
;+ 'float or 'double.

(defun Dichotomie (fin &key num print)
  (progv '(rz un dx)
         (cond ((equal num 'float) '(0.0 1.0 2.0))
               ((equal num 'double) '(0.0D0 1.0D0 2.0D0))
               (t '(0 1 2)))
    (dotimes (n fin rz)
      (incf rz (/ un (expt dx (+ un n))))
      (when print (format t "~&~4D: ~D~%" (1+ n) rz))) ))

;-  Exemples... / Beispiele... / Examples...

;-    (Dichotomie 4 :print nil) => 15/16
;-    (Dichotomie 4 :num 'float :print nil) => 0.9375
;-    (= (Dichotomie 4) (Dichotomie 4 :num 'float)) => T

;-    (Dichotomie 20 :print nil) => 1048575/1048576
;-    (Dichotomie 20 :num 'float :print nil) => 0.99999905
;-    (= (Dichotomie 20) (Dichotomie 20 :num 'float)) => T

;;    partir d'une valeur limite de <n> (54 en MCL), l'approximation float
;; reste bloque  1.0, tandis que la valeur rationnelle demeure exacte...
;;   LispWorks : bloque  1.0 ds <n> = 25 en single floats. Bloque 
;; 0.9999999999999999D0 pour <n> = 53 en double floats.
;$   Ab einem Grenzwert von <n> (54 in MCL), verharrt die Fliekomma-Annherung
;$ bei 1.0, whrend der rationale Wert exakt bleibt...
;$   LispWorks: verharrt bei 1.0 sobald <n> = 25 in single floats. Verharrt bei
;$ 0.9999999999999999D0 sobald <n> = 53 in double floats.
;+   After a limit value of <n> (54 in MCL), the float approximation blocks at
;+ 1.0, whereas the rational value remains exact...
;+   LispWorks: blocks at 1.0 as soon as <n> = 25 in single floats. Blocks at
;+ 0.9999999999999999D0 for <n> = 53 in double floats.

;-    (Dichotomie 25 :num 'float :print nil) => 1.0
;-    (= (Dichotomie 25) (Dichotomie 25 :num 'float)) => NIL

;-    (Dichotomie 25 :num 'double :print nil) => 0.9999999701976776D0
;-    (= (Dichotomie 25) (Dichotomie 25 :num 'double)) => T

;-    (Dichotomie 100 :print nil)
;-        => 1267650600228229401496703205375/1267650600228229401496703205376
;-    (Dichotomie 100 :num 'float :print nil) => 1.0
;-    (Dichotomie 100 :num 'double :print nil) => 0.9999999999999999D0
;-    (= (Dichotomie 100) (Dichotomie 100 :num 'float)) => NIL
;-    (= (Dichotomie 100) (Dichotomie 100 :num 'double)) => NIL

;;   On a toujours (Dichotomie <n>) => ((2**n)-1)/(2**n) :
;$   Es gilt immer (Dichotomie <n>) => ((2**n)-1)/(2**n):
;+   One always has (Dichotomie <n>) => ((2**n)-1)/(2**n):
;-    (= (/ (1- (expt 2 100)) (expt 2 100)) (Dichotomie 100)) => T

;;; Gnrateur alatoire -- primitive random -- : dmonstration
;$$ Zufallsgenerator -- das Primitive random: Demonstration
;++ Random Generator--random Primitive: Demonstration

;;   Cette version pour LispWorks, qui initialise "alatoirement" son
;; *random-state*. Macintosh Common LISP et Allegro Common LISP (UNIX), au
;; contraire, avaient au dpart un *random-state* standard.
;$   Version fr LispWorks, welches seinen *random-state*  "zufllig"
;$ initialisiert. Im Gegensatz dazu starteten Macintosh Common LISP und Allegro
;$ Common LISP (UNIX) mit einem Standard-*random-state*.
;+   This version for LispWorks, which "randomly" initializes its
;+ *random-state*. Macintosh Common LISP and Allegro Common LISP (UNIX), on the
;+ contrary, started with a standard *random-state*.

;; Sq. 10 par dfaut : toujours diffrente...
;$ Sq. 10 standardmig: immer verschieden...
;+ Sq. 10 by default: always different...

;    (iter (random 100) 10)

;    (iter (random 1.0) 10)

;- Sq...

;    (continu (print (random 1.0)))

;----

;; (1) Mmoriser la valeur courante ou
;; (2) une valeur quelconque...
;$ (1) Speichere den laufenden oder
;$ (2) einen beliebigen Wert...
;+ (1) Memorize the current or
;+ (2) any other value...

(setq mem1 *random-state*)

(setq mem2 #S(RANDOM-STATE :J 14 :K 45 :SEED #(49828127 73920122 10143644 62986361 37312892 100499397 71925765 44896067 52122290 105887187 4264516 2697486 119378397 131078075 50834167 67094856 61871075 29955069 13825253 116077582 76287738 14984934 117408196 132793031 21644060 85878325 36033240 71340992 128395357 68594893 85370978 31208202 1225960 70503284 85547534 133783075 48638404 29825250 83410709 58811327 90739801 78015933 72452064 124147586 56363629 129515656 46132097 42829106 72692275 125594265 93781380 10342341 41318028 101914466 130005957)))

;; => Sq. toujours identique !!!
;$ => Sq. immer gleich!!!
;+ => Sq. always identical!!!

;-    (setq *random-state* (make-random-state mem1))

;-    (setq *random-state* (make-random-state mem2));=> 78, 57, 50, 49, 62...

;-    (iter (random 100) 10) ; x 10...
;-    (iter (random 1.0) 10) ; x 10...

;----

;; Exemples d'horloge...
;$ Uhr-Beispiele...
;+ Clock examples...

;-    (get-universal-time)

;; => Sq. toujours diffrente !!!
;$ => Sq. immer verschieden!!!
;+ => Sq. always different!!!

;-    (setq *random-state* (make-random-state t))

;-    (iter (random 100) 10)


