;; <1> Definition des Wurzel-Paths
(setq common-lisp::*wurzel-path*
  (merge-pathnames
    (make-pathname :directory (pathname-directory *load-truename*)) "*"))
;; <2> Zustzliche Aufladung(en)
(dolist (lefic '(
                 "ped.11.3.lisp"
                 ))
  (load (setq lefic (merge-pathnames lefic common-lisp::*wurzel-path*)))
  (format t "~&;; Geladen ~S~%" lefic))
;===============================================================================
;;; 11 --> DE LA RCURSIVIT, PAR L'EXEMPLE :
;;;        11.2b - Manipulations de listes
;$$ 11 --> REKURSIVITT MIT BEISPIELEN:
;$$        11.2b - Manipulation von Listen                            [d..H.B.]
;++ 11 --> OF RECURSIVITY THROUGH EXAMPLES:
;++        11.2b - List Manipulations
;===============================================================================

;;    concater <a> <b> => forme une liste comportant les lments de la liste
;;                        <a> suivis des lments de la liste <b>. quivalentes
;;                        LISP : append et concatenate
;;   Version 1 : trop "analogique", et... recourt  append !
;$    concater <a> <b> => bildet eine Liste aus den Elementen der Liste <a>
;$                        gefolgt von den Elementen der Liste <b>. 
;$                        LISP-Equivalent: append und concatenate
;$   Version 1: zu "analog", und... benutzt append!
;+    concater <a> <b> => forms a list composed of the elements of the list <a>
;+                        followed by the elements of the list <b>. LISP
;+                        equivalents: append and concatenate
;+   Version 1: too "analog", and... relies on append!

(defun concater1 (a b)
  (if (null a) b                                        ;"scotcher" fin de <a>
  (concater1 (butlast a) (append (last a) b))))         ;au dbut de <b>...

;;   Version 2 : la mme, du moins sans recourir  append...
;$   Version 2: das gleiche, aber ohne append zu benutzen...
;+   Version 2: the same, however not requiring append...

(defun concater2 (a b)
  (if (null a) b
  (concater2 (butlast a) (cons (car (last a)) b))))

;;   Version 3 : de l'audace !
;$   Version 3: Versuchen wir's!
;+   Version 3: let's dare!

(defun concater3 (a b)
  (if (null a) b                                        ;cas trivial
  (cons (car a) (concater3 (cdr a) b))))                ;le faire... !!!

;;    effacer <s> <li> => supprime toutes les occurrences de <s> au premier
;;                        niveau de <li>. quivalente LISP : remove
;$    effacer <s> <li> => lscht alle Vorkommen von <s> auf der ersten Ebene der
;$                        Liste <li>. LISP-Equivalent: remove
;+    effacer <s> <li> => suppresses all the occurrences of <s> at the first
;+                        level of <li>. LISP equivalent: remove

(defun effacer (s li)
  (cond ((null li)          li)                         ;cas trivial
        ((equal s (car li)) (effacer s (cdr li)))       ;OUI et suite...
        (t (cons (car li)   (effacer s (cdr li))))))    ;NON et suite...

;;    inter <a> <b> => forme une liste des lments communs aux listes <a> et
;;                     <b>
;$    inter <a> <b> => erstellt eine Liste der gemeinsamen Elemente der Listen 
;$                     <a> und <b>
;+    inter <a> <b> => forms a list of the elements common to <a> and <b>

(defun inter (a b) (cond
  ((null a) ())                                         ;cas terminal
  ((member (car a) b :test #'equal)                     ;OUI et voir...
    (prins "on garde : " (car a))
    (cons (car a) (inter (cdr a) b)))                   ;... la suite...
  (t (prins "on jette : " (car a))                      ;NON et voir la suite...
     (inter (cdr a) b))))

;;    antidbl <li> => limination des rptitions d'lments redondants de la
;;                    liste <li>. quivalente LISP : remove-duplicates
;$    antidbl <li> => Entfernung redundanter wiederholter Elemente in der Liste
;$                    <li>. LISP-Equivalent: remove-duplicates
;+    antidbl <li> => elimination of the redundant repeated elements of the list
;+                    <li>. LISP equivalent: remove-duplicates

(defun antidbl (li) (cond
  ((endp li) ())
  ((not (member (car li) (cdr li) :test #'equal))       ;OUI :  garder...
   (prins "on garde : " (car li))
   (cons (car li) (antidbl (cdr li))))                  ;... et voir la suite...
  (t (prins "on jette : " (car li))                     ;NON et voir la suite...
     (antidbl (cdr li)))))

;;    substit <nf> <vx> <li> => remplace,  tous les niveaux de <li>, les
;;                              occurrences de <vx> par <nf>. quivalente LISP :
;;                              (subst ... :test #'equal)
;$    substit <nf> <vx> <li> => ersetzt alle Vorkommen von <vx> durch <nf> auf
;$                              allen Ebenen von <li>. LISP-Equivalent:
;$                              (subst ... :test #'equal)
;+    substit <nf> <vx> <li> => replaces, at all levels of <li>, the occurrences
;+                              of <vx> by <nf>. LISP equivalent:
;+                              (subst ... :test #'equal)

(defun substit (nf vx li) (cond                         ;DCOMPOSITION D'ARBRE :
  ((equal li vx) nf)                                    ;cas de remplacement
  ((atom li)     li)                                    ;cas terminal
  (t             (cons (substit nf vx (car li))         ;rcursion dans car...
                       (substit nf vx (cdr li))))))     ;rcursion dans cdr...

;;    desos <li> => liste de tous les atomes,  tous les niveaux, de <li>.
;;                  quivaut  "dsosser" <li>, en supprimant toutes ses
;;                  sous-listes - en dtruisant toutes ses sous-structures
;$    desos <li> => Liste aller Atome auf allen Ebenen der <li>. Entspricht dem 
;$                  "entbeinen" der Liste <li>, durch Unterdrcken aller ihrer
;$                  Sublistenmarkmale -- durch Auslschen ihrer Substrukturen
;+    desos <li> => list of all atoms of <li>, at all levels. Amounts to
;+                  "boning" <li>, by suppressing all its sub-list marks--by
;+                  destroying its sub-structures

(defun desos1 (li)                                      ;DCOMPOSITION D'ARBRE :
  (cond ((endp li) ())                                  ;cas trivial
        ((listp (car li))                               ;car  dcomposer : ...
          (append (desos1 (car li))                     ;... desos1 du car et...
                  (desos1 (cdr li))))                   ;... desos1 du cdr...
        (t (append (list (car li))                      ;car OK :  garder et...
                   (desos1 (cdr li))))))                ;... desos1 du cdr...

(defun desos2 (li)                                      ;variante plus efficace
  (cond ((atom li) li)                                  ;cas trivial succinct
        ((listp (car li))                               ;car  dcomposer : ...
          (append (desos2 (car li))                     ;... desos2 du car et...
                  (desos2 (cdr li))))                   ;... desos2 du cdr...
        (t (cons (car li)                ;<-- variante - car OK :  garder et...
                 (desos2 (cdr li))))))                  ;... desos2 du cdr...

;;    retro <li> NIL => rtrograde des lments de premier niveau de la liste
;;                      <li>. quivalente LISP : reverse. N.B. : second argument
;;                      inutile!
;$    retro <li> NIL => Umkehrung der Elemente der ersten Ebene der Liste <li>.
;$                      LISP-Equivalent: reverse. N.B.: nutzloses zweites
;$                      Argument!
;+    retro <li> NIL => retrograde of the first level elements of list <li>.
;+                      LISP equivalent: reverse. N.B.: useless second argument!

(defun retro1 (li rez) (declare (notinline retro1))              ;Cf. [Wertz:28]
  (if (null li) rez
  (retro1 (cdr li) (cons (car li) rez))))

;;   Si l'on utilise append, au lieu de cons, pour construire le rsultat, la
;; variable auxiliaire initialise  nil peut tre vite. Tester en traant
;; ventuellement aussi append.
;$   Wenn append anstatt cons zur Konstruktion des Ergebnisses benutzt wird,
;$ kann die mit nil initialisierte Hilfsvariable vermieden werden. Eventuell mit
;$ Ablaufverfolgung auch von append prfen.
;+   If append is used to construct the result, instead of cons, the auxiliairy
;+ variable initialized to nil can be avoided. Eventually test while also
;+ tracing append.

(defun retro2 (li) (declare (notinline retro2))
  (if (null li) nil
  (append (retro2 (cdr li)) (cons (car li) nil))))

;;   Cf. [<Manuel Le_Lisp>:72] -- boutade d'efficacit douteuse !
;$   Vgl. [<Manuel Le_Lisp>:72] -- dubiose Witzelei!
;+   Cf. [<Manuel Le_Lisp>:72] -- dubious wisecrack!

(defun retro3 (li) (declare (notinline retro3))
  (if (null (cdr li)) li
  (cons (car (retro3 (cdr li)))
        (retro3 (cons (car li)
                      (retro3 (cdr (retro3 (cdr li)))))))))

;;   Mais retro3, rduite  son essentiel, donne un excellent algorithme !
;$   Wenn man allerdings die Funktion retro3 auf ihre Hauptsache reduziert,
;$ ergibt sich ein ausgezeichneter Algorithmus!
;+   However, reduced to its essence, retro3 gives an excellent algorithm!

(defun retro31 (li) (declare (notinline retro31))
  (if (null (cdr li)) li
  (cons (car (last li))
        (retro31 (butlast li)))))

;;   Autre solution simple. quivaut  retro2, mais avec un niveau de recursion
;; de moins.
;$   Andere einfache Lsung. Gleichwertig zu retro23, aber mit einer
;$ Rekursionsebene weniger.
;+   Other simple solution. Equivalent to retro2, but with one less recursion
;+ level.

(defun retro4 (li) (declare (notinline retro4))
  (if (null (cdr li)) li
  (append (retro4 (cdr li)) (list (car li)))))

