;; <1> Definition des Wurzel-Paths
(setq common-lisp::*wurzel-path*
  (merge-pathnames
    (make-pathname :directory (pathname-directory *load-truename*)) "*"))
;; <2> Zustzliche Aufladung(en)
(dolist (lefic '(
                 "ped.08.lisp"
                 ))
  (load (setq lefic (merge-pathnames lefic common-lisp::*wurzel-path*)))
  (format t "~&;; Geladen ~S~%" lefic))
;===============================================================================
;;; 11 --> DE LA RCURSIVIT, PAR L'EXEMPLE :
;;;        11.3 - Autres exemples varis
;$$ 11 --> REKURSIVITT MIT BEISPIELEN:
;$$        11.3 - Verschiedene andere Beispiele                       [d..H.B.]
;++ 11 --> OF RECURSIVITY THROUGH EXAMPLES:
;++        11.3 - Various Other Examples
;===============================================================================

;;   Utilitaire d'impression d'une squence quelconque de formes en une seule
;; ligne de texte : algorithme du genre "analyse de listes", cf. ped.11.2a.
;;    prins1 {<forme(s)>}* => impression squentielle de(s) <forme(s)>
;$   Werkzeug, um eine Folge von Strukturen in einer einzigen Zeile auszugeben:
;$ Algorithmus typisch fr die Analyse von Listen, Vgl. ped.11.2a.
;$    prins1 {<forme(s)>}* => sequentielle Ausgabe der <forme(s)>
;+   Utility for the printout of any sequence of forms on a single line of text:
;+ algorithm typical of list analysis, cf. ped.11.2a.
;+    prins1 {<forme(s)>}* => sequential printout of <forme(s)>
;-  Exemple d'utilisation : / Anwendungsbeispiel: / Example usage:
;-    (prins1 "a = " a " ; b = " b ".")

(defun prins1 (&rest li)                  ;fonction interface : limination du
  (if (endp li) nil                       ;cas trivial, et emballage d'un nombre
  (prinss li)))                           ;inconnu de formes en une seule liste

(defun prinss (li)                        ;fonction effective
  (cond ((endp li) (terpri) t)            ;cas terminal
        (t         (princ (car li))       ;imprimer la premire...
                   (prinss (cdr li)))))   ;... et continuer...

;;   "Rotations", "permutations circulaires" :
;$   "Rotationen", "zirkulre Permutationen" :
;+   "Rotations", "circular permutations":
;-
;- <==G==\   " Gauche"                                  " Droite"      /==D==>
;- (*    *    *    *    *    *    *)           (*    *    *    *    *    *    *)
;-  \----------------------------->             <-----------------------------/
;-
;;    rotag2 <li> &optional (<n> 1) => des lments de <li> de 1 ou <n>
;;                                     positions  gauche
;;    rotad2 <li> &optional (<n> 1) => des lments de <li> de 1 ou <n>
;;                                     positions  droite
;$    rotag2 <li> &optional (<n> 1) => rotiert die Elemente von <li>  1 oder <n>
;$                                     Positionen nach links
;$    rotad2 <li> &optional (<n> 1) => rotiert die Elemente von <li>  1 oder <n>
;$                                     Positionen nach rechts
;+    rotag2 <li> &optional (<n> 1) => of the elements of <li> of 1 or <n>
;+                                     positions left
;+    rotad2 <li> &optional (<n> 1) => of the elements of <li> of 1 or <n>
;+                                     positions right

(defun rotag2 (li &optional (n 1))
  (cond ((atom li) li)                                         ;cas atome ou ()
        ((zerop n) li)                                         ;cas terminal
        (t (rotag2 (append (cdr li) (list (car li)))           ;un cran et...
                   (1- n)))))                                  ;... continuer...

(defun rotad2 (li &optional (n 1)) (declare (notinline rotad2))
  (cond ((atom li) li)                                         ;cas atome ou ()
        ((zerop n) li)                                         ;cas terminal
        (t         (rotad2 (append (last li) (butlast li))     ;un cran et...
                           (1- n)))))                          ;... continuer...

;;   "Queue de vache" : procd indien de dveloppement d'une squence <seq>,
;; par rptitions avec liminations successives du premier lment, ou
;; resserrement graduel vers le dernier lment. Exemple :
;;    (qdv1 '(a b c d e)) => (A B C D E B C D E C D E D E E)
;; Pour ce procd, comme pour les trois suivants, un argument d'appel <seq> de
;; longueur <N> produit un rsultat de (+ 1 2 ... <N>) = (/ (* <N> (1+ <N>)) 2)
;; lments. Par exemple :
;;    <seq> de longueur 8 => squence rsultante de 36 lments
;; Dans tous les cas, il s'agit d'un nombre "triangulaire", <N>-ime de la srie
;;    0, 1, 3, 6, 10, 15, 21, 28, 36, 45, 55, ...
;$   "Kuhschwanz": indianischer Entwicklungprozess einer Sequenz <seq>, durch
;$ Wiederholungen mit nachfolgender Eliminierung des ersten Elements, bzw.
;$ gradueller Verengung auf das letzten Element. Beispiel:
;$    (qdv1 '(a b c d e)) => (A B C D E B C D E C D E D E E)
;$ Fr diesen Prozess, sowie die drei folgenden, ergibt ein <seq> Aufrufargument
;$ fr die Lnge <N> ein Ergebnis mit (+ 1 2 ... <N>) = (/ (* <N> (1+ <N>)) 2)
;$ Elementen. Zum Beispiel:
;$    <seq> der Lnge 8 =>  Ergebnissequenz von 36 Elemente
;$ Auf alle Flle erhlt man eine "dreieckige" Nummer, die <N>-te der Reihe
;$    0, 1, 3, 6, 10, 15, 21, 28, 36, 45, 55, ... 
;+   "Cow Tail": indian development process of a <seq> sequence, by repetitions
;+ with successive elimination of the first element, or gradual stricture
;+ towards the last element. Example:
;+    (qdv1 '(a b c d e)) => (A B C D E B C D E C D E D E E)
;+ In this process, as well as the three following, a <seq> calling argument of
;+ length <N> produces a result of (+ 1 2 ... <N>) elements. For instance:
;+    <seq> of length 8 => resulting sequence of 36 elements
;+ In all cases a "triangular" number is obtained, <N>-th of the series
;+    0, 1, 3, 6, 10, 15, 21, 28, 36, 45, 55, ...

(defun qdv1 (seq) (declare (notinline qdv1))
  (if (endp (cdr seq)) seq
  (append seq (qdv1 (cdr seq)))))

;;   "Estuaire" : procd indien de dveloppement d'une squence <seq>, oppos
;; de la "Queue de vache" ci-dessus. Il s'agit de rptitions  partir du
;; dernier lment, avec ajouts successifs de l'lment prcdent. On obtient un
;; largissement graduel, du dernier jusqu'au premier lment. Exemple :
;$   "Mndung": indianische Entwicklungprozess einer Sequenz <seq>, gegenlufig
;$ des obigen "Kuhschwanzes". Das heit Wiederholung des letzten Elements mit
;$ sukzessiven Hinzufgungen des vorigen Elements. Erzeugt eine graduelle
;$ Erweiterung vom letzten zum ersten Element. Beispiel:
;+   "Estuary": indian development process of a <seq> sequence, opposite of the
;+ above "Cow Tail". It consists in repetitions from the last element, with
;+ successive additions of the preceding element. Produces a gradual broadening,
;+ from the last element towards the first. For instance:
;-    (est1 '(1 2 3 4 5)) => (5 4 5 3 4 5 2 3 4 5 1 2 3 4 5)

(defun est1 (seq) (declare (notinline est1))
  (if (endp (cdr seq)) seq
  (append (est1 (cdr seq)) seq)))

;;   Variante de la prcdente "Queue de vache" classique, par liminations
;; successives du dernier lment, ou resserrement graduel vers le premier. Ex.
;$   Nebenform des vorigen klassischen "Kuhschwanzes", durch die sukzessive
;$ Elimination des letzten Elements, oder gradueller Verengung auf das erste
;$ Element. zB.:
;+   Variant of the preceding classical "Cow Tail", by successive eliminations
;+ of the last element, or gradual stricture towards the first. Ex.:
;-    (qdv2 '(a b c d e)) => (A B C D E A B C D A B C A B A)

(defun qdv2 (seq) (declare (notinline qdv2))
  (if (endp (cdr seq)) seq
  (append seq (qdv2 (butlast seq)))))

;;   Variante du prcdent "Estuaire" classique : rptitions  partir du
;; premier lment, avec largissement graduel jusqu'au dernier. Exemple :
;$   Nebenform der vorigen klassischen "Mndung": Wiederholungen vom ersten
;$ Element an, mit graduellen Erweiterung bis dem letzten. Beispiel:
;+   Variant of the preceding classical "Estuary": repetitions from the first
;+ element, with gradual broadening towards the last. Example:
;-    (est2 '(1 2 3 4 5)) => (1 1 2 1 2 3 1 2 3 4 1 2 3 4 5)

(defun est2 (seq) (declare (notinline est2))
  (if (endp (cdr seq)) seq
  (append (est2 (butlast seq)) seq)))

;;   Autre procd de dveloppement d'une squence d'lments, de genre
;; "volutionniste darwinien".
;;    croitre <li> <n> => croissance du nombre d'lments de <li> jusqu' <n>,
;;                        par "prolifration alatoire"
;; N.B. : utilisation de &aux pour dfinir des variables auxiliaires locales.
;$   Andere Entwicklungprozesse einer Elementsequenz, "evolutionr
;$ darwinistischer" Art.
;$    croitre <li> <n> => Steigerung der Elementanzahl der <li> bis <n> durch
;$                        "zufllige Ausbreitung"
;$ N.B.: Anwendung von &aux um lokale Hilfsvariablen zu definieren.
;+   Other development process for a sequence of elements, of type "darwinian
;+ evolutionary".
;+    croitre <li> <n> => growth of the number of elements of <li> until <n>,
;+                        through "random proliferation"
;+ N.B.: &aux used to define local auxiliairy variables.

(defun croitre1 (l max &aux lng ele pt) (declare (notinline croitre1))
  (setq lng (list-length l))                          ;longueur de la liste
  (if (>= lng max) l                                  ;clause terminale
  (progn
    (setq ele (nth (random lng) l))                   ;choix d'1 lment
    (setq pt (random (1+ lng)))                       ;choix d'1 pt. d'insertion
    (croitre1 (append (firstn2 l pt)                  ;firstn2 : cf. ped.08
                      (list ele)
                      (lastn2 l (- lng pt)))          ;lastn2 : cf. ped.08
              max))))                                 ;rcursion jusqu' <max>

;;   Exploitation de fonctions de contrle dfinies par interpolation linaire
;; entre points d'inflexion donns. Mise en oeuvre du procd classique de
;; break-point functions. Noter les dtections d'erreurs, toujours utiles pour
;; crer des fonctions robustes et universelles. Premire version, peu
;; efficace -- cf. version amliore au Chapitre 12.
;;    bkpt <fdt> <tp> => valeur de la fonction <fdt> pour l'abscisse <tp>.
;;                       Autrement dit, dans le cas classique, bkpt rpond 
;;                       l'ordre : "Donne-moi la valeur de <fdt> au temps <tp>",
;;    o <fdt> := liste de couples temps-valeur, sous forme de paires pointes :
;;                       ( (<t>1 . <v>1) (<t>2 . <v>2) ... (<t>n . <v>n) )
;$   Implementierung von Kontrollfunktionen durch lineare Interpolation
;$ zwischen gegebenen Haltepunkten. Klassische Anwendung. Man beachte die
;$ Fehlererkennung, immer ntzlich in robusten und universellen Funktionen.
;$ Erste Version, wenig effizient -- vgl. verbesserte Version im 12. Kapitel. 
;$    bkpt <fdt> <tp> => Wert der Funktion <fdt> an der Abszisse <tp>. Im
;$                       sozusagen klassischen Fall beantwortet die Funktion die
;$                       Anfrage: "Gib mir den Wert von <fdt> am Zeitpunkt
;$                       <tp>",
;$    wobei <fdt> := Liste von Zeit-Wert-Paaren in Form von dotted pairs:
;$                       ( (<t>1 . <v>1) (<t>2 . <v>2) ... (<t>n . <v>n) )
;+   Implementation of control functions defined by linear interpolation between
;+ given break-points. Classical application. Note the error detections, always
;+ usefull in a robust and universal function. First version, poorly efficient--
;+ cf. improved version in Chapter 12.
;+    bkpt <fdt> <tp> => value of the <fdt> function at abscissa <tp>. So to
;+                       speak, in the classical application, bkpt answers the
;+                       query: "Give-me the value of <fdt> at time <tp>",
;+    where <fdt> := list of time-value couples, formated as dotted pairs:
;+                       ( (<t>1 . <v>1) (<t>2 . <v>2) ... (<t>n . <v>n) )

(defun bkpt1 (fdt tp) (cond                          ;dtection prliminaire de
  ((null (cdr fdt))                                  ;trois erreurs possibles
    (error "bkpt1 : temps au-dela de la portee : ~A" tp))
  ((< tp (caar fdt))
    (error "bkpt1 : temps en-deca de la portee : ~A" tp))
  ((<= (caadr fdt) (caar fdt))
    (error "bkpt1 : temps non croissant : ~A" (caadr fdt)))
  ((> tp (caadr fdt)) (bkpt1 (cdr fdt) tp))          ;parcours rcursif de <fdt>
  (t (+ (cdar fdt)                                   ;calcul effectif :
        (* (- (cdadr fdt) (cdar fdt))                ;thorme de Thals
           (/ (- tp (caar fdt))
              (- (caadr fdt) (caar fdt))))))))

;;   Exemple de structure pour la ralisation d'un dessin fractal rcursif en
;; "C" (dragon de Lvy) : cf. [Winston & Horn:60sq.]. Exemple :
;$   Beispielstruktur fr die Realisierung einer fraktalen "C" Zeichnung (Lvy
;$ Drache): Vgl. [Winston & Horn:60sq.]. Beispiel:
;+   Example of structure realizing a fractal "C" drawing (Lvy Dragon): cf.
;+ [Winston & Horn:60sq.]. Example:
;-    (DesC 30.5 1.765)

(defparameter rac2    (sqrt 2.))                      ;dfinitions prliminaires
(defparameter pis4    (/ pi 4.))
(defparameter lon_min 10.0     )

(defun DesC (lon ang)                                 ;fonction rcursive
  (prins1 "lon = " lon " ang = " ang)                 ;vecteur  parcourir :
  (if (< lon lon_min) (tracer lon ang)                ;le tracer ou...
  (progn (DesC (/ lon rac2)                           ;... le diviser...
               (+ ang pis4))                          ;... en deux...
         (DesC (/ lon rac2)                           ;... segments...
               (- ang pis4)))))

(defun tracer (lon ang)                               ;fonction pseudo-graphique
  (prins1 "------>    ligne : " lon " ; ang : " ang ))
