;; <1> Definition des Wurzel-Paths
(setq common-lisp::*wurzel-path*
  (merge-pathnames
    (make-pathname :directory (pathname-directory *load-truename*)) "*"))
;; <2> Zustzliche Aufladung(en)
(dolist (lefic '(
                 "ped.11.3.lisp"
                 ))
  (load (setq lefic (merge-pathnames lefic common-lisp::*wurzel-path*)))
  (format t "~&;; Geladen ~S~%" lefic))
;===============================================================================
;;; 12 --> CRATION DYNAMIQUE DE VARIABLES ET FONCTIONS TEMPORAIRES :
;;;        primitives let et labels, etc.
;$$ 12 --> DYNAMISCH GENERIERUNG TEMPORRER VARIABLEN UND FUNKTIONEN:
;$$        Primitives let und labels, usw.                            [d..H.B.]
;++ 12 --> DYNAMIC CREATION OF TEMPORARY VARIABLES AND FUNCTIONS:
;++        let and labels Primitives, etc.
;===============================================================================

;;; -1- Utilisation de variables temporaires pour augmenter l'efficacit en
;;;     vitant des valuations redondantes
;$$ -1- Anwendung temporrer Variablen zur Effizienzsteigerung durch
;$$     Eliminierung redundanter Aufrufe
;++ -1- Use of temporary variables to increase efficiency by eliminating
;++     redundant evaluations

;;   Cf. [Wertz:44sq.]. Exemple de dpart : sans utilisation de la primitive
;; let -- manque d'efficacit. Cette fonction agit comme interface d'appel de la
;; fonction pgcd-bis pour la recherche du PGCD(<m>,<n>) : le plus grand commun
;; diviseur de deux nombres <m> et <n>. Deux nombres sont dits premiers entre
;; eux lorsque leur PGCD est 1.
;;    pgcd <m> <n> => PGCD(<m>,<n>) s'il est diffrent de 1, ou message si <m>
;;                    et <n> sont premiers entre eux
;$   Vgl. [Wertz:44sq.]. Anfangsbeispiel: ohne das Primitive let -- es mangelt
;$ an Effizienz. Diese Funktion fungiert als Aufrufschnittstelle zur pgcd-bis
;$ Funktion um den ggT(<m>,<n>) zu suchen: den grten gemeinsamen Teiler zweier
;$ Zahlen <m> und <n>. Zwei Zahlen sind teilerfremd wenn ihr ggT 1 ist.
;$    pgcd <m> <n> => ggT(<m>,<n>) falls nicht 1, oder Meldung wenn <m> und <n>
;$                    teilerfremd sind
;+   Cf. [Wertz:44sq.]. Beginning example: without the let primitive -- lacks
;+ efficiency. This function acts as a calling interface for the pgcd-bis
;+ function to search the GCD(<m>,<n>): the greatest common diviser of two
;+ numbers <m> and <n>. Two numbers are called coprime if their GCD equals 1.
;+    pgcd <m> <n> => GCD(<m>,<n>) if different from 1, or message if <m> and
;+                    <n> are coprime

(defun pgcd1 (m n)                                          ;pas efficace :
  (if (pgcd-bis m n) (pgcd-bis m n)                         ;sera calcul 2 fois
  (prins "-> " m " et " n " sont premiers entre eux")))

;;   Gain d'efficacit avec la primitive let, crant la variable temporaire <x>.
;$   Verbesserte Effizienz durch Erzeugen einer temporren Variable <x> mit dem
;$ let-Primitive.
;+   Increased efficiency whith the let primitive creating a temporary variable
;+ <x>.

(defun pgcd2 (m n)                                          ;efficace :
  (let ((x (pgcd-bis m n)))                                 ;sera calcul 1 fois
    (if x x
    (prins "-> " m " et " n " sont premiers entre eux"))))

;;   Fonction de calcul effectif, appliquant l'algorithme d'Euclide -- procdure
;; de recherche du PGCD(<m>,<n>), cf. [Wertz:45]. Cette fonction rend le plus
;; grand commun diviseur, ou nil lorqu'il est 1. Elle serait utilisable
;; directement si l'on ne souhaitait pas imprimer de message spcifique dans le
;; cas d'arguments <m> et <n> premiers entre eux. 
;;   Les deux fonctions pgcd ci-dessus constituent, en fait, des interfaces
;; d'habillage de pgcd-bis.
;$   Effektive Berechnungsfunktion, die Euklid's Algorithmus anwendet --
;$ Suchfunktion fr den ggT(<m>,<n>), vgl. [Wertz:45]. Diese Funktion gibt den
;$ grten gemeinsamen Teiler zurck, oder nil wenn er 1 ist. Sie wre ohne
;$ weiteres benutzbar, wenn man keine Meldung fr den Fall zweier teilerfremder
;$ Argumente wnschte.
;$   Die obigen zwei pgcd-Funktionen sind tatschlich Benutzerschnittstellen der
;$ Funktion pgcd-bis.
;+   Effective computing function, applying Euclide's algorithm -- research
;+ procedure for the GCD(<m>,<n>), cf. [Wertz:45]. This function gives the
;+ greatest common diviser, or nil if it is 1. It could be used directly if one
;+ did not wish to print a message in the case of coprime calling arguments.
;+   The two above pgcd functions are in fact dress up interfaces of pgcd-bis.

(defun pgcd-bis (m n)
  (cond ((> m n) (pgcd-bis n m))                         ;ordre des paramtres
        ((= (rem n m) 0) (if (= m 1) nil m))             ;fin : rend nil ou PGCD
        (t (pgcd-bis (rem n m) m))))                     ;rcursion...

;;   Cependant... une programmation sans interception de 1 en tant que rsultat
;; particulier,  traiter spcifiquement, rendrait l'exemple de Wertz superflu :
;;    pgcd3 <m> <n> => PGCD(<m>,<n>), mme si ce n'est "que" 1
;; On retrouve ici la primitive Common LISP gcd [CLtL2:298], mais cette dernire
;; est limite  des arguments entiers, ce qui n'est pas le cas de pgcd3.
;$   Allerdings... eine Berechnung ohne die 1 als einen Spezialfall abzufangen,
;$ der eine Extrabehandlung bentigt, wrde Wertzs Beispiel berflssig machen:
;$    pgcd3 <m> <n> => ggT(<m>,<n>) auch wenn er "nur" 1 ist
;$ Hier findet man das Common LISP-Primitive gcd [CLtL2:298], aber letzteres ist
;$ auf Ganzzahlen begrenzt, whrend das bei pgcd3 nicht der Fall ist.
;+   However... programming without intercepting 1 as a particular case
;+ requiring spcific handling, would make Wertz's example superfluous:
;+    pgcd3 <m> <n> => GCD(<m>,<n>), even if it is "only" 1
;+ We find here the Common LISP gcd primitive [CLtL2:298], but the latter is
;+ limited to integer arguments, which is not the case for pgcd3.

(defun pgcd3 (m n)
  (cond ((> m n) (pgcd3 n m))                            ;ordre des paramtres
        ((= (rem n m) 0) m)                              ;cas final
        (t (pgcd3 (rem n m) m))))                        ;rcursion...

;;   Redfinition d'un exemple du Chapitre 11.3 : utilisation de let pour crer
;; des variables temporaires locales. La version prcdente recourait  des
;; variables auxiliaires de type &aux cres dans la lambda-liste, et gres
;; avec des setq dans la suite de l'algorithme (cf. croitre1, ped.11.3).
;;    croitre <li> <n> => croissance du nombre d'lments de <li> jusqu' <n>,
;;                        par "prolifration alatoire"
;$   Neudefinition eines Beispiels des Kapitels 11.3: Anwendung von let um
;$ temporre lokale Variablen zu erzeugen. Die vorige Version verwendete
;$ Hilfsvariablen des &aux-Typs, die in der Lambda-Liste erstellt und spter mit
;$ setq im Algorithmus verwaltet werden (vgl. croitre1, ped.11.3).  
;$    croitre <li> <n> => Vergrerung der Elementenanzahl von <li> bis <n>
;$                        durch "zufllige Ausbreitung"
;+   Redefinition of an example of Chapter 11.3: use of let to create temporary
;+ local variables. The preceding version used auxiliairy variables of type &aux
;+ created in the lambda-list, subsequently managed with setq in the algorithm
;+ (cf. croitre1, ped.11.3).
;+    croitre <li> <n> => growth of the number of elements of <li> until <n>,
;+                        through "random proliferation"

(defun croitre2 (li max)
  (let ((lng (list-length li)))                      ;lng = longeur de <li>
    (if (>= lng max) li                              ;cas final
    (let ((ele (nth (random lng) li))                ;<ele> = choix d'1 lment
          (pt (random (1+ lng))))                    ;<pt> = chx pt. d'insertion
      (croitre2                                      ;rcursion...
       (append (firstn2 li pt)                       ;firstn2 cf. ped.08
               (list ele)
               (lastn2 li (- lng pt)))               ;lastn2 cf. ped.08
       max)))))               ;<-  amliorer : transmission redondante de <max>
                              ;cf. croitre3, ci-dessous

;;; -2- Utilisation de fonctions et/ou de variables temporaires pour les besoins
;;;     internes d'algorithmes
;$$ -2- Anwendung temporrer Funktionen und/oder Variablen fr den internen
;$$     Bedarf der Algorithmen
;++ -2- Use of temporary functions and/or variables for the internal needs of
;++     algorithms

;;   Interface d'habillage de retro1 (dfinie d'aprs [Wertz:28], cf. ped.09 et
;; ped.11.2b) : la fonction effective retro-bis, et sa variable auxiliaire
;; <rez>, sont temporaires, internes, invisibles pour l'utilisateur.
;;    retro <li> => rtrograde des lments de premier niveau de la liste <li>.
;;                  quivalente LISP : reverse
;; N.B. : la variable <rez> est en ralit inutile, comme le dmontre la version
;; retro2 (cf. ped.11.2b).
;$   Benutzerschnittstelle fr retro1 (definiert nach [Wertz:28], Vgl. ped.09
;$ und ped.11.2b): die effiziente Funktion retro-bis und ihre Hilfsvariable
;$ <rez> sind temporr, intern, unsichtbar fr den Benutzer.
;$    retro <li> => Umkehrung der Elemente der ersten Ebene der Liste <li>.
;$                  LISP-Equivalent: reverse
;$ N.B.: die <rez> Variable ist wirklich nutzlos, wie die Funktion retro2 zeigt
;$ (Vgl. ped.11.2b).
;+   Dress up interface of retro1 (defined from [Wertz:28], cf. ped.09 and
;+ ped.11.2b): the retro-bis effective function, and its auxiliairy variable
;+ <rez>, are temporary, internal, invisible for the user.
;+    retro <li> => retrograde of the first level elements of list <li>. LISP
;+                  equivalent: reverse
;+ N.B.: the <rez> variable is in  reality useless, as demonstrated by version
;+ retro2 (cf. ped.11.2b).

(defun retro5 (li)
  (labels ((retro-bis (li rez)                          ;dfinition de retro-bis
             (if (atom li) rez                          ;cas terminal
             (retro-bis (cdr li) (cons (car li) rez)))));rcursion...
    (retro-bis li ())))                                 ;corps du labels :
                                                        ;premier appel

;;   Interface d'habillage : la fonction effective auXil, et sa variable
;; auxiliaire <res>, sont temporaires, internes, invisibles pour l'utilisateur.
;;    bin>dec <li> => conversion dcimale d'un nombre binaire reprsent par une
;;                    squence de chiffres binaires (1 et 0) dans la liste <li>.
;;                    Cf. [Wertz:50]
;$   Benutzerschnittstelle: die effiziente auXil-Funktion, und ihre
;$ Hilfsvariable <res> sind temporr, intern, unsichtbar fr den Benutzer.
;$    bin>dec <li> => dezimale Konvertierung einer binren Zahl dargestellt 
;$                    durch eine Sequenz binrer Ziffern (1 und 0) in der Liste
;$                    <li>. Vgl. [Wertz:50]
;+   Dress up interface: the auXil effective function, and its auxiliairy
;+ variable <res>, are temporary, internal, invisible for the user.
;+    bin>dec <li> => decimal conversion of a binary number represented by a
;+                    sequence of binary digits (1 and 0) in list <li>. Cf.
;+                    [Wertz:50]
;- Exemple : / Beispiel: / Example:
;-    (bin>dec '(1 1 0 1 0 1)) => 53

(defun bin>dec (li)
  (labels ((auXil (li res)                           ;dfinition de auXil
             (if li
               (auXil (cdr li) (+  (* res 2)         ;calcul et rcursion...
                                   (car li)))
             res)))                                  ;cas terminal
    (auXil li 0)))                                   ;corps du labels :
                                                     ;premier appel

;;   Redfinition plus concise d'un exemple du Chapitre 11.3 (cf. ped.11.3).
;; La primitive labels permet d'viter de recourir  defun pour dfinir une
;; seconde fonction effective.
;;   Utilitaire d'impression d'une squence quelconque de formes en une seule
;; ligne de texte.
;;    prins2 {<forme(s)>}* => impression squentielle de(s) <forme(s)>
;$   Knpper wiederdefinition eines Beispiels des 11.3 Kapitels (cf. ped.11.3).
;$ Die labels Stammfunktion nicht defun zu benutzen ermglicht, um eine zweite
;$ effektive Funktion zu definieren. 
;$   Nutzfunktion um eine Formsequenz in einer einzigen Zeile zu drucken.
;$    prins2 {<forme(s)>}* => sequentiellen Ausdruck der <forme(s)>
;+   Terser redefinition of an example from Chapter 11.3 (cf. ped.11.3). The
;+ labels primitive enables to avoid using defun to define a second effective
;+ function.
;+   Utility for the printout of any sequence of forms on a single line of text.
;-  Exemple d'utilisation : / Anwendungsbeispiel: / Example usage:
;-    (prins2 "a = " a " ; b = " b ".")

(defun prins2 (&rest frms)  ;fonction interface : limination du cas trivial, et
  (if (endp frms) nil       ;emballage d'un nbr inconnu de forme(s) en une liste
  (labels ((prinsX (li)                              ;fonction effective
             (cond ((endp li) (terpri) t)            ;cas terminal
                   (t    (princ (car li))            ;imprimer la premire...
                         (prinsX (cdr li))))))       ;... et continuer...
    (prinsX frms))))                                 ;corps du labels :
                                                     ;premier appel

;;; -3- Utilisation de fonctions temporaires pour augmenter l'efficacit en
;;;     vitant la transmission redondante de constantes lors de l'application
;;;     d'un processus rcursif
;$$ -3- Anwendung temporrer Funktionen zur Effizienzsteigerung durch Vermeidung
;$$     der redundanten bertragung von Konstanten in rekursiven Prozessen
;++ -3- Use of temporary functions to increase efficiency by avoiding the
;++     redundant transmission of constants in recursive processes

;;   Amlioration d'un exemple du Chapitre 11.2a (cf. ped.11.2a) : limination
;; de la transmission de la constante <it> lors des rcursions.
;;    membre <it> <li> => rend le cdr de <li> commenant par <it> s'il existe,
;;                        ou nil autrement. quivalente LISP : member, avec
;;                        :test #'equal
;$   Verbessertes Beispiel aus Kapitel 11.2a (Vgl. ped.11.2a): Eliminierung der
;$ bertragung der Konstante <it> in den Rekursionen.
;$    membre <it> <li> => gibt das cdr von <li> beginnend mit <it> zurck, falls
;$                        es existiert, sonst nil. LISP-Equivalent: member mit
;$                        :test #'equal
;+   Improved example from Chapter 11.2a (cd. ped.11.2a): suppression of the
;+ transmission of the <it> constant throughout recursions.
;+    membre <it> <li> => returns the cdr of <li> beginning by <it>, if it
;+                        exists, or nil otherwise. LISP equivalent: member,
;+                        with :test #'equal

(defun membre3 (it li)                               ;<it> est constant
  (labels ((membr-aux (li)                           ;membr-aux : 1 argument
             ;(prins "membr-aux : li= " li)          ;pour tracer au besoin
             (if (or (atom li) (equal it (car li)))  ;cas terminal
               li
             (membr-aux (cdr li)))))                 ;continuer par rcursion...
    (membr-aux li)))                                 ;corps du labels :
                                                     ;premier appel

;; Utile pour Csound
;$ Csound nutzbar
;+ Useful for Csound

;;   Cette fonction construit une liste de <nbr> lments, chacun tant le
;; rsultat d'une valuation de l'argument <fonction> :
;;    faire-liste <fonction> &optional (<nbr> 10)
;; Elle est analogue  la primitive make-list, mais tous les lments de la
;; liste rsultante peuvent tre ici dtermins individuellement par des
;; valuations aussi complexes que souhaites.
;;   Utile pour prparer une liste de valeurs pour un champ p<n> d'une squence
;; de notes Csound (cf. fonctions faire-notes et iCs, ped.14).
;$   Diese Funktion erstellt eine Liste mit <nbr> Elementen, wobei jedes das
;$ Ergebnis der Evaluation des <fonction> Arguments ist:
;$    faire-liste <fonction> &optional (<nbr> 10)
;$ Sie ist zum Primitive make-list analog, jedoch knnen hier alle
;$ Listenelemente durch individuelle, beliebig komplexe Evaluationen bestimmt
;$ werden.
;$   Ntzlich um eine Liste von Werten fr ein p<n> Datenfeld einer Csound
;$ Notensequenz vorzubereiten (Vgl. die Funktionen faire-notes und iCs, ped.14).
;+   This function creates a list of <nbr> elements, each resulting from an
;+ evaluation of the <fonction> argument:
;+    faire-liste <fonction> &optional (<nbr> 10)
;+ It is analog to the make-list primitive, but all elements of the list can
;+ here result from individual evaluations, as complex as needed.
;+   Useful to prepare a list of values for a p<n> field of a sequence of Csound
;+ notes (cf. faire-notes and iCs functions, ped.14).
;-   Exemples : / Beispiele: / Examples:
;-    (faire-liste 3.3) => (3.3 3.3 3.3 3.3 3.3 3.3 3.3 3.3 3.3 3.3)
;-    (faire-liste '(random 10) 22)
;-                      => (7 6 0 6 4 0 5 6 4 8 8 5 9 2 1 6 2 0 0 8 0 2)

(defun faire-liste (fonction &optional (nbr 10))
  (labels ((interne () (cond
            ((= 0 nbr) ())
            (t (decf nbr)
               (cons (eval fonction) (interne))))))
    (interne)))

;; Utile pour Csound
;$ Csound nutzbar
;+ Useful for Csound

;;   Comme faire-liste ci-dessus, mais la condition de fin de la liste est le
;; dpassement de <max> par le <cumul> des valeurs insres dans la liste, et
;; non leur nombre :
;;    faire-durees <fonction> &optional (<max> 10)
;; Cette fonction est donc propice  la prparation de listes d'intervalles de
;; temps se succdant jusqu' ce que soit atteinte -- ou dpasse -- une dure
;; totale <max>.
;;   Utile pour prparer une liste de valeurs pour le champ p3 (dure) d'une
;; squence de notes Csound (cf. fonctions faire-notes et iCs, ped.14).
;$   Wie faire-liste oben, die Endebedingung der Liste aber ist das
;$ berschreiten von <max> durch die <cumul> Aufsummierung der eingefgten Werte
;$ in der Liste, nicht deren Anzahl:
;$    faire-durees <fonction> &optional (<max> 10)
;$ Daher ist diese Funktion gnstig zur Vorbereitung von Zeitabstandslisten, in
;$ denen die Werte aufeinanderfolgen bis -- oder jenseits -- einer Totaldauer
;$ <max>.
;$   Ntzlich zur Vorbereitung einer Liste von Werten fr das p3-Datenfeld
;$ (Dauer) einer Csound-Notensequenz  (Vgl. die Funktionen faire-notes und iCs,
;$ ped.14).
;+   Similar to faire-liste above, but the list's end condition is the
;+ surpassing of <max> by the <cumul> accumulation of the inserted value, not
;+ their number.
;+    faire-durees <fonction> &optional (<max> 10)
;+ This function is thus favorable for the preparation of lists of successive
;+ time intervals until a <max> total duration is reached--or surpassed.
;+   Useful for the preparation of a list of values for the p3 field (duration)
;+ of a sequence of Csound notes (cf. faire-notes and iCs functions, ped.14).
;-   Exemple : / Beispiel: / Example:
;-    (faire-durees '(1+ (random 10)) 100)
;-                      => (8 5 5 6 1 7 4 1 6 2 7 7 8 6 4 10 5 5 8)

(defun faire-durees (fonction &optional (max 10))
  (let ((cumul 0.0) temp)
    (labels ((interne () (cond
               ((>= cumul max) ())
               (t (incf cumul (setq temp (eval fonction)))
                  (cons temp (interne))))))
      (interne))))

;; Calcul diffrentiel et intgral
;$ Differenzial- und Integralrechnung
;+ Differential and Integral Calculus

;;   "Dans son <De arte combinatoria> (1666), [Leibnitz tudie] la suite des
;; carrs [qui] lui permet d'laborer une mthode formelle pour calculer les
;; sommes et les diffrences des infinitsimaux" [DAHAN-DALMEDICO, Amy, &
;; PEIFFER, Jeanne, 1986 : Une histoire des mathmatiques, Seuil, Collection
;; Points sciences : 195].
;;   Soit x = (SC <n>), la suite des nombres 0  <n> levs au carr. Exemple :
;$   "In seinem <De arte combinatoria> (1666), [untersucht Leibnitz] die Folge
;$ der Quadratzahlen, [die] es ihm erlauben, eine formale Methode zur Berechnung
;$ der Summen und Differenzen von Infinitesimalzahlen zu entwickeln"
;$ [DAHAN-DALMEDICO, Amy, & PEIFFER, Jeanne, 1986: Une histoire des
;$ mathmatiques, Seuil, Collection Points sciences: 195].
;$   Sei x = (SC <n>), die Zahlenfolge von 0 bis <n> im Quadrat. Beispiel:
;+   "In his <De arte combinatoria> (1666), [Leibnitz studies] the series of
;+ squares [which] allows him to elaborate a formal method to compute the sums
;+ and differences of infinitesimals" [DAHAN-DALMEDICO, Amy, & PEIFFER, Jeanne,
;+ 1986: Une histoire des mathmatiques, Seuil, Collection Points sciences:
;+ 195].
;+   Let x = (SC <n>), the series of numbers 0 to <n> squared. Example:
;-    (setq x (SC 10)) => (0 1 4 9 16 25 36 49 64 81 100)
;; Soit dx = (deriv (SC <n>)), la liste des diffrences entres les termes
;; successifs de (SC <n>). Exemple :
;$ Sei dx = (deriv (SC <n>)), die Folge der Differenzen zwischen den
;$ aufeinanderfolgenden Gliedern von (SC <n>). Beispiel:
;+ Let dx = (deriv (SC <n>)), the list of the differences between the successive
;+ terms of (SC <n>). Example:
;-    (setq dx (deriv (SC 10))) => (1 3 5 7 9 11 13 15 17 19)
;; La fonction deriv est dfinie dans le fichier ped.11.2a. Il s'agit de la
;; drive de la fonction (SC <n>).
;; On remarque les proprits suivantes :
;;    - dx est la suite des entiers impairs 1, 3, 5, 7...
;;    - la somme des termes de dx est gale  <n>**2, qui est donc le dernier
;;      terme de la liste x :
;$ Die Funktion deriv ist in der Datei ped.11.2a definiert. Es ist eine
;$ Abwandlung der (SC <n>)-Funktion.
;$ Man erkennt folgende Eigenschaften:
;$    - dx ist die Folge ungerader Ganzzahlen: 1, 3, 5, 7...
;$    - die Summe der Glieder von dx entspricht <n>**2, was das letzte Glied
;$      der Folge x ist:
;+ The deriv function is defined in file ped.11.2a. It is the derivative of the
;+ (SC <n>) function.
;+ One notes the following properties:
;+    - dx is the series of odd integers 1, 3, 5, 7...
;+    - the sum of the terms of dx is equal to <n>**2, which is the last term of
;+      the x list
;-        (eval (cons '+ dx)) := (apply #'+ dx) => 100 = 10**2 = (car (last x))
;;      Il s'agit de l'intgrale de la drive de la fonction (SC <n>), qui est
;;      gale  la fonction (SC <n>).
;;    - la drive de dx est constante  2 :
;$      Es handelt sich um das Integral der Ableitung der (SC <n>)-Funktion, 
;$      welche der (SC <n>)-Funktion entspricht.
;$    - die Ableitung von dx ist konstant 2:
;+      It is the integral of the derivative of the (SC <n>) function, which is
;+      equal to the (SC <n>) function.
;+    - the derivative of dx is constantly 2:
;-        (setq ddx1 (deriv dx)) => (2 2 2 2 2 2 2 2 2)
;-        (setq ddx2 (deriv (deriv (SC 10)))) => (2 2 2 2 2 2 2 2 2)
;;      Il s'agit de la drive seconde de la fonction (SC <n>).
;$      Es handelt sich um die zweite Ableitung der (SC <n>)-Funktion.
;+      It is the second derivative of the (SC <n>) function.

(defun SC (n)
  (labels ((interne (m)
             (if (= m n) (list (* m m))
             (cons (* m m) (interne (1+ m))))))
    (interne 0)))

;; Utile pour Csound
;$ Ntzlich in Csound
;+ Useful for Csound

;;   Cumul des valeurs d'une liste ("intgrale" discrte, pas  pas). Utile, par
;; exemple, pour convertir une liste d'intervalles de temps en une liste de
;; dates de dpart pour une succession de notes Csound.
;;   La fonction faire-notes (ped.14) tant limite  l'emploi de l'abrviation
;; "+" pour la ralisation de squences, la dfinition ci-dessous servirait dans
;; des cas plus complexes ou gnraux o la liste des temps de dpart devrait
;; tre gre explicitement : rpartition des notes successives sur diffrents
;; instruments, tuilages, etc.
;;    integ <liste> => liste de <n>+1 valeurs, commencant  0.0, spares par
;;                     les intervalles successifs de <liste>,
;;    o <liste> := liste de <n> intervalles
;;   Exemple d'utilisation simple pour obtenir une liste de <n>+1 dates
;; d'attaques  partir d'une liste de <n> intervalles de temps successifs :
;$   Aufsummierung der Werte einer Liste (diskretes schrittweises "Integral"). 
;$ Zum Beispiel ntzlich um eine Zeitabstandsliste in eine Zeitpunktliste fr
;$ die Startzeiten aufeinanderfolgender Csound-Noten zu konvertieren.
;$   Whrend die faire-notes Funktion (ped.14) auf die Anwendung der
;$ "+"-Abkrzung beschrnkt ist um Sequenzen zu realisieren, kann die folgende
;$ Definition auch in anderen, komplexeren oder allgemeineren, Fllen angewendet
;$ werden, in denen die Liste der Startzeiten explizit gehandhabt werden soll:
;$ Verteilung aufeinanderfolgender Noten ber verschiedene Instrumente,
;$ berlappung, etc.
;$    integ <liste> => Liste von <n>+1 Werten, beginnend mit 0.0, unterteilt
;$                     durch <n> aufeinanderfolgende Abstnde der <liste>
;$    wobei <liste> := Liste von <n> Abstnden
;$   Einfaches Benutzungsbeispiel um eine Liste von <n>+1 Startzeitpunkten nach
;$ einer Liste von <n> aufeinanerfolgenden Abstnden zu erhalten:
;+   Accumulation of the values of a list (discrete stepwise "integral").
;+ Useful, for instance, to convert a list of time intervals into a list of
;+ start times for a sequence of Csound notes.
;+   The faire-notes function (ped.14) being limited to the use of the "+"
;+ abbreviation to realize sequences, this definition could be used in more
;+ complex or general cases where the list of start times should be explicitly
;+ managed: distribution of successive notes over different instruments, tiling,
;+ etc.
;+    integ <liste> => list of <n>+1 values, starting from 0.0, separated by
;+                     the successive intervals of <liste>,
;+    where <liste> := list of <n> intervals
;+   Example of simple use to obtain a list of <n>+1 start times from a list of
;+ <n> successive time intervals:
;-    ? (setq intervs '(0.5 1 2 1.0 10 1.5 2.3))
;-    (0.5 1 2 1.0 10 1.5 2.3)
;-    ? (integ intervs)
;-    (0 0.5 1.5 3.5 4.5 14.5 16.0 18.3)
;;   Les intervalles peuvent, par exemple, provenir d'un appel  faire-durees
;; ou faire-liste (cf. ci-dessus) :
;$   Die Abstnde knnen z. B. aus einem Aufruf von faire-durees oder
;$ faire-liste (siehe oben) kommen:
;+   The intervals could, for instance, come from a call to faire-durees or
;+ faire-liste (cf. above):
;-    ? (integ (faire-durees '(random 2.3) 30.0))
;-    (0 1.1376717 2.9503121 4.7485895 6.5437164 8.2698345 10.039417
;-    10.728195 11.689748 13.279189 14.601515 16.386076 17.844444 18.169851
;-    18.491207 19.729498 21.490362 23.677176 25.702206 27.483305 27.922861
;-    29.45892 30.250208)
;;   Le mme procd peut tre appliqu  partir d'une valeur de dpart donne,
;; autre que 0.0. Par exemple pour raliser mlodiquement une squence
;; d'intervalles successifs  partir d'une note de dpart donne :
;;    integ <liste> <cumul> => liste de <n>+1 valeurs, commencant  <cumul>,
;;                             spares par les <n> intervalles successifs de
;;                             <liste>
;;   Exemple d'utilisation mlodique en MIDI pitches :
;$   Derselbe Prozess kann von einem gegebenen Wert ungleich 0.0 durchgefhrt
;$ werden. Zum Beispiel um eine melodische Sequenz aufeinanderfolgender
;$ Intervalle aufgrund einer gegebenen Anfangsnote zu erhalten:
;$    integ <liste> <cumul> => Liste von <n>+1 Werten, beginnend mit <cumul> und
;$                             unterteilt durch <n> aufeinanderfolgende
;$                             Intervalle der <liste>
;$   Beispiel einer melodischen Anwendung in MIDI-Pitches:
;+   The same procedure can be applied from a given starting value other than
;+ 0.0. For instance in order to realize a melodic sequence from a sequence of
;+ successive intervals and a given starting note:
;+    integ <liste> <cumul> => list of <n>+1 values starting from <cumul> and
;+                             separated by the <n> successive intervals of
;+                             <liste>
;+   Example of melodic application in MIDI pitches:
;-    (integ '(4 3 5 -13 2 -1) 60) => (60 64 67 72 59 61 60)

(defun integ (liste &optional (cumul 0))
  (labels ((interne (li)
             (if (endp li) ()
             (cons (incf cumul (car li)) (interne (cdr li))))))
    (cons cumul (interne liste))))

;;; -4- Mlanges des avantages prcdents de let et labels
;$$ -4- Mischung der vorangegangenen Vorzge von let und labels
;++ -4- Mixture of preceding advantages of let and labels

;;   Translater une liste de nombres dans [0,1[ en une liste de nombres dans
;; [<min>,<max>[. Utile pour mettre  l'chelle dsire une liste de nombres
;; "abstraits" dtermins entre 0.0 et 1.0 -- ou -1.0 et 1.0 : dans ce cas,
;; <min> correspondra  0.0, et <max>  la valeur absolue maximale.
;$   bertragen einer Liste von Zahlen in [0,1[ in eine Liste von Zahlen in
;$  [<min>,<max>[. Ntzlich zur Konvertierung einer Liste "abstrakter" Zahlen
;$ zwischen 0.0 und 1.0 -- oder zwischen -1.0 und 1.0: in diesem Fall wrde
;$ <min> 0.0 und <max> dem absoluten Maximalwert entsprechen -- in eine
;$ bestimmte Skala.
;+   To translate a list of numbers in [0,1[ into a list of numbers in
;+ [<min>,<max>[. Useful to convert to a desired scale a list of "abstract"
;+ numbers computed between 0.0 and 1.0--or -1.0 and 1.0: in this case <min>
;+ would correspond to 0.0 and <max> to the maximum absolute value.

(defun translat (list min max)
  (let ((ambit (- max min)))
    (labels ((interne (li)
               (if (endp li) ()
               (cons (+ min (* (car li) ambit)) (interne (cdr li))))))
      (interne list))))

;;   Augmentation de l'efficacit par une fonction et des variables
;; temporaires : cf. croitre1 (ped.11.3) et croitre2 ci-dessus.
;;    croitre <li> <n> => croissance du nombre d'lments de <li> jusqu' <n>,
;;                        par "prolifration alatoire"
;$   Effizienzsteigerung durch temporre Funktionen und Variablen: vgl. croitre1
;$ (ped.11.3) und croitre2 (s. o.).
;$    croitre <li> <n> => Steigerung der Elementenanzahl von <li> bis <n>, durch
;$                        "zufllige Ausbreitung"
;+   Efficiency gain through temporay function and variables: cf. croitre1
;+ (ped.11.3) and croitre2 above.
;+    croitre <li> <n> => growth of the number of elements of <li> until <n>,
;+                        through "random proliferation"

(defun croitre3 (li max)                             ;<max> est constant
  (labels ((crAux (li)                               ;crAux : 1 argument
             (let ((lng (list-length li)))           ;<lng> = longeur de <li>
               (if (>= lng max) li                   ;cas final
               (let ((ele (nth (random lng) li))     ;<ele> = choix d'1 lment
                     (pt (random (1+ lng))))         ;<pt> = chx pt. d'insertion
                 (crAux (append                      ;rcursion : <max>, tant
                         (firstn2 li pt)             ;constant, n'a pas  tre
                         (list ele)                  ;retransmis...
                         (lastn2 li (- lng pt)))))))))
    (crAux li)))                                     ;corps du labels :
                                                     ;premier appel

;;   Augmentation de l'efficacit par une fonction et des variables
;; temporaires : cf. bkpt1 (ped.11.3).
;;    bkpt <fdt> <tp> => valeur de la fonction <fdt> pour l'abscisse <tp>.
;;                       Autrement dit, dans le cas classique, bkpt rpond 
;;                       l'ordre : "Donne-moi la valeur de <fdt> au temps <tp>",
;;    o <fdt> := liste de couples temps-valeur, sous forme de paires pointes :
;;                       ( (<t>1 . <v>1) (<t>2 . <v>2) ... (<t>n . <v>n) )
;$   Effizienzsteigerung durch temporre Funktionen und Variablen: Vgl. bkpt1
;$ (ped.11.3).
;$    bkpt <fdt> <tp> => Wert der Funktion <fdt> an der Abszisse <tp>. Anders
;$                       gesagt beantwortet die Funktion bei der klassischen
;$                       Anwendung diese Abfrage:
;$                       "Gib mir den Wert von <fdt> am Zeitpunkt <tp>",
;$    wo <fdt> := Liste von Zeit-Wert-Paaren in Form von dotted pairs:
;$                       ( (<t>1 . <v>1) (<t>2 . <v>2) ... (<t>n . <v>n) )
;+   Efficiency gain through temporay function and variables: cf. bkpt1
;+ (ped.11.3).
;+    bkpt <fdt> <tp> => value of the <fdt> function at abscissa <tp>. So to
;+                       speak, in the classical application, bkpt answers the
;+                       query: "Give-me the value of <fdt> at time <tp>",
;+    where <fdt> := list of time-value couples, formated as dotted pairs:
;+                       ( (<t>1 . <v>1) (<t>2 . <v>2) ... (<t>n . <v>n) )

(defun bkpt2 (fdt tp)                                ;<tp> est constant
  (labels ((bkptAUX (fdt)                            ;bkptAUX : 1 argument
             (if (null (cdr fdt))
               (error "bkpt2 : temps au-dela de la portee ~A" tp)
             (let ((t1 (caar fdt))                   ;variables locales, 
                   (t2 (caadr fdt)))                 ;calculer une seule fois
               (cond ((< tp t1)
                       (error "bkpt2 : temps en-deca de la portee ~A" tp))
                     ((<= t2 t1)
                       (error "bkpt2 : temps non croissant ~A" t2))
                     ((> tp t2) (bkptAUX (cdr fdt)))
                     (t (let ((v1 (cdar fdt))        ;variables locales, 
                              (v2 (cdadr fdt)))      ;calculer une seule fois
                          (+ v1 (* (- v2 v1) (/ (- tp t1)
                                                (- t2 t1)))))))))))
            (bkptAUX fdt)))                          ;premier appel

;;   Combinaison libre de techniques diffrentes : utilisation de variables &aux
;; dans la lambda-liste, ainsi que de labels et let pour crer une fonction et
;; des variables temporaires.
;;    dodeca => une srie dodcaphonique... plus ou moins bien constitue !
;;   L'algorithme ci-dessous manque quelque peu d'lgance, car il comporte deux
;; clauses quasi redondantes --  la seule diffrence de <rezA> et <rezB>.
;$   Freie Kombination verschiedener Techniken: Anwendung von &aux-Variablen in 
;$ der Lambda-Liste, sowie labels und let um eine temporre Funktion und
;$ ebensolche Variablen zu erhalten.
;$    dodeca => eine Zwlftonreihe... mehr oder weniger gut erstellt!
;$   Dem folgenden Algorithmus fehlt ein wenig Eleganz, da er zwei
;$ quasi-redundante Klauseln enthlt -- mit dem einzigen Unterschied von <rezA>
;$ und <rezB>.
;+   Free mixture of different techniques: use of &aux variables in the lambda-
;+ list, as well as labels and let to create temporary function and variables.
;+    dodeca => twelve tone row... more or less well behaved!
;+   The following algorithm lacks some elegance, because it includes two quasi-
;+ redundant clauses--with the sole difference of <rezA> and <rezB>.
 
(defun dodeca (&aux (seri ())                        ;variables auxiliaires
                    (rezA '(C E G D A B))            ;utiles  l'algorithme
                    (rezB '(C+ D+ F F+ G+ A+)))
  (labels ((dodAUX (basc)                            ;dodAUX : 1 argument
             (cond ((null (if basc rezA rezB)) seri)
                   (basc                             ;choix interne  la clause
                     (let ((chx (nth (random (list-length rezA)) rezA)))
                       (setq seri (cons chx seri))
                       (setq rezA (remove chx rezA))
                       (dodAUX (not basc))))
                   (t                                ;choix interne  la clause
                     (let ((chx (nth (random (list-length rezB)) rezB)))
                       (setq seri (cons chx seri))
                       (setq rezB (remove chx rezB))
                       (dodAUX (not basc)))))))
    (dodAUX (zerop (random 2)))))                    ;premier : appel pile/face
