;; <1> Definition des Wurzel-Paths
(setq common-lisp::*wurzel-path*
  (merge-pathnames
    (make-pathname :directory (pathname-directory *load-truename*)) "*"))
;; <2> Zustzliche Aufladung(en)
(dolist (lefic '(
                 "ped.08.lisp"
                 "ped.11.2a.lisp"
                 "ped.12.lisp"
                 ))
  (load (setq lefic (merge-pathnames lefic common-lisp::*wurzel-path*)))
  (format t "~&;; Geladen ~S~%" lefic))
;===============================================================================
;;; 13 --> ITRATIONS EXPLICITES ET QUELQUES STRUCTURES DE CONTRLE :
;;;        primitives mapcar, apply, prog1, prog2, progn, do, dotimes, dolist,
;;;        prog, catch et throw  --  lambda-expressions  --  notion de values
;$$ 13 --> AUSDRCKLICHE WIEDERHOLUNGEN UND EINIGE KONTROLLSTRUKTUREN:
;$$        Die Primitives mapcar, apply, prog1, prog2, progn, do, dotimes, 
;$$        dolist, prog, catch und throw -- Lambda-Ausdrucke -- values-Begriff
;$$                                                                   [d..H.B.]
;++ 13 --> EXPLICIT ITERATIONS AND A FEW CONTROL STRUCTURES:
;++        mapcar, apply, prog1, prog2, progn, do, dotimes, dolist, prog, catch
;$$        and throw Primitives--Lambda-Expressions--Notion of Values
;===============================================================================

;;; -1- mapcar et lambda-expressions (fonctions anonymes)
;$$ -1- mapcar und Lambda-Ausdrucke (anonyme Funktionen)
;++ -1- mapcar and lambda-expressions (anonymous functions)

;;    1+sq <li> => incrmentation de 1 de tous les lments de <li>
;$    1+sq <li> => Erhhung aller Elemente von <li> um 1
;+    1+sq <li> => incrementation of 1 of all elements of <li>

(defun 1+sq (li)
  (mapcar #'1+ li))

;;   Equivalent rcursif :
;$   Rekursives quivalent:
;+   Recursive equivalent:

(defun 1+1+ (li)
  (if (endp li) nil
  (cons (1+ (car li)) (1+1+ (cdr li)))))

;;    ETelem1 <li1> <li2> => crible des lments communs  <li1> ET <li2>, sous
;;                           forme d'une liste d'atomes t et nil
;$    ETelem1 <li1> <li2> => Sieb aus den Element, die <li1> UND <li2> gleich 
;$                           sind, als eine Liste aus t und nil Atomen
;+    ETelem1 <li1> <li2> => sieve of the common elements of <li1> AND <li2>, as
;+                           a list of t and nil atoms
;- Ex. : / Beispiel: / Ex.:
;-    (ETelem1 '(a b c d e f g) '(x b x d e x x)) => (nil t nil t t nil nil)

(defun ETelem1 (li1 li2)
  (mapcar #'equal li1 li2))

;;   Equivalent rcursif :
;$   Rekursives quivalent:
;+   Recursive equivalent:

(defun ETET (li1 li2)
  (if (or (endp li1) (endp li2)) nil
    (cons (equal (car li1) (car li2)) (ETET (cdr li1) (cdr li2)))))

;;    transpoz1 <li> <n> => transposition de la liste simple de MIDI-pitches
;;                          <li> de <n> demi-tons
;;   Dans les deux exemples ci-dessous (transpoz1 et transpoz2), le recours  la
;; fonction cirlist n'est pas essentiel : elle peut tre remplace plus
;; prosaquement par une forme make-list comme dans la version transpoz1, ou
;; <n> pourrait tre intgr  une lambda-expression -- fonction anonyme et
;; temporaire -- comme dans la version transpoz1b.
;$    transpoz1 <li> <n> => Transponierung der einfachen MIDI-Pitches-Liste
;$                          <li> um <n> Halbtne
;$   In den beiden folgenden Beispielen (transpoz1 und transpoz2) ist der
;$ Gebrauch der cirlist-Funktion nicht notwendig: sie kann sachlicher durch
;$ eine make-list Form ersetzt werden, wie in der Version transpoz1, oder <n> 
;$ knnte in einen Lambda-Ausdruck -- als anonyme und temporre Funktion --
;$ wie in Version transpoz1b integriert werden.
;+    transpoz1 <li> <n> => transposition of the simple MIDI-pitches list <li>
;+                          by <n> semi-tones
;+   In the two examples below (transpoz1 and transpoz2), the use of the cirlist
;+ function is not essential: it can be replaced more blandly by a make-list
;+ form, as in the transpoz1 version, or <n> could be integrated in a lambda-
;+ expression--anonymous and temporary function-- as in the transpoz1b version.

(defun transpoz1 (li n)
  (mapcar #'+ li (make-list (list-length li) :initial-element n)))

;;   Plus lgant, avec liste circulaire :
;$   Eleganter mit zirkulrer Liste:
;+   More elegant, with circular list:

(defun transpoz1a (li n)
  (mapcar #'+ li (cirlist n)))

;;   Equivalent rcursif : ex.
;$   Rekursives quivalent: z.B.
;+   Recursive equivalent: ex.
;-    (TRTR '(60 64 67 70 72) (cirlist 12))

(defun TRTR (li1 li2)
  (if (or (endp li1) (endp li2)) nil
    (cons (+ (car li1) (car li2)) (TRTR (cdr li1) (cdr li2)))))

;;   Solution universelle : primitive lambda (lambda-expression) crant une
;; fonction anonyme ad hoc :
;$   Universale Lsung: lambda Primitive (Lambda-Ausdruck) erstellend einer
;$ anonymen ad-hoc-Funktion:
;+   Universal solution: lambda primitive (lambda-expression) creating an
;+ anomymous ad hoc function:

(defun transpoz1b (li n)
  (mapcar #'(lambda (x) (+ x n)) li))

;;    transpoz2 <li> <n> => transposition de la liste simple de <hauteurs> <li>
;;                          de <n> demi-tons. Une <hauteur> est dfinie comme
;;                          une paire pointe de deux entiers :
;;                          (<note> . <octave>), o <note> est dans [0,11]
;; Dans le cas o elle n'aurait pas d'autre raison d'tre, on peut viter la
;; dfinition de la fonction auxiliaire tr-bis au moyen d'une lambda-expression
;; intgre au processus : cf. version transpoz2a.
;$    transpoz2 <li> <n> => Transponierung der einfachen <hauteur> Liste <li>
;$                          um <n> Halbtne. Eine <hauteur> ist ein dotted pair
;$                          aus zwei Ganzzahlen: (<Note> . <Oktave>) wobei
;$                          <Note> im Bereich [0,11] liegt
;$ Falls sie nicht anderweitig gebraucht wrde, knnte man die Definition der
;$ Hilfsfunktion tr-bis durch einen in die Funktion integrierten
;$ Lambda-Ausdruck vermeiden: Vgl. Version transpoz2a.
;+    transpoz2 <li> <n> => transposition of the simple <hauteur> list <li> by
;+                          <n> semi-tones. A <hauteur> is defined as a dotted
;+                          pair of two integer numbers: (<note> . <octave>),
;+                          where <note> is in [0,11]
;+ In case it had no other use, one can avoid the definition of the auxiliairy
;+ function tr-bis, by means of a lambda-expression integrated in the process:
;+ cf. version transpoz2a.

(defun transpoz2 (li n)
  (mapcar #'tr-bis li (cirlist n)))
(defun tr-bis (hauteur n)
  (let ((num (+ (+ (car hauteur)
                   (* 12 (cdr hauteur)))
                n)))
    (cons (mod num 12) (nth-value 0 (floor (/ num 12))))))

(defun transpoz2a (li n)
  (mapcar #'(lambda (hauteur)
              (let ((num (+ (+ (car hauteur)
                               (* 12 (cdr hauteur)))
                            n)))
                (cons (mod num 12) (floor (/ num 12)))))
          li))

;;   Fonction auxiliaire pour obtenir une liste infinie (circulaire) de nombres
;; <n>, par "modification physique" : cf. Chapitre 15.
;$   Hilfsfunktion um unendliche (zirkulre) Listen von Zahlen <n> durch
;$ "physische Modifizierung" zu erhalten: Vgl. 15. Kapitel.
;+   Auxiliairy function to obtain an infinite (circular) list of numbers <n>,
;+ by "physical modification": cf. Chapter 15.

(defun cirlist (n)
  (let ((li (list n)))
    (nconc li li)))

;;    profond <arb> => nombre de niveaux de profondeur dans l'arbre <arb>.
;;                     Variante de l'algorithme de nbratoms2 : cf. ci-dessous
;$    profond <arb> => Anzahl der Ebenen in der Baumstruktur <arb>. Variante des
;$                     des nbratoms2-Algorithmus: Vgl. unten
;+    profond <arb> => number of depth levels in the tree <arb>. Variant of the
;+                     nbratoms2 algorithm: cf. below

(defun profond (arb)
  (cond ((null arb) 1)
        ((atom arb) 0)
        (t (1+ (max1 (mapcar #'profond arb))))))

;;    max1 <li> => le plus grand des nombres prsents dans <li>. quivalentes
;;                 LISP, plus fiables : (eval (cons 'max <li>)), c'est--dire
;;                 (apply #'max <li>). Cf. ci-dessous
;$    max1 <li> => die grte Zahl in der Liste <li>. Zuverlssigeres 
;$                 LISP-Equivalent: (eval (cons 'max <li>)), d. h.
;$                 (apply #'max <li>). Vgl. unten
;+    max1 <li> => largest number included in <li>. LISP equivalents, more
;+                 reliable: (eval (cons 'max <li>)) or, in other words,
;+                 (apply #'max <li>). Cf. below

(defun max1 (li)
  (labels ((maxAUX (l n)
             (cond ((null l) n)
                   ((> (car l) n) (maxAUX (cdr l) (car l)))
                   (t (maxAUX (cdr l) n)))))
    (maxAUX li most-negative-fixnum)))

;;; -2- apply

;;   Apply d'une fonction (ex. la primitive max) aux lments d'une liste.
;; Permet de traiter des arguments dj regroups en liste alors que la fonction
;; utilise prvoit d'y accder sparment. Exemple : (apply #'max a).
;$   Apply einer Funktion (z. B.: des Primitive max) auf die Elemente einer
;$ Liste. Ermglicht schon in einer Liste zusammengefasste Argumente zu
;$ verarbeiten obwohl die verwendete Funktion dafr gedacht ist, auf diese
;$ einzeln zuzugreifen. Beispiel: (apply #'max a).
;+   Apply of a function (ex. the max primitive). Allows to process arguments
;+ already collected in a list, although the function used is intended to access
;+ them separately. Example: (apply #'max a).

;;    addlist <li> => somme des nombres de la liste <li>
;$    addlist <li> => Summe der Zahlen der Liste <li>
;+    addlist <li> => summ of the numbers of the <li> list
;-   quivaut  / quivalent: / Equivalent:
;-    (eval (cons '+ <li>))

(defun addlist (li) (apply #'+ li))

;;   Amlioration de ord<1 d'exo.11 : utilisation de let, labels et apply, pour
;; une meilleure ralisation, ainsi que du mot-clef :debut pour intervertir le
;; sens du tri -- croissant ou dcroissant.
;$   Verbesserung von ord<1 aus exo.11: Anwendung von let, labels und apply fr
;$ eine bessere Umsetzung, und des Schlsselworts :debut um die
;$ Sortierreihenfolge zu verndern -- auf- oder absteigend.
;+   Improvement of ord<1 of exo.11: use of let, labels and apply, for a better
;+ realization, and of the :debut key-word to invert the sorting order --
;+ increasing or decreasing.

(defun ord<>2 (li &key (debut #'min))
  (labels ((interne (l)
             (if (endp l) ()
             (let ((prem (apply debut l)))
               (cons prem (interne (remove prem l :count 1)))))))
    (interne li)))

;; -3- apply et mapcar utilises de concert
;$ -3- apply und mapcar gemeinsam benutzt
;+ -3- apply and mapcar used together

;;    nbratoms <li> => nombre d'atomes prsents,  tous les niveaux, dans <li>.
;;                     Plus efficace que la programmation du Chapitre 11.2a.
;;                     Variante de l'algorithme de profond : cf. ci-dessus
;$    nbratoms <li> => Anzahl der Atome auf allen Ebenen von <li>. Grere 
;$                     Effizienz als die Programmgestaltung von Kapitel 11.2a. 
;$                     Variante des profond-Algorithmus: vgl. oben
;+    nbratoms <li> => number of atoms, at all levels of <li>. More efficient
;+                     than the programming of Chapter 11.2a. Variant of the
;+                     profond algorithm: cf. above

(defun nbratoms2 (li) (declare (notinline nbratoms2))
  (cond ((null li) 0)
        ((atom li) 1)
        (t (apply #'+ (mapcar #'nbratoms2 li)))))

;;   Intervertir les deux dimensions d'un tableau, reprsent par une liste de
;; listes. Par exemple :
;$   Invertieren der zwei Dimensionen einer Tabelle, die durch einer Liste von
;$  Listen reprsentiert wird. Zum Beispiel:
;+   Invert the two dimensions of a table represented by a list of lists. For
;+ instance:
;-    (lideli>ldliei '((aaa bbb ccc)     =>  ((aaa 111 foo)
;-                     (111 222 333)          (bbb 222 bar)
;-                     (foo bar zut)))        (ccc 333 zut))
;-    Appel : / Aufruf: / Call:
;-        (lideli>ldliei '((aaa bbb ccc) (111 222 333) (foo bar zut)))
;-            => ((aaa 111 foo) (bbb 222 bar) (ccc 333 zut))
;; On peut concevoir ceci comme la "verticalisation" de donnes "horizontales",
;; ou vice-versa.
;;   Algorithme, dcrit en plusieurs tapes :
;; a. Il s'agit d'arriver  :
;$ Das kann wie die "Vertikalisierung" von "horizontalen" Daten verstanden 
;$ werden, oder umgekehrt.
;$   Beschreibung des Algorithmus in einzelnen Schritten:
;$ a. Man muss folgendes evaluieren:
;+ This can be conceived as the "verticalization" of "horizontal" data, or vice-
;+ versa.
;+   Description of the algorithm in several steps:
;+ a. One must evaluate the form: 
;-     (mapcar #'list '(aaa bbb ccc) '(111 222 333) '(foo bar zut))
;; b. Construction des quotes devant toutes les sous-listes d'une liste
;;    <lideli> :
;$ b. Hinzufgen von quotes vor allen Sublisten einer Liste <lideli>:
;+ b. Construction of quotes before all sub-lists of a list <lideli>:
;-     (labels ((interne (ldl)
;-               (if (endp ldl) ()
;-               (cons (list 'quote (car ldl))
;-                     (interne (cdr ldl))))))
;-       (interne <lideli>))
;; c. On ne peut malheureusement pas rdiger directement la forme
;$ c. Es ist leider nicht mglich direkt diesen Ausdruck zu schreiben:
;+ c. It is unfortunately not possible to write directly the form
;-     (apply #'mapcar '(#'list... ))
;;    qui aboutit  une erreur -- #'list can not be FUNCALLed or APPLYed. Il
;;    faut donc utiliser la formulation quivalente
;;    (eval (cons 'mapcar ... )) :
;$    der einen Fehler produziert -- Argument to apply/funcall is not a
;$    function. Also ist es ntig, den gleichwertigen Ausdruck
;$    (eval (cons 'mapcar ... )) zu verwenden:
;+    which creates an error--Argument to apply/funcall is not a function. The
;+    use of the equivalent formulation
;+    (eval (cons 'mapcar ... )) is thus required:
;-     (eval (cons 'mapcar (cons (quote #'list) <rsultat-de-b.-ci-dessus>)))
;; d. En combinant b. et c. on arrive  l'algorithme complet. Exemple :
;$ d. Durch Kombination von b. und c. erhlt man den kompletten Algorithmus.
;$    Beispiel:
;+ d. The complete algorithm joins b. and c. together. For instance:
;-     (lideli>ldliei '((L I) (D E) (L I))) => ((L D L) (I E I))
;;    Ou encore, reprsent sous forme de matrices :
;$    Oder dargestellt als Matrizen:
;+    Or, in the form of matrices:
;-     LI
;-     DE  =>  LDL
;-     LI      IEI

(defun lideli>ldliei (lideli)
  (eval (cons 'mapcar (cons (quote #'list)
                            (labels ((interne (ldl)
                                       (if (endp ldl) ()
                                       (cons (list 'quote (car ldl))
                                             (interne (cdr ldl))))))
                              (interne lideli))))))

;;   Gnralisation de la fonction prcdente, o la fonction argument de la
;; primitive mapcar -- qui tait la constante #'list -- est remplace par la
;; variable <Op>. On peut ainsi appliquer la fonction donne en argument <Op> 
;; tous les car successifs des sous-listes de <lideli>. Exemples :
;$   Verallgemeinerung der vorhergehenden Funktion, in welcher das
;$ Funktionsargument des mapcar-Primitives -- welches die Konstante #'list war
;$ -- durch die <Op>-Variable ersetzt wird. Man kann daher die als <Op>-Argument
;$ bergebene Funktion auf alle sukzessiven cars der Teillisten von <lideli>
;$ anwenden. Beispiele:
;+   Generalization of the preceding function, where the function argument of
;+ the mapcar primitive--which was the constant #'list--is replaced by the <Op>
;+ variable. One can thus apply the function given as <Op> argument to all
;+ successive cars of the sublists of <lideli>. Examples:
;-    (MxOpC #'list '((L I) (D E) (L I))) => ((L D L) (I E I));cf. lideli>ldliei
;-    (MxOpC #'append '(((10.1 55.5) (333 222)) ((15.3 75.5) (666 111))))
;-        => ((10.1 55.5 15.3 75.5) (333 222 666 111))
;-    (MxOpC #'+ '((10 20 30) (1 2 3) (9 7 5) (4 3 7))) => (24 32 45)
;-    (MxOpC #'< '((3.4 5.5 1.7) (5.5 3.2 4.9) (9.1 3.3 9.4))) => (T NIL T)
;-    (MxOpC #'cons '((c c+ f b g f+) (4 3 4 2 5 4)))
;-        => ((C . 4) (C+ . 3) (F . 4) (B . 2) (G . 5) (F+ . 4))
;-    (MxOpC #'pgcd3 '((100 55.5 333) (150 75.5 666))) => (50 0.5 333)
;-    (MxOpC #'gcd '((100 555 333) (150 755 666) (650 325 222))) => (50 5 111)
;-    (MxOpC #'min '((100 55.5 333) (150 75.5 666) (57.5 98.2 49.9)))
;-         => (57.5 55.5 49.9)
;;   Si <lideli> est vue comme une matrice, l'opration <Op> est effectue sur
;; ses "colonnes" successives, et une liste des rsultats est rendue.
;$   Falls <lideli> als Matrix betrachtet wird, wird die <Op>-Operation auf ihre
;$ einzelnen "Spalten" angewendet, und eine Liste der Ergebnisse wird
;$ zurckgegeben.
;+   If <lideli> is seen as a matrix, the <Op> operation is performed on its
;+ successive "columns", and a list of the results is returned.

(defun MxOpC (Op lideli)
  (eval (cons 'mapcar (cons Op
                            (labels ((interne (ldl)
                                       (if (endp ldl) ()
                                       (cons (list 'quote (car ldl))
                                             (interne (cdr ldl))))))
                              (interne lideli))))))

;;   Complment de la fonction prcdente : si <lideli> est vue comme une
;; matrice, l'opration <Op> est effectue sur ses "lignes" successives, et une
;; liste des rsultats est rendue. Exemples :
;$   Ergnzung zur vorherigen Funktion: falls <lideli> als Matrix betrachtet
;$ wird, wird die <Op>-Operation auf ihre einzelnen "Zeilen" angewendet und eine
;$ Liste mit den Ergebnissen zurckgegeben. Beispiele:
;+   Complement of the preceding function: if <lideli> is seen as a matrix, the
;+ <Op> operation is performed on its successive "lines", and a list of the
;+ results is returned. Examples:
;-    (MxOpL #'list '((L I) (D E) (L I))) => ((L I) (D E) (L I))    ;redondant !
;-    (MxOpL #'append '(((a) (b c)) ((1 2 3)) ((x y) (z))))
;-        => ((A B C) (1 2 3) (X Y Z)) 
;-    (MxOpL #'+ '((10 20 30) (1 2 3) (9 7 5) (4 3 7))) => (60 6 21 14)
;-    (MxOpL #'< '((3.4 5.5 1.7) (5.5 3.2 4.9) (3.3 9.1 9.4))) => (NIL NIL T)
;-    (MxOpL #'cons '((4 x) (11 y) (0 z))) => ((4 . X) (11 . Y) (0 . Z))
;-    (MxOpL #'pgcd3 '((100 55.5) (100 75))) => (0.5 25)
;-    (MxOpL #'gcd '((111 555 333) (150 755 665) (650 325 225))) => (111 5 25)
;-    (MxOpL #'min '((100 55.5 333) (150 75.5 666) (57.5 98.2 49.9)))
;-         => (55.5 75.5 49.9)

(defun MxOpL (Op lideli)
  (mapcar (lambda (souli) (apply Op souli)) lideli))

;;; -4- Squencement de formes : corps de defun ou de clause de cond, mais aussi
;;;     prog1, prog2 et progn -- exemples de structures
;$$ -4- Sequenzierung von Ausdrcken: defun-Rumpf oder cond-Bedingung, aber auch
;$$     prog1, prog2 und progn -- Strukturbeispiele
;++ -4- Sequencing of forms: defun body or cond clause, and also prog1, prog2,
;++     progn--examples of structures

;;   Les exemples suivants supposent des dfinitions pralables des variables
;; <a>, <b> et <c>.
;$   Die folgenden Beispiele setzen die vorherige Definition der Variablen <a>,
;$ <b> und <c> voraus.
;+   The following examples presume earlier definitions of variables <a>, <b>
;+ and <c>.

(defun structd ()
  "forme numero 1"                                    ;corps de defun
  "forme numero 2"
  (prins "variable a : " a)
  (prins "variable b : " b)
  (+ 33 12)
  "forme numero n")

(defun structc (&rest flag)
  (cond (flag flag)                                  ;corps de clause 1 de cond
        (t "forme numero 1"                          ;corps de clause 2 de cond
           "forme numero 2"
           (prins "variable a : " a)
           (prins "variable b : " b)
           (+ 33 12)
           "forme numero n")))

(defun struct1 (&rest flag)
  (if flag flag
  (prog1 "forme numero 1"                             ;prog1
         "forme numero 2"
         (prins "variable a : " a)
         (prins "variable b : " b)
         (+ 33 12)
         "forme numero n")))

(defun struct2 (&rest flag)
  (if flag flag
  (prog2 "forme numero 1"                             ;prog2
         "forme numero 2"
         (prins "variable a : " a)
         (prins "variable b : " b)
         (+ 33 12)
         "forme numero n")))

(defun structn (&rest flag)
  (if flag flag
  (progn "forme numero 1"                             ;progn
         "forme numero 2"
         (prins "variable a : " a)
         (prins "variable b : " b)
         (+ 33 12)
         "forme numero n")))

;;; -5- Do : programmation structure de boucles itratives
;$$ -5- Do: strukturierte Programmierung von Iterationsschleifen
;++ -5- Do: structured programming of iterative loops

;;   Redfinition de <m> exposant <n>, analogue  la version itrative en C de
;; ped.11.1.c : cf. Chapitre 11.1.
;$   Neudefinition der <n>ten Potenz von <m>, hnlich der iterativen Variante in
;$ C in ped.11.1.c: Vgl. Kapitel 11.1.
;+   Redefinition of <m> exponent <n>, similar to the C iterative version of
;+ ped.11.1.c: cf. Chapter 11.1.

(defun pow5 (m n)
  (if (not (integerp n)) (error "pow5 : l'exposant n'est pas entier : ~A." n)
  (do ((i 0 (1+ i))                                          ;variables internes
       (rez 1))
      ((= i n) rez)                                          ;test fin et retour
      (setq rez (* rez m)))))                                ;corps

;;    enumerer1 <li> => numration verbeuse des lments de la liste <li>
;$    enumerer1 <li> => wortreiche Aufzhlung der Elemente der Liste <li>
;+    enumerer1 <li> => verbose enumeration of the elements of the list <li>

(defun enumerer1 (li)
  (do ((i 0 (1+ i))                                          ;variables internes
       (tot (list-length li)))
      ((= i tot) (prins "soit : " tot " element(s)."))       ;test fin et retour
      (prins "    element " i "    :    " (nth i li))))      ;corps

;;   Srie de May/Feigenbaum : X[n+1] = rX[n](1-X[n]). Il s'agit d'une relation
;; rcursive entre le [n]ime et le [n+1]ime termes d'une srie de valeurs de
;; la variable <X>, avec un coefficient de contrle <r>. Malgr l'vidence de la
;; rcursivit, la programmation itrative est plus simple dans ce cas.
;;    feigen <X> <r> <n> => squence de <n> termes,  partir de <X>, d'une srie
;;                          de May/Feigenbaum de coefficient <r>
;$   May/Feigenbaum-Reihe: X[n+1] = rX[n](1-X[n]). Sie besteht aus einer
;$ rekursiven Beziehung zwischen dem [n]ten und [n+1]ten Element einer Reihe der
;$ Werte der Variable <X>, mit einem Kontrolkoeffizienten <r>. Trotz der
;$ offensichtlichen Rekursivitt ist eine iterative Programmierung in diesem
;$ Fall einfacher.
;$    feigen <X> <r> <n> => Reihe von <n> Elementen, beginnend mit <X>, einer
;$                          May/Feigenbaum-Reihe mit Koeffizient <r>
;+   May/Feigenbaum series: X[n+1] = rX[n](1-X[n]). It consists in a recursive
;+ relation between the [n]th and [n+1]th terms of a series with the control
;+ coefficient <r>. In spite of the obvious recursivity, iterative programming
;+ is simpler in tis case.
;+    feigen <X> <r> <n> => <n> terms, starting from <X>, of the May/Feigenbaum
;+                          series with coefficient <r>

(defun feigen1 (X r n)
  (do ((i      0 (1+ i))                                     ;variables internes
       (terme  X       ))
      ((= i n) 'ok)                                          ;test fin et retour
      (prins i " :    " terme)                               ;corps
      (setq terme (* r terme (- 1 terme)))))

;;   Programmation rcursive de la mme srie : moins lisible.
;$   Rekursive Programmierung derselben Reihe: weniger lesbar.
;+   Recursive programming of the same series: less readable.

(defun feigen2 (X r n)                                      ;<r> est constant
  (labels ((AuX (terme i)                                   ;dfinition de AuX
             (cond ((= i n) 'ok)                            ;clause terminale
                   (t (prins i " :    " terme)              ;sinon impression...
                      (setq terme (* r terme (- 1 terme)))  ;... calcul ...
                      (AuX terme (1+ i))))))                ;... et rcursion...
    (AuX X 0)))                                             ;1er appel de AuX

;;; -6- dotimes et dolist : boucles itratives simplifies
;$$ -6- dotimes und dolist: vereinfachte Iterationsschleifen
;++ -6- dotimes and dolist: simplified iterative loops

;-   dotimes

;;   <m>**<n> : <m> exposant <n> : cf. Chapitre 11.1. Equivalente LISP :
;;    (expt <m> <n>)
;$   <m>**<n>: <m> Exponent <n>: Vgl. Kapitel 11.1. LISP-Equivalent:
;$    (expt <m> <n>)
;+   <m>**<n>: <m> exponent <n>: cf. Chapter 11.1. LISP equivalent:
;+    (expt <m> <n>)

(defun pow6 (m n)
  (let ((rez 1))
    (dotimes (i n rez)
      (setq rez (* rez m)))))

;;   <n>! : <n> factorielle : cf. Chapitre 11.1.
;$   <n>!: <n> Fakultt: Vgl. Kapitel 11.1.
;+   <n>!: <n> factorial: cf. Chapter 11.1.

(defun fact4 (n)
  (let ((rez 1))
    (dotimes (i n rez)
      (setq rez (* rez (1+ i))))))

;;    enumerer2 <li> => numeration verbeuse des lments de la liste <li>
;;   La primitive dotimes n'intgre pas de mcanisme pour accder aux lments
;; successifs d'une liste : le corps de la boucle est simplement valu un
;; certain nombre de fois. L'utilisateur doit grer lui-mme un ventuel accs
;; aux lments successifs d'une liste.
;$    enumerer2 <li> => wortreiche Aufzhlung der Elemente der Liste <li>
;$   Das Primitive dotimes enthlt keinen Mechanismus um Zugriff auf
;$ aufeinanderfolgende Elemente einer Liste zu bekommen: der Rumpf der Schleife
;$ wird einfach eine bestimmte Anzahl von Malen ausgefhrt. Der Benutzer muss
;$ den eventuellen Zugriff auf die aufeinanderfolgenden Elemente selbst
;$ verwalten.
;+    enumerer1 <li> => verbose enumeration of the elements of the list <li>
;+   The dotimes primitive does not integrate a mechanism to give access to the
;+ successive elements of a list: the loop's body is simply evaluated a certain
;+ number of times. The user must manage himself the eventual access to the
;+ successive elements of a list.

(defun enumerer2 (li)
  (dotimes (cmpt (list-length li) (prins "soit : " cmpt " element(s)."))
    (prins "    element " cmpt " : " (nth cmpt li))))                     ;corps

;-   dolist

;;    enumerer3 <li> => numeration verbeuse des lments de la liste <li>
;;   La primitive dolist n'intgre pas de compteur : le corps de la boucle est
;; simplement appliqu rptitivement  tous les lments successifs de la
;; liste. L'utilisateur doit grer lui-mme un ventuel compteur.
;$    enumerer3 <li> => wortreiche Aufzhlung der Elemente der Liste <li>
;$   Das Primitive dolist enthlt keinen Zhler: der Rumpf der Schleife wird
;$ einfach auf alle aufeinanderfolgenden Elemente der Liste angewendet. Der
;$ Benutzer muss einen eventuellen Zhler selbst verwalten.
;+    enumerer3 <li> => verbose enumeration of the elements of the list <li>
;+   The dolist primitive does not integrate a counter: the loop's body is
;+ simply applied repetitively to all successive elements of the list. The user
;+ must manage himself an eventual counter.

(defun enumerer3 (li &aux (cmpt -1))
  (dolist (elem li (prins "soit : " (incf cmpt) " element(s)."))
    (prins "    element " (incf cmpt) " : " elem)))                       ;corps

;;; -7- prog : programmation non structure, avec tiquettes et sauts
;$$ -7- prog: unstrukturierte Programmierung, mit labels und Sprngen
;++ -7- prog: unstructured programming, with labels and jumps

;;   Redfinition de max1 : cf. ci-dessus.
;$   Neudefinition von max1: Vgl. oben.
;+   Redefinition of max1: cf. above.

(defun max2 (li)
  (prog (tmp                                                ;variables internes
         (max  most-negative-fixnum)
         (n   (1- (list-length li))))                       ;<n> in [0,<length>[
    pasfini (when (minusp n) (return max))                  ;test fin et retour
            (setq tmp (nth n li))                           ;corps
            (when (> tmp max) (setq max tmp))
            (setq n (1- n))
            (go pasfini)))                                  ;itration explicite

;;   Rend F<n>, le <n>-ime terme de la srie de Fibonacci, pour <n> > 1. Si le
;; mot-clef :rapport est t, rend aussi le rapport F<n> / F<n>-1 comme second
;; lment d'une paire pointe. Ce dernier rapport tend vers le nombre d'or
;;    (/ (+ 1.0 (sqrt 5.0)) 2.0) = 1.618033988749895
;;  mesure que crot <n>. En MCL, on atteint cette valeur ds le 40-ime terme.
;;   Tester ventuellement avec la forme suivante :
;$   Gibt F<n> zurck, das <n>te Glied der Fibonacci-Folge, wobei <n> > 1. Wenn
;$ das Schlsselwort :rapport t ist, wird auch das Verhltnis F<n> / F<n-1> als
;$ zweites Element eines dotted pair ausgegeben. Letzteres Verhltnis
;$ nhert sich mit steigendem <n> dem goldenen Schnitt an:
;$    (/ (+ 1.0 (sqrt 5.0)) 2.0) = 1.618033988749895
;$ In MCL wird dieser Wert schon beim 40. Glied erreicht.
;$   Mit dem folgenden Ausdruck berprfbar:
;+   Outputs F<n>, the <n>-th term of the Fibonacci serie, for <n> > 1. If the
;+ :rapport key-word is t, outputs also the ratio F<n> / F<n-1> as second
;+ element of a dotted pair. This last ratio tends towards the Golden Mean
;+    (/ (+ 1.0 (sqrt 5.0)) 2.0) = 1.618033988749895
;+ as <n> increases. In MCL, this value is attained after only 40 terms.
;+   Test eventually within the following form:
;-    (let ((i 1)) (pasapas (prins (incf i) " => " (fibo9 i :rapport t))))

(defun fibo9 (n &key rapport)
  (prog ((n0 1)
         (n1 1)
         tmp)
    (when (not (integerp n)) (go erreur1))
    (when (< n 2) (go erreur2))
    boucle   (when (< n 2) (go resultat))
             (setq tmp (+ n0 n1))
             (setq n0 n1)
             (setq n1 tmp)
             (decf n)
             (go boucle)
    resultat (return (if rapport (cons n1 (float (/ n1 n0))) n1))
    erreur1  (error "fibo9 : ~A n'est pas entier." n)
    erreur2  (error "fibo9 : ~A n'est pas > 1." n)))

;;; -8- Utiles pour Csound : mapcar, dotimes, dolist et lambda-expressions
;$$ -8- Ntzlich in Csound: mapcar, dotimes, dolist und Lambda-Ausdrucke
;++ -8- Useful for Csound: mapcar, dotimes, dolist and lambda-expressions

;;   Interpolations linaires ou non-linaires entre deux listes de valeurs :
;; <listA> de valeurs de dpart, et <listB> de valeurs d'arrive.
;;   L'interpolation est effectue en <netap> tapes, les listes de dpart et
;; d'arrive tant incluses.
;;    interpolist <listA> <listB> <netap> [:expon <c>]
;;        => liste de <netap> sous-listes : la premire est <listA>, la dernire
;;           est <listB>. Les <netap>-2 sous-listes intermdiaires sont
;;           composes de valeurs interpoles linairement ou non-linairement,
;;           entre les valeurs respectives de <listA> et de <listB>, selon la
;;           valeur du mot-clef :expon [nil par dfaut => linaire]
;;   La valeur <c> du mot-clef :expon, lorsqu'elle n'est pas nil, permet de
;; contrler la courbure des interpolations non-linaires :
;;    <c> = 0 +/- epsilon => interpolation quasi linaire
;;    0 < <c> < 1 => courbure "moins prononce"
;;    <c> = 1 => exponentielle "ordinaire"
;;    <c> > 1 => courbure "plus prononce"
;;    <c> < 0 => courbure logarithmique, symtrique de l'exponentielle obtenue
;;               avec (abs <c>), par rapport au point milieu d'une interpolation
;;               linaire
;; Pour toute interpolation non-linaire :
;;    - <c> ne peut tre zro
;;    - les valeurs respectives de <listA> et de <listB> doivent tre de mmes
;;      signes deux  deux, et aucune ne peut tre zro
;; Ces contraintes sont les mmes que celles imposes aux segments exponentiels
;; en Csound par exemple.
;;   Dans tous les cas, <listA> et <listB> doivent tre de mme longueur, et
;; <netap> doit tre un entier suprieur  1. Exemple d'utilisation :
;$   Lineare oder nichtlineare Interpolation zwischen zwei Listen von Werten: 
;$ von einer Liste der Anfangswerte <listA> zu einer Liste der Endwerte <listB>.
;$   Die Interpolation wird mit <netap> Schritten durchgefhrt, inklusive der
;$ Anfangs- und Endlisten.
;$    interpolist <listA> <listB> <netap> [:expon <c>]
;$        => Liste von <netap> Sublisten: die erste ist <listA>, die letzte
;$           <listB>. Die dazwischenliegenden <netap>-2  Unterlisten bestehen
;$           aus den linear oder nichtlinear interpolierten Werten zwischen den
;$           jeweiligen Werten der Listen <listA> and <listB>, entsprechend
;$           dem Wert des Schlsselwortes :expon [standardmig nil => linear]
;$   Der Wert <c> des Schlsselwortes :expon erlaubt es, falls ungleich nil, die
;$ Kurve der nichtlinearen Interpolation zu kontrollieren:
;$    <c> = 0 +/- epsilon => quasi-lineare Interpolation
;$    0 < <c> < 1 => "nicht so markante" Krmmung
;$    <c> = 1 => "normales" Exponential
;$    <c> > 1 => "markantere" Krmmung
;$    <c> < 0 => logarithmische Krmmung, symmetrisch zur exponentiellen Kurve,
;$               die man mit (abs <c>) erhlt, relativ zum Mittelpunkt einer
;$               linearen Interpolation
;$ Fr alle nichtlinearen Interpolationen gilt:
;$    - <c> darf nicht 0 sein
;$    - die jeweiligen Werte der <listA> und <listB> mssen paarweise gleiche 
;$      Vorzeichen haben und ungleich 0 sein
;$ Diese Einschrnkungen entsprechen denen fr exponentielle Segmente z. B. 
;$ in Csound.
;$   Auf jeden Fall mssen <listA> und <listB> die gleiche Lnge haben und
;$ <netap> muss eine Ganzzahl grer als 1 sein. Benutzungsbeispiel:
;+   Linear or non-linear interpolation between two list of values: <listA> of
;+ begining values, <listB> of end values.
;+   The interpolation is performed in <netap> steps, the beginning and end
;+ lists included.
;+    interpolist <listA> <listB> <netap> [:expon <c>]
;+        => list of <netap> sub-lists: the first is <listA>, the last <listB>.
;+           The <netap>-2 intermediate sub-lists consist in linearly or non-
;+           linearly interpolated values between respectives values of <listA>
;+           and <listB>, according to the value of the :expon key-word [nil by
;+           default => linear]
;+   The value <c> of the :expon key-word, when not nil, allows to control the
;+ curvature of non-linear interpolations:
;+    <c> = 0 +/- epsilon => quasi-linear interpolation
;+    0 < <c> < 1 => "not so steep" curvature
;+    <c> = 1 => "ordinary" exponential
;+    <c> > 1 => "steeper" curvature
;+    <c> < 0 => logarithmic curvature, symetrical to the exponential obtained
;+               with (abs <c>), relative to the mid- point of a linear
;+               interpolation
;+ For all non-linear interpolations:
;+    - <c> must not be zero
;+    - the respective values of <listA> and <listB> must pairwise agree in sign
;+      and none should be zero
;+ These constraints are the same as those imposed on exponential segments in
;+ Csound for instance.
;+   In all cases, <listA> and <listB> must have the same length, and <netap>
;+ must be an integer greater than 1. Example usage:
;-    (interpolist '(1  5  10 -20) '(5  1 -10  60) 5) =>
;-        ((1 5 10 -20) (2 4 5 0) (3 3 0 20) (4 2 -5 40) (5 1 -10 60))
;-        ou plus proprement : / oder sauberer: / or prettyer:
;-            ((  1   5   10  -20)
;-             (  2   4    5    0)
;-             (  3   3    0   20)
;-             (  4   2   -5   40)
;-             (  5   1  -10   60))
;;   La fonction jolilideli, du Chapitre 14, serait utile ici pour la lisibilit
;; du rsultat. Exemple :
;$   Die jolilideli-Funktion des 14. Kapitels wre hier ntzlich um die
;$ Lesbarkeit der Ausgabe zu erhhen. Beispiel:
;+   The jolilideli function of Chapter 14 would be useful here to improve the
;+ result's readability. Example:
;-    (jolilideli
;-      (interpolist '(1 10 -30) '(100 0.0001 -0.5) 10 :expon 1.3)
;-      :largeur 15)
;;   La primitive push sera prsente au Chapitre 15.
;$   Das Primitive push wird im 15. Kapitel vorgestellt.
;+   The push primitive will be presented in Chapter 15.

(defun interpolist (listA listB netap &key expon
                   &aux baz amb ntp etap incrs rez)
  ;Dtection d'erreurs
  (when (/= (list-length listA) (list-length listB))
    (error "interpolist : deux listes de longueurs differentes."))
  (when (not (and (integerp netap) (> netap 1)))
    (error "interpolist : nombre d'etapes incoherent : ~A." netap))
  ;Initialisations et dtections d'erreurs spcifiques au cas exponentiel
  (if expon
    (progn (when (zerop expon)
             (error "interpolist, option :expon ~A : ne peut etre zero." expon))
           (let ((produit (mapcar #'* listA listB)))
             (when (eval (cons 'or (mapcar #'zerop produit)))
               (error "interpolist, option :expon : zero impossible."))
             (when (eval (cons 'or (mapcar #'minusp produit)))
               (error "interpolist, option :expon : signes differents")))
           (setq baz (mapcar #'(lambda (a0 aN) (* expon (log (/ aN a0))))
                             listA listB))                            ;(dbg baz)
           (setq amb (mapcar #'(lambda (a0 aN bz)
                                 (if (= aN a0) 0.0    ;viter 0/0 si aN=a0
                                 (/ (- aN a0) (- 1.0 (exp bz)))))
                             listA listB baz))                        ;(dbg amb)
           (setq ntp (1- netap)))
    (progn (setq incrs (mapcar #'/ (mapcar #'- listB listA);incrs -> lin.
                               (cirlist (1- netap))))               ;(dbg incrs)
           (setq etap listA)))                             ;tape de dpart
  ;Boucle de calcul
  (dotimes (n (- netap 2)                                  ;<netap>-2  calculer
              (append (cons listA (reverse rez)) (list listB)));constr. rsultat
    (if expon (push (mapcar                                ;non-linaire
                     #'(lambda (a0 am bz)
                         (+ a0 (* am (- 1.0 (exp (/ (* (1+ n) bz) ntp))))))
                     listA amb baz)
                    rez)
        (push (setq etap (mapcar #'+ etap incrs)) rez)))) ;linaire

;;   Dterminer une liste de valeurs par une rgle tenant compte des valeurs
;; d'une premire liste originelle pr-existante.
;$   Bilden einer Liste von Werten durch eine Regel, die die Werte einer schon
;$ bestehenden Originalliste verndert.
;+   Build a list of values through a rule transforming the values of an
;+ original preexisting list.

;;   a. Exemples : par application d'une fonction avec la primitive mapcar : cf.
;;                 dfinition de faire-liste, fichier ped.12
;$   a. Beispiele: durch die Anwendung einer Funktion mit dem Primitive mapcar:
;$                 Vgl. Definition von faire-liste, Datei ped.12
;+   a. Examples: by applying a function with the mapcar primitive: cf.
;+                definition of faire-liste, ped.12 file

(defun alea+/- (val)                                            ;=> val +/-[1,3]
  (+ val (* (1+ (random 2.0)) (if (zerop (random 2)) -1.0 1.0))))

(setq mesfreqs (faire-liste                                         ;La +/-1 ton
                '(* 440.0 (expt 2.0 (/ (* (- (random 3) 1) 2) 12)))))
(setq mesfreqs-var (mapcar #'alea+/- mesfreqs))       ;+/-[1,3] Hz => battements

;;    => Les listes de frquences <mesfreqs> et <mesfreqs-var> pourraient tre
;;       utilises pour crer des battements entre couples d'oscillateurs...
;$    => Die Frequenzlisten <mesfreqs> und <mesfreqs-var> knnten benutzt werden
;$       um Schwebungen zwischen Oszillatorpaaren herzustellen...
;+    => the frequency lists <mesfreqs> and <mesfreqs-var> could be used to
;+       create beatings between oscillator pairs...

;;   En ralit, il n'est pas toujours ncessaire de crer une fonction
;; spcifique pour raliser la prparation de la liste dpendante : une
;; lambda-expression suffit pour une utilisation unique.
;$   In Wirklichkeit ist es nicht immer ntig eine spezifische Funktion zur 
;$ Vorbereitung der voneinander abhngigen Liste zu erstellen: ein Lambda-
;$ Ausdruck fr einmalige Anwendung gengt.

(setq mesfreqs-var1
 (mapcar #'(lambda (val)
             (+ val (* (1+ (random 2.0)) (if (zerop (random 2)) -1.0 1.0))))
         mesfreqs))

;;   On peut envelopper le tout dans la fonction utilitaire suivante, destine 
;; voiler l'appel  mapcar, sans rien gagner cependant, ni en efficacit ni en
;; concision :
;$   Ein solcher Prozess kann in die folgende Funktion gepackt werden, die den
;$ Aufruf von mapcar versteckt, was aber keinen Vorteil bei Effizienz oder
;$ Kompaktheit bringt:
;+   Such a process can be wrapped in the following function, intended to hide
;+ the call to mapcar, but without any gain in efficiency nor terseness:

(defun faire-li<li (fonction liste)
    (mapcar fonction liste))

;; Exemple d'utilisation dans ce dernier cas :
;$ Benutzungbeispiel fr den letzten Fall:
;+ Example usage in this last case:

(defun PhiSur (f) (/ 1.61828 f))

(setq mesmoduls (faire-li<li #'PhiSur mesfreqs))

;; On ne gagne donc rien par rapport  l'utilisation directe de mapcar et d'une
;; lambda-expression :
;$ So gewinnt man nichts im Vergleich mit der direkten Benutzung von mapcar und
;$ einem Lambda-Ausdruck:
;+ Thus one does not gain anything compared to the straight use of mapcar and of
;+ a lambda-expression:

(setq mesmoduls1 (mapcar #'(lambda (f) (/ 1.61828 f)) mesfreqs))

;;   b. Exemples : par application d'une rgle avec la primitive dolist
;$   b. Beispiele: Anwendung einer Regel mit dem Primitive dolist
;+   b. Examples: by applying a rule with the dolist primitive

;;   Des algorithmes du type des prcdents peuvent aussi recourir  la
;; primitive dolist pour traiter squentiellement les lments de la liste
;; originelle. Ils prennent alors un aspect diffrent, plus explicitement
;; itratif.
;;   Par rapport  mapcar, la diffrence essentielle de dolist est que cette
;; dernire ne construit pas automatiquement la liste des rsultats des
;; valuations successives : l'algorithme doit s'en charger explicitement.
;$   Algorithmen der vorangegangenen Art knnen auch das Primitive dolist
;$ benutzen um die Elemente der ursprnglichen Liste sequenziell zu verarbeiten.
;$ Sie erhalten dadurch eine andere, deutlicher iterative, Form.
;$   Der wesentliche Unterschied von dolist und mapcar ist, dass ersteres nicht
;$ automatisch die Liste aller Ergebnisse der aufeianderfolgenden Evaluationen
;$ bildet: dafr ist der Algorithmus selbst verantwortlich.
;+   Algorithms of the preceding type can also use the dolist primitive to
;+ process sequentially the elements of the original list. They will then take
;+ another form, more explicitly iterative.
;+   The essential difference of dolist compared to mapcar is that the former
;+ does not automatically build the list of all the results of the successive
;+ evaluations: this must be explicitly performed by the algorithm.

(defun A->B (listA &aux temp rez)
    (setq temp (apply #'min listA))
    (setq temp (/ (apply #'max listA) temp))
    (dolist (la_valeur listA (reverse rez))         ;retour : liste de rsultats
      (setq rez (cons (* (/ la_valeur -440.0) temp) ;contruction liste rsultats
                      rez))))

(setq mesdB (A->B mesfreqs))

;;    => Les termes successifs de la liste <mesfreqs-var> sont utiliss pour la
;;       prparation des termes de la liste <mesdB>, en fonction du rapport
;;       entre les valeurs maximale et minimale de <mesfreqs-var>
;$    => Die aufeinanderfolgenden Elemente der Liste <mesfreqs-var> werden,
;$       gem dem Verhltnis zwischen den Maximal- und Minimalwerten der
;$       <mes-freqs-var>. zum Erstellen der Elemente der Liste <mesdB> benutzt
;+    => The successive terms of the <mesfreqs-var> list are used to prepare the
;+       successive terms of the <mesdB> list, in relation with the ratio of the
;+       maximal and minimal values of <mesfreqs-var>

;; Ou mieux, si usage unique, en lambda-expression :
;$ Oder besser, bei einmaliger Benutzung, als Lambda-Ausdruck:
;+ Or better, if used only once, as a lambda-expression:

(setq mesdB1 ((lambda (listA &aux temp rez)                 ;lambda-expression :
                (setq temp (apply #'min listA))             ;rendra la valeur
                (setq temp (/ (apply #'max listA) temp))    ;d'une fonction
                (dolist (la_valeur listA (reverse rez))
                  (setq rez (cons (* (/ la_valeur -440.0) temp)
                                  rez))))
              mesfreqs))                                    ;argument d'appel

;;   Dans tous les cas, rien n'empche d'tendre le principe  plusieurs listes
;; originelles pr-existantes. Par exemple :
;$   Auf jeden Fall kann dieses Prinzip auf viele schon vorhandene Listen
;$ angewendet werden. Zum Beispiel:
;+   In all cases, the principle can be applied to several original preexisting
;+ lists. For instance:

(setq Or1 '(440.0 333.0 668.5  167.9 1247.5))
(setq Or2 '(880.0 136.9 999.7 1256.9 645.9 ))
(setq pds1 (make-list 5 :initial-element 0.25))
(setq pds2 (make-list 5 :initial-element 0.50))
(setq pds3 '(0.75 0.80 0.85 0.90 0.95))
(defun interp (v1 v2 pds)
    (+ v1 (* pds (- v2 v1))))
(setq Mf1 (mapcar #'interp Or1 Or2 pds1))
(setq Mf2 (mapcar #'interp Or1 Or2 pds2))
(setq Mf3 (mapcar #'interp Or1 Or2 pds3))

;;    => <Mf1>, <Mf2> et <Mf3> seront trois liste de cinq valeurs pondres
;;       entre celles de <Or1> et <Or2>,  respectivement 25%, 50% et divers
;;       pourcentages
;$    => Die Listen <Mf1>, <Mf2> und <Mf3> werden aus fnf gewichteten Werte
;$       zwischen <Or1> und <Or2> bestehen, jeweils zu 25%, 50% und anderen
;$       Prozentstzen
;+    => <Mf1>, <Mf2> and <Mf3> will consist in lists of five weighted values
;+       between those of <Or1> and <Or2>, respectively at 25%, 50% and varied
;+       percentages

;;; -9- catch et throw : sauts non lexicaux
;$$ -9- catch und throw: nicht-lexikale Sprngen
;++ -9- catch and throw: non-lexical jumps

;;   Recherche mthodique de correspondance entre un objet donn et un ensemble
;; <corpus> d'objets de rfrence. Il s'agit ici de structures harmoniques. On
;; reprsente les 12 notes par les entiers dans [0,11]. Le <corpus> peut tre
;; modifi librement : les fonctions de recherche ci-dessous s'adapteront
;; automatiquement  son nombre d'lments.
;$   Methodische Suche nach bereinstimmungen zwischen einem gegebenen Objekt
;$ und einem <corpus> Set von Referenzobjekten. Dieses Beispiel bezieht sich auf
;$ harmonische Strukturen. Die 12 Noten werden durch die Ganzzahlen [0,11]
;$ dargestellt. Der <corpus> kann frei verndert werden: die folgende
;$ Suchfunktion passt sich automatisch an die Elementenanzahl an.
;+   Methodical search for correspondences between a given object and a <corpus>
;+ set of reference objects. This example deals with harmonic structures. The 12
;+ notes are repressented by integers in [0,11]. The <corpus> can be modified
;+ freely: the following search functions will automatically adapt to its number
;+ of elements.

(defparameter corpus '(("Maj" 0 4 7)            ("min" 0 3 7)
                       ("7eD" 0 4 7 10)         ("5ed" 0 3 6)
                       ("5eA" 0 4 8)            ("7eM" 0 4 7 11)
                               ("perso" 0 3 4 5 7 9 10)
                       ("7em" 0 3 7 10)         ("9eD" 0 2 4 7 10)))

;;   Fonctions de recherche :
;; a. accord slectionne les lments successifs du <corpus>, et
;; b. pogne effectue les comparaisons avec tous leurs renversements et
;;    transpositions possibles
;;     accord <ac> => si <ac> correspond  l'un des accords du <corpus>, avec ou
;;                    sans transposition et/ou renversement, rend trois values :
;;                    1. le "<nom>" de l'accord correspondant du <corpus>
;;                    2. un intervalle de transposition <tr>
;;                    3. un numro de renversement <n>
;;                    Rend nil autrement
;; La rdaction en deux fonctions distinctes n'est pas rellement essentielle.
;; Les princ et terpri sont superflus.
;$   Suchfunktionen:
;$ a. accord whlt die aufeinanderfolgenden Elemente des <corpus> aus, und
;$ b. pogne fhrt die Vergleiche mit allen ihren mglischen Umkehrungen und
;$    Transpositionen durch
;$    accord <ac> => wenn <ac> mit einem der Akkorde des <corpus> bereinstimmt,
;$                   mit oder ohne Transponierung und/oder Umkehrung, gibt es
;$                   drei Werte aus:
;$                   1. den Namen "<nom>" des bereinstimmenden Akkordes des
;$                      <corpus>
;$                   2. ein Transpositionsinterval <tr>
;$                   3. eine Umkehrungsnummer <n>
;$                   Ansonsten gibt es nil aus
;$ Die Ausfertigung in zwei getrennten Funktionen ist nicht wirklich notwendig.
;$ Die Funktionen princ und terpri sind berflssig.
;+   Search functions:
;+ a. accord selects the successive elements of <corpus>, and
;+ b. pogne performs the comparisons with all their posssible transpositions and
;+    positions
;+    accord <ac> => if <ac> corresponds to one of the chords of <corpus>, with
;+                   or without transposition and/or change of position, outputs
;+                   three values:
;+                   1. the name "<nom>" of the corresponding <corpus> chord
;+                   2. a <tr> transposition interval
;+                   3. a position number <n>
;+                   Outputs nil otherwise
;+ The construction in two separate functions is not really essential. The princ
;+ and terpri are superfluous.
;-   Exemple : / Beispiel: / Example:
;-    (accord (ord<>2
;-                (mapcar #'mod
;-                          (SymNot>midi '((g+ . 3) (f . 2) (c+ . 4) (a+ . 3)))
;-                          (cirlist 12))))

(defun accord (ac)
  (catch 'trouvaille                                      ;sortie : succs !
    (do ((n     0                     (1+ n))             ;boucle sur les <fin>
         (fin   (list-length corpus)        )             ;lments du corpus...
         (patt                              )
         (nrev                              ))
        ((= n fin))                                       ;sortie : bredouille !
      (setq patt (nth n corpus))
      (setq nrev  (1- (list-length patt)))
      (pogne ac patt nrev))))

(defun pogne (accd struct nrv)
  (do ((n        0  (1+ n))                               ;boucle sur les <nrv>
       (renv              ))                              ;renversements...
      ((= n nrv) nil)
    (princ (car struct))
    (princ "(renv. ")
    (princ n)
    (princ ") : ")
    (setq renv (rotag2 (cdr struct) n))                   ;rotag2 : cf. ped.11.3
    (do ((tr 0 (1+ tr)))                                  ;boucle sur les 12
        ((= 12 tr) nil)                                   ;transpositions...
      (when (equal accd (transpoz3 renv tr))
        (throw 'trouvaille                                ;sortie - non lexicale
               (values (car struct)                       ;1. "<nom>" d'accord
                       tr                                 ;2. interv. de transp.
                       n)))                               ;3. n de renversement
      (princ tr)
      (princ " "))
    (terpri)))

;;   Fonction auxiliaire pour la transposition de notes modulo 12 : cf.
;; transpoz1 ci-dessus.
;$   Hilfsfunktion fr die Modulo 12-Notentransposition: Vgl. transpoz1 oben.
;+   Auxiliary function for the modulo 12 transposition of notes: cf. transpoz1
;+ above.

(defun transpoz3 (li n)
  (mapcar #'mod (mapcar #'+ li (cirlist n)) (cirlist 12)))
