;===============================================================================
;;; 15 --> MODIFICATIONS "PHYSIQUES/DESTRUCTIVES" DE LISTES :
;;;        primitives setf, rplaca, rplacd, nsubstitute, delete, nconc, pop,
;;;        push -- listes "circulaires"
;$$ 15 --> "PHYSIKALISCHE/DESTRUKTIVE" MODIFIKATION VON LISTEN:
;$         die Primitives setf, rplaca, rplacd, nsubstitute, delete, nconc, pop,
;$         push -- "Kreisfrmige" Listen                              [d..H.B.]
;++ 15 --> "DESTRUCTIVE/PHYSICAL" MODIFICATION OF LISTS:
;++        setf, rplaca, rplacd, nsubstitute, delete, nconc, pop, push
;++        Primitives--"Circular" Lists
;===============================================================================

;;   Primitives :
;;    rplaca <liste> <vl> => remplace le car de <liste> par <vl> [CLtL2:423]
;;    rplacd <liste> <vl> => remplace le cdr de <liste> par <vl> [CLtL2:423]
;;    setf {<lx> <vl>}*   => remplace le contenu de <lx> par <vl>. <lx> est
;;                           une forme accdant  une donne [CLtL2:124]
;;    nsubstitute <nf> <vx> <sq> => remplace les occurrences de <vx> par <nf>
;;                           dans la squence <sq> [CLtL2:403]. quivalente non-
;;                           destructive : substitute
;;    delete <it> <sq>    => limine les occurrences de <it> dans la squence
;;                           <sq> [CLtL2:400]. quivalente non-destructive :
;;                           remove
;;    nconc &rest <li...> => concatnation des listes <li...> [CLtL2:419].
;;                           quivalente non-destructive : append
;;    pop <liste>         => rend et dtruit le car de <liste> [CLtL2:421].
;;                           quivalente non-destructive : car
;;    push <it> <liste>   => insre <it> en tte de <liste> [CLtL2:420].
;;                           quivalente non-destructive : cons
;;  utiliser  bon escient...
;$   Primitives:
;$    rplaca <liste> <vl> => ersetzt das car der <liste> durch <vl> [CLtL2:423]
;$    rplacd <liste> <vl> => ersetzt das cdr der <liste> durch <vl> [CLtL2:423]
;$    setf {<lx> <vl>}*   => ersetzt den Inhalt von <lx> durch <vl>. <lx> ist
;$                           ein Befehl zum Datenzugriff [CLtL2:124]
;$    nsubstitute <nf> <vx> <sq>
;$                        => ersetzt die Vorkommen von <vx> durch <nf> in der
;$                           Sequenz <sq> [CLtL2:403]. Nicht-destruktives
;$                           Equivalent: substitute
;$    delete <it> <sq>    => lscht die Vorkommen von <it> in der Sequenz <sq>
;$                           [CLtL2:400]. Nicht-destruktives Equivalent: remove
;$    nconc &rest <li...> => verkettet die Listen <li...> [CLtL2:419]. Nicht-
;$                           destruktives Equivalent: append
;$    pop <liste>         => gibt zurck und zerstrt das car der <liste>
;$                           [CLtL2:421]. Nicht-destruktives Equivalent: car
;$    push <it> <liste>   => fgt <it> am Anfang der <liste> ein [CLtL2:420].
;$                           Nicht-destruktives Equivalent: cons
;$ Ganz bewusst einzusetzen...
;+   Primitives:
;+    rplaca <liste> <vl> => replaces the car of <liste> by <vl> [CLtL2:423]
;+    rplacd <liste> <vl> => replaces the cdr of <liste> by <vl> [CLtL2:423]
;+    setf {<lx> <vl>}*   => replaces the content of <lx> by <vl>. <lx> is
;+                           a data access form [CLtL2:124]
;+    nsubstitute <nf> <vx> <sq> => replaces the occurrences of <vx> by <nf> in
;+                           the <sq> sequence [CLtL2:403]. Non-destructive
;+                           equivalent: substitute
;+    delete <it> <sq>    => suppresses occurrences of <it> in the <sq> sequence
;+                           [CLtL2:400]. Non-destructive equivalent: remove
;+    nconc &rest <li...> => concatenation of lists <li...> [CLtL2:419]. Non-
;+                           destructive equivalent: append
;+    pop <liste>         => returns and destroys the car of <liste>
;+                           [CLtL2:421]. Non-destructive equivalent: car
;+    push <it> <liste>   => inserts <it> at the beginning of <liste>
;+                           [CLtL2:420]. Non-destructive equivalent: cons
;+ To be used knowingly...

;;   Exemple [Wertz:156] :  partir de deux argument d'appel --  l'atome Z et la
;; liste (a b c ... ) --  construire ((Z b c ... ) a b c ... )  la mme adresse
;; en mmoire que la liste originelle (a b c ... ). 
;;   Les fonctions suivantes abordent graduellement des solutions possibles.
;$   Beispiel [Wertz:156]: Erstelle, von zwei Aufrufparametern -- dem Atom Z und
;$ der Liste  (a b c ...) -- ausgehend, ((Z b c...) a b c ...) an der selben
;$ Speicheradresse wie die Originalliste (a b c ...).
;$   Die folgenden Funktionen nhern sich schrittweise einer mglichen Lsung.
;+   Example [Wertz:156]: starting from two calling arguments--the atom Z and
;+ the list (a b c ...)--build ((Z b c...) a b c ...) at the same memory address
;+ as the original (a b c ...) list.
;+   The following functions gradually approach possible solutions.

(defun autocar1 (neuf liste)
  (cons (rplaca liste neuf) liste))

(defun autocar2 (neuf liste)
  (let ((copie liste))
    (cons (rplaca liste neuf) copie)))

(defun autocar3 (neuf liste)
  (let ((orig (copy-list liste))
        (modif (rplaca (copy-list liste) neuf)))
    (setf liste (nconc (list modif) orig))))

(defun autocar4 (neuf liste)
  (let ((orig (copy-list liste))
        (modif (rplaca (copy-list liste) neuf)))
    (setf (car liste) modif)
    (setf (cdr liste) orig))
  liste)

(defun autocar5 (neuf liste)
  (let ((premier (list (car liste))))
    (setf (car liste) (cons neuf (cdr liste)))
    (setf (cdr liste) (nconc premier (cdr liste))))
  liste)

;;   En rsum :
;;    (autocar1 99 zz) ne fonctionne pas correctement ; autocar2 non plus
;;    (autocar3 99 zz) mieux, mais pas tout  fait : pas de modification de zz
;;    (autocar4 99 zz) fonctionne correctement. valuer squentiellement :
;$   Zusammenfassung:
;$    (autocar1 99 zz) funktioniert nicht richtig: autocar2 auch nicht
;$    (autocar3 99 zz) besser, aber noch nicht ganz: keine Vernderung von zz
;$    (autocar4 99 zz) funktioniert korrekt. Evaluiere nacheinander:
;+   To recapitulate:
;+    (autocar1 99 zz) does not work properly; autocar2 neither
;+    (autocar3 99 zz) better, but not quite: no modification of zz
;+    (autocar4 99 zz) works correctly. Sequentially evaluate:
          (defparameter zz '(a b c d e f g))
          zz
          (autocar4 99 zz)
          zz
          (nsubstitute '22 'b zz)
          (nsubstitute '66 'f zz) ;=> ((99 B C D E F G) A 22 C D E 66 G)
;;    (autocar5 99 zz) fonctionne tout aussi bien, et plus conomiquement, en
;;                     rutilisant les mmes doublets pour les deux "parties" du
;;                     rsultat, qui restent donc forcment lies. valuer :
;$    (autocar5 99 zz) funktioniert genauso gut, und sparsamer durch
;$                     Wiederverwendung derselben CONS-Zellen fr die beiden
;$                     "Teile" des Ergebnisses, welche dadurch zwangslufig
;$                     verbunden bleiben. Evaluiere:
;+    (autocar5 99 zz) works just as well, and more thriftily, by reusing the
;+                     same cons cells for the two "parts" of the result, which
;+                     thus perforce remain linked. Evaluate:
          (defparameter zz '(a b c d e f g))
          zz
          (autocar5 99 zz)
          zz
          (nsubstitute '22 'b zz)
          (nsubstitute '66 'f zz) ;=> ((99 22 C D E 66 G) A 22 C D E 66 G)

(defparameter zz '(a b c d e f g))

;;   Permutation alatoire des lments d'une liste. Cf. bibliothque
;; stochastique et exercices du Chapitre 10.
;;    permuter1 <obj> => rend une copie permute de la liste <obj>. La liste
;;                       originelle n'est pas modifie
;$   Zufllige Permutation der Glieder einer Liste. Vgl. die stochastische
;$ Bibliothek und die bungen zu Kapitel 10.
;$    permuter1 <liste> => gibt eine permutierte Kopie der Liste <obj> zurck.
;$                         Die Ursprungsliste wurde nicht verndert
;+   Random permutation of the elements of a list. Cf. stochastic library and
;+ Chapter 10 Exercises.
;+    permuter1 <obj> => returns a permutated copy of the <obj> list. The
;+                       original list is not modified

(defparameter gg '(do re mi fa sol la si))

(defun permuter1 (obj)
    (let    ((perm ())
            (trav (list 0))
            (ic (1- (list-length obj)))
            (ix 0))
    (terpri) (princ "------>    orig :    ") (princ obj) (terpri)
    (do () ((= ix ic)) (nconc trav (list (incf ix))))
    (princ "ic :    ") (princ ic)
    (princ "    trav :    ") (princ trav)
    (princ "    perm :    ") (princ perm) (terpri) (terpri)
    (do () ((< ic 0))    ;<ic> = [(1- (list-length obj)),0] <--- dbut de boucle
        (setq ix (random (1+ ic)))                  ;<ic>.1 <-------------------
        (push (nth (nth ix trav) obj) perm)         ;<ic>.2 <-------------------
        (setf (car (nthcdr ix trav)) (nth ic trav)) ;<ic>.3 <-------------------
        (princ "ic :    ") (princ ic)
        (princ "    ix :    ") (princ ix)
        (princ "    trav :    ") (princ trav)
        (princ "    perm :    ") (princ perm) (terpri)
        (decf ic))                                  ;<ic>.4 <-------------------
    perm))

;;   Listes circulaires... pas aisment imprimables ! Les valuations des
;; fonctions suivantes doivent tre interrompues manuellement, quand elles sont
;; lances depuis le top-level -- sauf si la variable d'environnement
;; Common LISP *print-length* en limite l'impression.
;;    ncirclist <liste> => la <liste> originelle est dfinitivement modifie
;;    circlist  <liste> => rend une copie : la <liste> originelle n'est pas
;;                         modifie. quivalente non-destructive de ncirclist.
;;                         Cf. aussi fonction cirlist, fichier ped.13
;$   Kreisfrmige Listen... nicht leicht auszugeben! Evaluierungen der folgenden
;$ Funktionen mssen normalerweise manuell unterbrochen werden wenn sie im
;$ top-level ausgefhrt werden -- auer wenn die CommonLISP-Umgebungsvariable
;$ *print-length* die Ausgabe limitiert.
;$    ncirclist <liste> => die Original-<liste> wird dauerhaft verndert
;$    circlist  <liste> => gibt eine Kopie zurck: die Original-<liste> wird
;$                         nicht verndert. Nicht-destruktives Equivalent zu
;$                         ncirclist. Siehe auch die Funktion cirlist, Datei
;$                         ped.13
;+   Circular lists... not easily printable! Evaluations of the following
;+ functions must be manually interrupted when launched from the top-level--
;+ except if the Common LISP environment variable *print-length* limits their
;+ printout.
;+    ncirclist <liste> => the original <liste> is permanently modified
;+    circlist  <liste> => returns a copy: the original <liste> is not modified.
;+                         Non-destructuve equivalent of ncirclist. See also
;+                         cirlist function, ped.13 file

(defun ncirclist (lst)
  (nconc lst lst))

(defun circlist (lst)
  (let ((copie (copy-list lst)))
    (nconc copie copie)))
