;; <1> Definition des Wurzel-Paths
(setq common-lisp::*wurzel-path*
  (merge-pathnames
    (make-pathname :directory (pathname-directory *load-truename*)) "*"))
;; <2> Zustzliche Aufladung(en)
(dolist (lefic '(
                 "ped.11.3.lisp"
                 ))
  (load (setq lefic (merge-pathnames lefic common-lisp::*wurzel-path*)))
  (format t "~&;; Geladen ~S~%" lefic))
;===============================================================================
;;; 16.1a --> P-LISTES : exemple d'application  des liens familiaux
;;;           -- cf. H. Wertz, p. 57, Exercice 3, et solution pp. 275 sq.
;$$ 16.1a --> P-LISTEN: Anwendungsbeispiel auf Verwandtschaftsverhltnisse
;$$           -- Vgl. H. Wertz, S. 57, bung 3, und Lsung S. 275 sq. [d..H.B.]
;++ 16.1a --> P-LISTS: Example Application to Familial Relations
;++           --cf. H. Wertz, p. 57, Exercise 3, and Solution pp. 275 sq.
;===============================================================================

;;   Utilitaire pour crer un couple proprit-valeur dans la P-liste d'un
;; symbole.
;;    put <symb> <propr> <val> => ajout du couple <propr> <val> en tte de la
;;                                P-liste de <symb>
;$   Hilfsfunktion zum Erzeugen eines Eigenschaft-Wert-Paars in der P-Liste
;$ eines Symbols.
;$    put <symb> <propr> <val> => fgt ein <propr> <val>-Paar am Anfang der
;$                                P-Liste des <symb> hinzu
;$    put <symb> <propr> <val> => ...
;+   Utility to create a property-value couple in a symbol's P-list.
;+    put <symb> <propr> <val> => addition of the <propr> <val> couple at the
;+                                beginning of the P-list of <symb>

(defun put (symb propr val)
  (setf (get symb propr) val))

;;   La variable globale *recensement* contiendra une liste de tous les
;; individus du groupe familial.
;$   Die globale Variable *recensement* (Volkszhlung) wird eine Liste aller
;$ Individuen einer Familiengruppe enthalten.
;+   The *recensement* (census) global variable will contain a list of all
;+ individuals of the familial group.

(defparameter *recensement* ())

;;   La fonction suivante sert  dclarer une naissance aux fins du recensement,
;; et  inscrire l'tat-civil du nouvel individu dans sa P-liste, sous forme de
;; trois couples proprit-valeur :
;;        pere <nom-du-pre> mere <nom-de-la-mre> sexe <mf>
;;    naissance '<nom> '(<nom-du-pre> <nom-de-la-mre> <mf>)
;;        => ajout de <nom> en tte de la liste *recensement*, et cration de la
;;           P-liste approprie de <nom> contenant son tat-civil
;$   Die folgende Funktion wird verwendet um eine Geburt in der Volkszhlung zu
;$ deklarieren und den Personenstand des neuen Individuums in ihrer/seiner
;$ P-Liste in Form von drei Eigenschafts-Wert-Paaren zu erfassen:
;$        pere <Name-des-Vaters> mere <Name-der-Mutter> sexe <mf>
;$    naissance '<Name> '(<Name-des-Vaters> <Name-der-Mutter> <mf>)
;$        => Hinzufgen von <name> am Anfang der *recensement*-Liste, und
;$           erstellen einer geeigneten P-Liste von <name> mit seinem/ihrem
;$           Personenstand
;+   The following function is used to declare a birth in the census and to
;+ enroll the civil status of the new individual in his/her P-list, in the form
;+ of three property-value couples:
;+        pere <name-of-the-father> mere <name-of-the-mother> sexe <mf>
;+    naissance '<name> '(<name-of-the-father> <name-of-the-mother> <mf>)
;+        => addition of <name> at the beginning of the *recensement* list, and
;+           creation of the appropriate P-list of <name> containing his/her
;+           civil status

(defun naissance (indiv et_civ)
  (set indiv indiv)            ;afin d'viter de quoter les noms par la suite...
  (put indiv 'pere (car et_civ))
  (put indiv 'mere (cadr et_civ))
  (put indiv 'sexe (caddr et_civ))
  (prins indiv " := " (symbol-plist indiv))
  (prins "*recensement* := " (push indiv *recensement*))
  'ok)

;;   Quelques fonctions de recherche, donnant des listes d'individus possdant
;; un certain lien de parent, ou nil. Mode d'utilisation :
;;    <fonction> <nom> => liste d'individus possdant avec <nom> la relation
;;                        de parent <fonction>
;; Par exemple :
;;    (fils pierre) => liste des fils de pierre
;$   Ein paar Suchfunktionen, um Listen von Individuen mit bestimmten familiren
;$ Beziehungen zurckzugeben, oder nil. Anwendung:
;$    <Funktion> <Name> => Liste der Individuen, die eine familire Beziehung
;$                         <function> zu <name> besitzen
;$ Zum Beispiel:
;$    (fils pierre) => Liste von Pierre's Shnen
;+   A few search functions returning list of individuals possessing a certain
;+ familial relation, or nil. Usage:
;+    <function> <name> => list of individuals possessing with <name> the
;+                         <function> familial relation
;+ For instance:
;+    (fils pierre) => list of the sons of pierre

(defun freres (indiv)
  (let ((pere (get indiv 'pere))
        (mere (get indiv 'mere)))
    (remove indiv
            (cond ((and pere mere)
                   (ttoutt 'sexe 'm
                           (remove-duplicates  ;possiblilit demi-frre(s) := OR
                            (append (ttoutt 'mere mere *recensement*)
                                    (ttoutt 'pere pere *recensement*)))))
                  (pere
                   (ttoutt 'sexe 'm
                           (ttoutt 'pere pere *recensement*)))
                  (mere
                   (ttoutt 'sexe 'm
                           (ttoutt 'mere mere *recensement*)))
                  (t nil)))))

(defun soeurs (indiv)
  (let ((pere (get indiv 'pere))
        (mere (get indiv 'mere)))
    (remove indiv
            (cond ((and pere mere)
                   (ttoutt 'sexe 'f
                           (remove-duplicates  ;possiblilit demi-soeur(s) := OR
                            (append (ttoutt 'mere mere *recensement*)
                                    (ttoutt 'pere pere *recensement*)))))
                  (pere
                   (ttoutt 'sexe 'f
                           (ttoutt 'pere pere *recensement*)))
                  (mere
                   (ttoutt 'sexe 'f
                           (ttoutt 'mere mere *recensement*)))
                  (t nil)))))

(defun freres+soeurs (indiv)
  (append (freres indiv) (soeurs indiv)))

(defun fils (indiv)
  (append (ttoutt 'sexe 'm (ttoutt 'pere indiv *recensement*))
          (ttoutt 'sexe 'm (ttoutt 'mere indiv *recensement*))))

(defun filles (indiv)
  (append (ttoutt 'sexe 'f (ttoutt 'pere indiv *recensement*))
          (ttoutt 'sexe 'f (ttoutt 'mere indiv *recensement*))))

(defun enfants (indiv)
  (append (fils indiv) (filles indiv)))

(defun oncles+tantes (indiv)
  (let ((pere (get indiv 'pere))
        (mere (get indiv 'mere)))
    (append (freres+soeurs pere)
            (freres+soeurs mere))))

;;   Fonction rcursive de recherche effective,  trois arguments d'appel :
;;    <propri> := proprit  contrler dans les P-listes
;;    <val>    := valeur  chercher pour la proprit donne
;;    <lst>    := liste des individus (P-listes) parmi lesquel(le)s effectuer
;;                la recherche
;$   Effiziente rekursive Suchfunktion mit drei Aufrufparametern:
;$    <propri> := Eigenschaft, die in den P-Listen geprft werden soll
;$    <val>    := Wert, der fr die gegebene Eigenschaft gesucht werden soll
;$    <lst>    := Liste der Individuen (P-Listen) unter denen gesucht werden
;$                soll
;+   Effective recursive search function, with three calling arguments:
;+    <propri> := property to be tested in the P-lists
;+    <val>    := value searched for the given property
;+    <lst>    := list of individuals (P-lists) among which to perform the
;+                search

(defun ttoutt (propri val lst)
  (labels ((ttauxtt (lst)
             (cond ((null lst) ())
                   ((equal (get (car lst) propri) val)
                    (cons (car lst) (ttauxtt (cdr lst))))
                   (t (ttauxtt (cdr lst))))))
    (ttauxtt lst)))

;;   Fonctions accdant  diverses autres relations familiales.
;$   Funktionen die geben verschiedene andere familire Beziehungen zurck.
;+   Functions accessing different other family relations.

(defun parents (indiv)
  (list (get indiv 'pere)
        (get indiv 'mere)))

(defun grand-parents-paternels (indiv)
  (let ((pere (get indiv 'pere)))
    (list (get pere 'pere) (get pere 'mere))))

(defun grand-parents-maternels (indiv)
  (let ((mere (get indiv 'mere)))
    (list (get mere 'pere) (get mere 'mere))))

(defun grand-parents (indiv)
  (append (grand-parents-paternels indiv)
          (grand-parents-maternels indiv)))

;;   Les naissances suivantes forment le groupe familial de la Figure de
;; [Wertz:57]. Elles peuvent tre compltes par d'autres naissances pour
;; dmontrer les fonctions en direct...
;$   Die folgenden Geburten ergeben die Familiengruppe der Abbildung in
;$ [Wertz:57]. Sie knnen durch andere Geburten ergnzt werden um die Funktionen
;$ direkt zu veranschaulichen...
;+   The following births compose the familial group of the Figure of
;+ [Wertz:57]. They can be completed by other births to demonstrate the
;+ functions online...

(naissance 'claude    '(nil nil m))
(naissance 'martine   '(nil nil f))
(naissance 'jean      '(nil nil m))
(naissance 'albert    '(claude martine m))
(naissance 'eve       '(nil nil f))
(naissance 'pierre    '(jean martine m))
(naissance 'julie     '(nil nil f))
(naissance 'alice     '(nil nil f))
(naissance 'klaus     '(albert eve m))
(naissance 'sarah     '(nil nil f))
(naissance 'gerard    '(pierre julie m))
(naissance 'guy       '(pierre julie m))
(naissance 'marie     '(pierre julie f))
(naissance 'jacques   '(pierre julie m))
(naissance 'jane      '(nil nil f))
(naissance 'alain     '(klaus alice m))
(naissance 'anne      '(gerard sarah f))
(naissance 'paul      '(nil nil m))
(naissance 'catherine '(jacques jane f))
(naissance 'brian     '(alain catherine m))
(naissance 'judy      '(paul anne f))

;-   Exemples : / Beispiele: / Examples:
;-    (enfants pierre) => (jacques guy gerard marie)
;-    (naissance 'barnabe '(pierre eve m))
;-        => barnabe := (sexe m mere eve pere pierre)
;-           *recensement* := (barnabe judy brian catherine anne alain ...
;-    (fils pierre) => (barnabe jacques guy gerard)
;-    (naissance 'thomas '(klaus sarah m))
;-        => thomas := (sexe m mere sarah pere klaus)
;-           *recensement* := (thomas barnabe judy brian catherine anne ...
;-    (freres+soeurs thomas) => (alain anne)
;-    (oncles+tantes judy) => (thomas)
