;===============================================================================
;;; 16.1d -->  PROPOS DES APPELS RCURSIFS CALCULANT LA SQUENCE DE FIBONACCI
;$$ 16.1d --> BER REKUSRIVE AUFRUFE, WELCHE DIE FIBONACCI REIHE BERECHNEN
;$$                                                                   [d..F.Z.]
;++ 16.1d --> ABOUT THE RECURSIVE CALLS COMPUTING THE FIBONACCI SEQUENCE
;===============================================================================

;;   Pour mesurer les avantages des mmo-fonctions fibo3 et fibo4 (ped.16.1c),
;; qui liminent pratiquement tous les appels rcursifs, il faut les comparer 
;; fibo1, version "ultra-rcursive" de la squence de Fibonacci F(n) :
;$   Um die Vorteile der Memo-Funktionen fibo3 und fibo4 (ped.16.1c) zu
;$ bewerten, die praktisch alle rekursiven Aufrufe elimieren, mssen sie mit
;$ fibo1, der "ultra-rekursiven" Version der Fibonacci Reihe F(n), verglichen
;$ werden:
;+   To measure the advantages of memo-functions fibo3 and fibo4 (ped.16.1c),
;+ which practically eliminate all recursive calls, they must be compared to
;+ fibo1, "ultra-recursive" version of the Fibonacci sequence F(n):
;-    F(n) = F(n-1) + F(n-2), avec / mit / with F(0) = F(1) = 1
;; (cf. ped.11.1). Cette version a l'lgance d'tre simplement calque sur la
;; dfinition originelle de la squence. Mais malheureusement,  cause d'un
;; nombre excessif de rcursions, son temps de calcul est beaucoup trop long
;; pour de grands n.
;;   valuations calculant une squence de valeurs de F(n) avec fibo1 :
;$ (Vgl. ped.11.1). Diese Version ist elegant, weil sie einfach der
;$ ursprnglichen Definition der Reihe nachempfunden ist. Aber
;$ unglcklicherweise ist die Rechenzeit wegen der bermigen Anzahl an
;$ Rekursionen viel zu lang fr groe n.
;$   Evaluationen fr die Berechnung einer Reihe von Werten von F(n) mit fibo1:
;+ (cf. ped.11.1). This version has the elegance of being simply modeled on the
;+ original definition of the sequence. But unfortunately, because of an
;+ excessive number of recursions, its computing time is much too long for large
;+ n.
;+   Evaluations to compute a sequence of values of F(n) with fibo1:
;-    (setq n -1)
;-    (pasapas(progn(incf n)(princ(format nil"F1(~2D) => ~7D"n(fibo1 n)))))
;; N.B. : il vaut mieux, dans tous ces cas, copier-coller dans la fentre
;; Listener, et ritrer pasapas avec <RETURN>.
;$ N.B.: es ist in allen Fllen vorzuziehen, in den Listener zu copy-pasten
;$ und pasapas mit <RETURN> zu wiederholen.
;+ N.B.: it is preferable, in all cases, to copy-paste in the Listener window
;+ and to reiterate pasapas with <RETURN>.

(defun fibo1 (n) (declare (notinline fibo1))
  (cond ((zerop n) 1)
        ((= n 1)   1)
        (t         (+ (fibo1 (- n 1))
                      (fibo1 (- n 2))))))

;;   Mais on a vu qu'une version rcursive trs efficace, construisant la
;; squence  partir de ses premiers termes, est aussi possible :
;$   Aber wir haben gezeigt, dass eine sehr effiziente rekursive Version, die
;$ die Reihe von ihren ersten Termen aufbaut, auch mglich ist:
;+   But we have shown that a very efficient recursive version, building the
;+ sequence from its first terms, is also possible:
;-    F(0)+F(1) => F(2), F(1)+F(2) => F(3), ... F(n-2)+2F(n-1) => F(n)
;; (cf. fibo8, pedexo.d/exo.11). F(n) ne demande alors que n rcursions.
;;   valuations calculant une squence de valeurs de F(n) avec fibo8 :
;$ (Vgl. fibo8, pedexo.d/exo.11). F(n) erfordert dann nur n Rekursionen.
;$   Evaluationen fr die Berechnung einer Reihe von Werten von F(n) mit fibo8:
;+ (cf. fibo8, pedexo.d/exo.11). F(n) then only takes n recursions.
;+   Evaluations to compute a sequence of values of F(n) with fibo8:
;-    (setq n -1)
;-    (pasapas(progn(incf n)(princ(format nil"F8(~2D) => ~7D"n(fibo8 n)))))
;- avec ou sans trace... / mit oder ohne trace... / with or without trace...
;-    (trace fibo1 fibo8) (untrace) (untrace fibo1) (untrace fibo8)

(defun fibo8 (n &optional (i 0) (n-2 1) (n-1 1)) (declare (notinline fibo8))
  (if (< (incf i) n)
    (fibo8 n i n-1 (+ n-2 n-1))
  n-1))

;;   On peut toutefois tudier pour lui-mme le calcul de RF(n), le nombre
;; d'appels rcursifs valus par (fibo1 n).
;$   Die Berechnung von RF(n), der Anzahl der rekursiven Aufrufe, die durch
;$ (fibo1 n) evaluiert werden, kann jedoch selbst studiert werden.
;+   However, the computation of RF(n), the number of recursive calls evaluated
;+ by (fibo1 n) can be studied for itself.

;;   RF(n) peut tre calcul selon une dfinition "ultra-rcursive", calque sur
;; fibo1 :
;$   RF(n) kann durch eine "ultra-rekursive" Definition berechnet werden,
;$ nach Vorbild von fibo1:
;+   RF(n) can be computed from an "ultra-recursive" definition, modeled on
;+ fibo1:
;-    RF(n) = 2 + RF(n-1) + RF(n-2), avec / mit / with RF(0) = RF(1) = 0
;; Cet algorithme souffre du mme problme d'excs de rcursions que fibo1
;; lui-mme : son temps de calcul est beaucoup trop long pour de grands n.
;;   valuations calculant une squence de valeurs de RF(n) avec RekFibo1 :
;$ Dieser Algorithmus hat das gleiche Problem der bermigen Rekursionen wie
;$ fibo1 selbst: die Rechenzeit ist zu lang fr groe n.
;$   Evaluation fr die Berechnung einer Reihe von Werten RF(n) mit RekFibo1:
;+ This algorithm suffers from the same excessive recursions problem as fibo1
;+ itself: its computing time is much too long for large n.
;+   Evaluations to compute a sequence of values of RF(n) with RekFibo1:
;-    (setq n 1)
;-    (pasapas(progn(incf n)(princ(format nil"RF1(~2D) => ~7D"n(RekFibo1 n)))))

(defun RekFibo1 (n) (declare (notinline RekFibo1))
  (cond ((zerop n) 0)                                                   ;1 => 0
        ((= n 1)   0)                                                   ;1 => 0
        (t         (+ 2                                                 ;2 + ...
                      (RekFibo1 (- n 1))
                      (RekFibo1 (- n 2))))))

;;   Il faudrait trouver une formule... moins rcursive ? multiplicative ?
;$   Eine bessere Formel, weniger rekursiv? oder multiplikativer? muss gefunden
;$ werden...
;+   A better formula, less recursive? or multiplicative? must be found...

;;   The Online Encyclopedia of Integer Sequences (cf. http://OEIS.org), pour
;; la squence RF(n) ci-dessus "2 4 8 14 24..." confirme (A019274) :
;$   Die Online Enzyklopdie der Ganzzahlreihen (cf. http://OEIS.org) besttigt
;$ fr die obere RF(n) Reihe "2 4 8 14 24..." (A019274):
;+   The Online Encyclopedia of Integer Sequences (cf. http://OEIS.org), for the
;+ RF(n) sequence above "2 4 8 14 24..." confirms (A019274):
;- "Number of recursive calls needed to compute the n-th Fibonacci number F(n),
;- starting with F(1) = F(2) = 1"
;; et donne la formule applique dans RekFibo1 ci-dessus, ainsi que trois
;; autres. Utilisant des termes de la squence de Fibonacci, ces formules,
;; lorsqu'elles ont recours  fibo1, souffrent encore du mme problme d'excs
;; de rcursions que fibo1 elle-mme.
;$ und gibt die Formel, die in RekFibo1 oben angewendet wurde, sowie drei
;$ andere. Da sie Terme der Fibonacci Reihe benutzen, haben diese Formeln,
;$ rckbezogen auf fibo1, wieder das gleiche Problem der bermigen Rekursionen
;$ wie fibo1 selbst.
;+ and gives the formula applied in RekFibo1 above, as well as three others.
;+ Using terms of the Fibonacci sequence, these formulas, when resorting to
;+ fibo1, again suffer from the same excesive recursions problem as fibo1
;+ itself.

;- <1> cf. OEIS.org (A019274)
;-    RF(n) = 2F(n) - 2
;;   valuations calculant une squence de valeurs de RF(n) avec Rekfibo2 :
;$   Evaluationen fr die Berechnung einer Reihe von Werten von RF(n) mit
;$ RekFibo2:
;+   Evaluations to compute a sequence of values of RF(n) with RekFibo2:
;-    (setq n 0)
;-    (pasapas(progn(incf n)(princ(format nil"RF2(~2D) => ~7D"n(RekFibo2 n)))))

(defun RekFibo2 (n)
  (- (* 2 (fibo1 n)) 2))

;;   Mais avec recours  fibo8, ces dfinitions deviennent utilisables, mme
;; pour de grands n.
;;   valuations calculant une squence de valeurs de RF(n) avec RekFibo21 :
;$   Aber bei Zurckwendung zu fibo8, wird diese Definition ntzlich, sogar
;$ fr groe n.
;$   Evaluationen fr die Berechnung einer Reihe von Werten von RF(n) mit
;$ RekFibo21:
;+   But resorting to fibo8 makes these definitions usable, event for large n.
;+   Evaluations to compute a sequence of values of RF(n) with RekFibo21:
;-    (setq n 1)
;-    (pasapas(progn(incf n)(princ(format nil"RF21(~2D) => ~7D"n(RekFibo21 n)))))

(defun RekFibo21 (n)
  (- (* 2 (fibo8 n)) 2))

;- <2> cf. OEIS.org (A019274, Paul Barry, 24.10.2007)
;; Une formule de somme impliquant aussi F(n), qui n'est pas implmente ici.
;$ Eine Summenformel setzt auch F(n) voraus, was hier nicht implementiert ist.
;+ A sum formula also implying F(n), which is not implemented here.

;- <3> cf. OEIS.org (A019274, Zerinvary Lajos, 31.01.2008)
;;   Cette dfinition est moins efficace, car elle recourt  deux termes de la
;; squence de Fibonacci :
;$   Diese Definition ist weniger effizient, weil es auf zwei Terme der
;$ Fibonacci Reihe zurckgreift:
;+   This definition is less efficient since it resorts to two terms of the
;+ Fibonacci sequence:
;-    RF(n) = F(n-2) + F(n+1) - 2
;;   valuations calculant une squence de valeurs de RF(n) avec RekFibo3:
;$   Evaluationen fr die Berechnung einer Reihe von Werten von RF(n) mit
;$ RekFibo3:
;+   Evaluations to compute a sequence of values of RF(n) with RekFibo3:
;-    (setq n 1)
;-    (pasapas(progn(incf n)(princ(format nil"RF3(~2D) => ~7D"n(RekFibo3 n)))))

(defun RekFibo3 (n)
  (+ (fibo8 (- n 2)) (fibo8 (+ n 1)) -2))

;;   Expliquons maintenant l'origine de la premire mthode de calcul de RF(n)
;; donne par OEIS.org (A019274, <1> ci-dessus) :
;$   Jetzt erklren wir die Herkunft der ersten Berechnungsmethode fr RF(n),
;$ die von OEIS.org (A019274, <1> oben) gegeben wird:
;+   Let us now explain the origin of the first computation method for RF(n)
;+ given by OEIS.org (A019274, <1> above):
;-    RF(n) = 2F(n) - 2

;;   Un arbre binaire complet de rcursions pour n serait de
;$   Ein kompletter binrer Baum von Rekursionen wrde fr n 
;+   A complete binary tree of recursions for n would have
;-    KB(n) = 2^n - 2
;; branches (cf. http://OEIS.org, A000918). Mais l'arbre est incomplet, car
;; certaines de ses branches se terminent plus tt -- dans sa partie droite dans
;; mes reprsentations.
;;   valuations calculant une squence de nombres de branches de l'arbre
;; binaire complet KB(n) :
;$ ste haben (vgl. http://OEIS.org, A000918). Aber der Baum ist nicht
;$ vollstndig, weil einige seiner ste eher enden -- in dem rechten Teil in
;$ meiner Reprsentation.
;$   Evaluationen zur Berechnung einer Reihe von Anzahlen von sten eines
;$ kompletten binren Baums KB(n):
;+ branches (cf. http://OEIS.org, A000918). But the tree is not complete, since
;+ certain of its branches end earlier--in its right part in my representations.
;+   Evaluations to compute a sequence of numbers of branches of the complete
;+ binairy tree KB(n):
;-    (setq n 1)
;-    (pasapas(progn(incf n)(princ(format nil"KB(~2D) => ~7D"n(KompletterBaum n)))))

(defun KompletterBaum (n)
  (- (expt 2 n) 2))

;;    partir de l : quel est AA(n), le nombre de branches de l'arbre complet
;; qui sont limines pour donner RF(n) ? On peut dj tablir empiriquement
;$   Daher: Was ist AA(n), die Anzahl der ste, die vom kompletten Baum
;$ ausgemerzt werden, um sie an RF(n) zu geben? Man kann bereits erfahrungsgem
;$ behaupten:
;+   Hence: what is AA(n), the number of branches eliminated from the complete
;+ tree to give RF(n)? One can already empirically state
;-    AA(n) = KB(n) - RF(n)

;-         n          | 2    3    4    5    6    7    8    9   10   11   12   13
;- -------------------|---------------------------------------------------------
;-        F(n)        | 2    3    5    8   13   21   34   55   89  144  233  377
;-  KB=KompletterBaum | 2    6   14   30   62  126  254  510 1022 2046 4094 8190
;-     RF=RekFibo     | 2    4    8   14   24   40   66  108  176  286  464  752
;- AA=AusgemerzteAEste| 0    2    6   16   38   86  188  402  846 1760 3630 7438

;;   OEIS.org, pour la squence des nombres de branches limines AA(n)
;; ci-dessus "0 2 6 16 38...", donne (A167821, Veikko Pohjola, 13.11.2009) :
;$   OEIS.org gibt fr die Zahlenreihe der ausgemerzten ste AA(n) "0 2 6 16
;$ 38..."  oben, (A167821, Veikko Pohjola, 13.11.2009):
;+   OEIS.org, for the sequence of numbers of eliminated branches AA(n) "0 2 6
;+ 16 38..." above, gives (A167821, Veikko Pohjola, 13.11.2009):
;- "a(n) is the number of n-tosses having a run of 3 or more heads or a run of 3
;- or more tails for a fair coin (i.e. probability is a(n)/2^n)".
;; Une premire dfinition en langage Mathematica est donne :
;$ Eine erste Definition wird in der Sprache Mathematica gegeben:
;+ A first definition is given in Mathematica language:
;-    CoefficientList[Series[(2 x^2)/(1 - 3 x + x^2 + 2 x^3), {x, 0, 30}], x]
;; Cette dfinition repose sur les coefficients d'un polynome de la srie de
;; Taylor ci-dessus. Elle n'est pas implmente ici.
;$ Diese Definition beruht auf den Polynom-Koeffizienten der Taylor Reihe oben.
;$ Sie ist hier nicht implementiert
;+ This definition rests on the polynomial coefficients of the Taylor series
;+ above. It is not implemented here.

;;   On a montr qu'on peut aussi obtenir la squence des nombres de branches
;; limines AA(n) en calculant la diffrence entre l'arbre complet KB(n) et le
;; nombre de rcursions RF(n) :
;$   Wir haben gezeigt, dass man auch die Zahlenreihe der ausgemerzten ste
;$ AA(n) erhalten kann, indem man die Differenz zwischen dem kompletten Baum
;$ KB(n) und der Anzahl der Rekursionen RF(n) berechnet:
;+   We have shown one can also obtain the sequence of numbers of eliminated
;+ branches AA(n) by computing the difference between the complete tree KB(n)
;+ and the number of recursions RF(n):
;-    AA(n) = KB(n) - RF(n)
;; ou, en termes de squences OEIS.org,
;$ oder, in den Worten der Reihe von OEIS.org:
;+ or, in terms of OEIS.org sequences:
;-    A167821(n) = A000918(n) - A019274(n)
;;   valuations calculant une squence de valeurs de AA(n) avec
;; AusgemerzteAEste1 :
;$   Evaluationen zur Berechnung einer Reihe von Werten von AA(n) mit
;$ AusgemerzteAEste1:
;+   Evaluations to compute a sequence of values of AA(n) with
;+ AusgemerzteAEste1:
;-    (setq n 1)
;-    (pasapas(progn(incf n)(princ(format nil"AA2(~2D) => ~7D"n(AusgemerzteAEste1 n)))))

(defun AusgemerzteAEste1 (n)
  (- (KompletterBaum n) (RekFibo21 n)))

;;   La fonction suivante applique le principe de AusgemerzteAEste1 ci-dessus,
;; mais calcul directement, sans appeler les fonctions Kompletterbaum et
;; RekFibo21, par combinaison algbrique des deux formules appliques dans ces
;; fonctions :
;$   Die folgende Funktion wendet das Prinzip von AusgemerzteAEste1 von oben an,
;$ aber berechnet direkt, ohne die zwei Funktionen Kompletterbaum und RekFibo21
;$ aufzurufen, durch algebraische Kombinationen der beiden Formeln, die in
;$ diesen Funktionen angewendet sind:
;+   The following function applies the principle of AusgemerzteAEste1 above,
;+ but computed directly, without calling the two functions Kompletterbaum and
;+ RekFibo21, by algebrical combination of the two formulas applied in these
;+ functions:
;-    AA(n) = KB(n) - RF(n) = (2^n - 2) - (2F(n) - 2) = 2^n - 2F(n)
;;   C'est la seconde dfinition donne dans OEIS.org pour A167821, mais avec un
;; dcalage dans la numrotation des termes de la squence de Fibonacci. En
;; effet, je la dfinis  partir de F(0) = F(1) = 1 (cf. ci-dessus), tandis que
;; OEIS.org (A000045 et A019274) la dfinit  partir de F(0) = 0 et F(1) = 1.
;$   Es ist die zweite Definition, die auf OEIS.org fr A167821 gegeben wird,
;$ aber mit einer Verschiebung in der Zhlung der Terme der Fibonacci Reihe.
;$ Ich definiere es beginnend mit F(0) = F(1) = 1 (siehe oben), whrend OEIS.org
;$ (A000045 und A019274) es beginnend mit F(0) = 0 und F(1) = 1 definiert.
;+   It is the second definition given in OEIS.org for A167821, but with an
;+ offset in the numbering of terms of the Fibonacci sequence. I define it
;+ starting with F(0) = F(1) = 1 (cf. above), whereas OEIS.org (A000045 and
;+ A019274) defines it as starting with F(0) = 0 and F(1) = 1.
;;   valuations calculant une squence de valeurs de AA(n) avec
;; AusgemerzteAEste2 :
;$   Evaluationen zur Berechnung einer Reihe von Werten von AA(n) mit 
;$ AusgemerzteAEste2:
;+   Evaluations to compute a sequence of values of AA(n) with
;+ AusgemerzteAEste2:
;-    (setq n 0)
;-    (pasapas(progn(incf n)(princ(format nil"AA3(~2D) => ~7D"n(AusgemerzteAEste2 n)))))

(defun AusgemerzteAEste2 (n)
  (- (expt 2 n) (* 2 (fibo8 n))))

;;   De l :
;$   Daher:
;+   Hence:
;-    RF(n) = KB(n) - AA(n) = (2^n - 2) - (2^n - 2F(n)) =  2F(n) - 2
;-  cf. / Vgl. / cf. OEIS.org, A019274, <1> ci-dessus / oben / above
