;===============================================================================
;;; 16.2 --> A-LISTES : exemple d'application  des objets musicaux
;$$ 16.2 --> A-LISTEN: Auf musikalische Objekte angewendete Beispiele [d..H.B.]
;++ 16.2 --> A-LISTS: Example Applied to Musical Objects
;===============================================================================

;;   Utilitaire de modification de la valeur associe  la <clef> dans une
;; A-liste. La nouvelle valeur sera <val>.
;;    rplacval <clef> <val> <a-list> => rend le nouveau couple (<clef> . <val>),
;;                                      aprs modification physique du cdr du
;;                                      couple dans <a-list>
;$   Hilfsfunktion zur Vernderung eines Werts, der einem Schlssel <clef> in
;$ einer A-Liste zugeordnet ist. Der neue Wert wird <val> sein.
;$    rplacval <clef> <val> <a-list> => gibt nach der physikalischen Vernderung
;$                                      des cdr des Paars in <a-list> das neue
;$                                      Paar (<clef> . <val>) zurck
;+   Utility to modify the datum associated to a key <clef> in an A-list. The
;+ new datum will be <val>.
;+    rplacval <clef> <val> <a-list> => returns the new pair (<clef> . <val>),
;+                                      after physical modification of the cdr
;+                                      of the pair in <a-list>

(defun rplacval (clef val a-list)
  (rplacd (assoc clef a-list :test #'equal) val))

;;   Des objets musicaux multi-dimensionnels possdent plusieurs attributs,
;; paramtres ou caractristiques, pouvant tre reprsents comme des couples
;; clef-valeur de A-listes, de forme
;$   Multidimensionale musikalische Objekte besitzen einige Eigenschaften,
;$ Parameter oder Charakteristiken, welche durch Schlssel-Wert-Paare einer
;$ A-Liste in folgender Form abgebildet werden knnen:
;+   Multidimensional musical objects possess several attributes, parameters or
;+ characteristics which can be represented by A-list key-datum pairs of form
;-    ( (<attribut1> . <valeur1>) ... (<attribut<n>> . <valeur<n>>) ).

;;   Exemples d'attributs possibles...
;$   Beispiele mglicher Attribute...
;+   Examples of possible attributes...

(defparameter *attrib-objet* '(
  fils       ;ascendance
  hauteur    ;hauteur(s) de rfrence
  figurN     ;liste de hauteurs
  duree      ;dure globale
  tempo      ;tempo(i) M.M.
  figurR     ;liste de dures
  timbre     ;liste de timbres
  intens     ;intensit(s) globale(s)
  figurI     ;liste d'intensits
))

;; ... avec leurs valeurs par dfaut.
;$ ... mit ihren Standardwerten.
;+ ... with their default values.

(defparameter *val-a-objet* '(
  nil        ;ascendance
  (c . 4)    ;hauteur(s) de rfrence - une ou davantage
  (c . 4)    ;liste de hauteurs := structure d'intervalles
  1/4        ;dure globale := nombre/figure de note
  60         ;tempo(i) M.M. - un ou davantage
  1/4        ;liste de dures := structure de rapports
  piano      ;liste de timbres
  mf         ;intensit(s) globale(s) - une ou davantage
  mf         ;liste d'intensits := structure de rapports
))

;;   Cration d'un objet :
;;    =obj <objet> {<attribut<n>> <valeur<n>>}*
;;        => cration de l'<objet>, possdant toutes les valeurs par dfaut,
;;           sauf le(s) couple(s) <attribut<n>>-<valeur<n>> spcifi(s)
;;           expressement
;$   Erstellen eines Objekts:
;$    =obj <objet> {<attribut<n>> <valeur<n>>}*
;$        => erstellt ein <objet>, das alle Standardwerte besitzt, bis auf die
;$           ausdrcklich angegebenen <attribut<n>>-<valeur<n>>-Paare
;+   Creation of an object:
;+    =obj <objet> {<attribut<n>> <valeur<n>>}*
;+        => creation of the <objet> possessing all default values except the
;+           expressly specified <attribut<n>>-<valeur<n>> pair(s)

(defun =obj (objet &rest listcouples)
  (when (oddp (list-length listcouples))
    (error "=obj : liste de couple(s) de longueur impaire."))
  (set objet (pairlis *attrib-objet* *val-a-objet*))       ;objet par dfaut,
  (do ()                                                   ;avec spcificits...
      ((endp listcouples))
    (rplacval (pop listcouples) (pop listcouples) (symbol-value objet)))
  (format t ";~A =>~%;~4T~A~%" objet (symbol-value objet)))

;;   Exemples de crations d'objets :
;$   Beispiele fr Objekterstellungen:
;+   Examples of object creations:

(=obj '33bis)                                                  ;objet par dfaut
(=obj (gentemp) 'timbre   '(tuba piccolo)                      ;autre objet
                'figurN   '((f+ . 1) (g+ . 6))
                'hauteur  '(c+ . 4)
                'figurR   '(3/16 1/16))

;;   Modification (criture) de la valeur d'un attribut :
;;    >obj <objet> <attribut> nouvelle-<valeur>
;$   Vernderung (schreiben) des Werts eines Attributs:
;$    >obj <objet> <attribut> neuer-<valeur>
;+   Modification (writing) of the value of an attribute:
;+    >obj <objet> <attribut> new-<valeur>

(defun >obj (objet attribut valeur)
  (rplacval attribut valeur (symbol-value objet)))

;;   Lecture de la valeur d'un attribut :
;$   Lesen des Werts eines Attributs:
;+   Reading the value of an attribute:
;-    <obj <objet> <attribut>

(defun <obj (objet attribut)
  (cdr (assoc attribut (symbol-value objet) :test #'equal)))

;;   Exemple simple de fonction d'aide  l'utilisateur :
;$   Einfaches Beispiel einer Hilfefunktion fr Benutzer:
;+   Simple example of user help function:

(defun ? ()
  (format t "Les fonctions suivantes :~%")
  (format t "    =obj '<objet> {'<attribut> '<valeur>}*~60Tcreation~%")
  (format t "    <obj '<objet> '<attribut>~60Tlecture~%")
  (format t "    >obj '<objet> '<attribut> '<nouvelle-valeur>~60Tecriture~%")
  (format t "... sont definies"))

;;   Cet exemple ne traite pas les aspects relevant de la gestion d'un
;; environnement -- liste d'objets connus, filiations, sauvegardes, etc.
;; L'exploitation concrte n'est pas, non plus, aborde ici, en particulier
;; l'engendrement de nouveaux objets par transformation, combinaison,
;; interpolation, etc. parmi un ensemble d'objets pr-existants. Les fichiers
;; ped.16.1* donnent cependant des pistes dans ces directions.
;$   Dieses Beispiel beschftigt sich nicht mit Umgebungsverwaltungsthemen --
;$ Listen der bekannten Objekte, der Abstammungen, Dateispeicherung, etc. Die
;$ konkrete Bedienung wird hier auch nicht behandelt, im einzelnen die
;$ Erstellung neuer Objekte durch Transformation, Kombination, Interpolation,
;$ etc. innerhalb einer Menge schon vorhandener Objekte. Jedoch zeigen die
;$ ped.16.1*-Dateien Mglichkeiten in dieser Richtung auf.
;+   This example does not deal with environment management issues--list of
;+ known objects, filiations, file saving, etc. The concrete operation is not
;+ tackled here either, in particular the generation of new objects by
;+ transforming, combining, interpolating, etc. within a set of preexisting
;+ objects. However, the ped.16.1* files show possibilities in these directions.

;;   Mme fonctionalits en macrofonction (cf. Chap. 17) :
;$   Gleiche Funktionalitten als Makrofunktion (Vgl. Kap. 17):
;+   Same functionalities as macrofunction (cf. Chap. 17):

(defmacro >>obj (objet attribut valeur)
  `(rplacval ',attribut ',valeur ,objet))

(defmacro <<obj (objet attribut)
  `(cdr (assoc ',attribut ,objet :test #'equal)))
