;; <1> Definition des Wurzel-Paths
(setq common-lisp::*wurzel-path*
  (merge-pathnames
    (make-pathname :directory (pathname-directory *load-truename*)) "*"))
;; <2> Zustzliche Aufladung(en)
(dolist (lefic '(
                 "ped.11.3.lisp"                                    ;ldt ped.08
                 ))
  (load (setq lefic (merge-pathnames lefic common-lisp::*wurzel-path*)))
  (format t "~&;; Geladen ~S~%" lefic))
;===============================================================================
;;; 16.3a --> VECTEURS ET TABLEAUX
;$$ 16.3a --> VEKTOREN UND ARRAYS                                     [d..H.B.]
;++ 16.3a --> VECTORS AND ARRAYS
;===============================================================================

;;   Permutation alatoire des lments d'une liste. Cf. bibliothque
;; stochastique, Exercices du Chapitre 10, et fichier ped.15. Ici ralise avec
;; manipulation interne de vecteurs, et de manire un peu plus concise.
;;    permuter2 <liste> => rend une copie permute : la <liste> originale n'est
;;                         pas modifie
;; N.B. : length est une primitive de squences.
;$   Zufllige Permutation der Elemente einer Liste. Vgl. stochastische
;$ Bibliothek, bungen zu Kapitel 10, und Datei ped.15. Hier umgesetzt durch
;$ interne Manipulation an Vektoren, und ein bisschen prgnanter.
;$    permuter2 <liste> => gibt eine permutierte Kopie zurck: die Original-
;$                         <liste> wird nicht verndert
;$ N.B.: length ist ein Primitive fr Sequenzen.
;+   Random permutation of the elements of a list. Cf. stochastic library,
;+ Exercises of Chapter 10, and file ped.15. Realized here through internal
;+ manipulations on vectors, and somewhat more concisely.
;+    permuter2 <liste> => returns a permuted copy: the original <liste> is not
;+                         modified
;+ N.B.: length is a primitive for sequences.

(defun permuter2 (liste)
  (let* ((perm ())
         (longueur (length liste))
         (orig (make-array longueur :initial-contents liste))
         (ic (1- longueur))
         (trav (make-array longueur
                           :initial-contents
                              (do* ((indx 0 (1+ indx))
                                    (listindx (list indx) (push indx listindx)))
                                   ((= indx ic) listindx)))))
    ;(prins "orig : " orig) (prins "trav : " trav) (prins "perm : " perm)
    (do (ix)
        ((= ic -1) perm)
      (setq ix (random (1+ ic)))
      (push (aref orig (aref trav ix)) perm)
      ;(prins "    perm : " perm)
      (setf (aref trav ix) (aref trav ic))
      ;(prins "    trav : " trav)
      (decf ic))))

;;   Listes pour la dmonstration de permuter2. Essayer les formes suivantes :
;$   Listen zur Demonstration von permuter2. Probiere die folgenden Ausdrcke:
;+   Lists to demonstrate permuter2. Try the following forms:
;-    (pasapas (print (permuter2 gg)))
;-    (pasapas (print (permuter2 cc)))
;-    (pasapas (apply #'prins (permuter2 ph)))

(defparameter gg '(Cn C+ Dn D+ En Fn F+ Gn G+ An A+ Bn))
(defparameter cc '(10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29))
(defparameter ph
      '("belle Marquise, " "vos beaux yeux " "me font " "mourir " "d'amour ! "))

;;   Suite des exemples du Chapitre 8 concernant les carrs magiques -- cf.
;; [Anonyme, 1998 : "Carrs hypermagiques", <Pour la science>, 253 : 32-33].
;;   Tous (?) les carrs hypermagiques d'ordre 4 ont la constante magique 30 :
;; sommes des lignes, des colonnes, des diagonales, des diagonales brises, et
;; mme des sous-carrs 2x2 ! La mthode dcrite peut tre tendue  des carrs
;; d'ordre 8 ou 32.
;$   Fortsetzung des Beispiels aus Kapitel 8 in Bezug auf magische Quadrate --
;$ Vgl. [Anonymus, 1998: "Carrs hypermagiques", <Pour la science>, 253: 32-33].
;$   Alle (?) hypermagischen Quadrate 4. Ordnung haben eine magische Konstante
;$ von 30: die Summe der Reihen, Spalten, Diagonalen, gebrochenen Diagonalen,
;$ und sogar der 2x2 Unterquadrate! Die beschriebene Methode kann auf Quadrate
;$ 8. oder 32. Ordnung erweitert werden.
;+   Continuation of the examples of Chapter 8 concerning magical squares--cf.
;+ [Anonymous, 1998 : "Carrs hypermagiques", <Pour la science>, 253 : 32-33].
;+   All (?) hypermagical squares of order 4 have the magical constant 30: sum
;+ of lines, columns, diagonals, broken diagonals, and even of the 2x2 sub-
;+ squares! The method described can be extended to squares of order 8 or 32.

;;   Quelques carrs de dpart, kr1, ... kr4 : carrs rversibles, c'est--dire
;; dont les nombres sont dans l'ordre naturel. Transforms par la fonction
;; hypermag ci-dessous, chacun des 48 carrs rversibles possibles donne un
;; carr hypermagique correspondant [ibid.:33].
;$   Ein paar erste Quadrate, kr1, ... kr4: umkehrbare Quadrate, d. h. dass in
;$ ihnen die Zahlen in natrlicher Reihenfolge sind. Wenn man sie mittels der
;$ hypermag-Funktion unten umwandelt, ergibt jedes der 48 mglichen umkehrbaren
;$ Quadrate ein entsprechendes hypermagisches Quadrat [ibid.:33].
;+   A few initial squares, kr1, ... kr4: reversible squares, that is to say in
;+ which the numbers are in natural order. Transformed by the hypermag function
;+ below, each of the 48 possible reversible squares gives a corresponding
;+ hypermagical square [ibid.:33].

(defparameter kr1 (make-array '(4 4) :initial-contents         ;carr rversible
                              '(( 0  1  2  3)
                                ( 4  5  6  7)
                                ( 8  9 10 11)
                                (12 13 14 15))))
(defparameter kr2 (make-array '(4 4) :initial-contents         ;carr rversible
                              '(( 0  4  8 12)
                                ( 1  5  9 13)
                                ( 2  6 10 14)
                                ( 3  7 11 15))))
(defparameter kr3 (make-array '(4 4) :initial-contents         ;carr rversible
                              '(( 3  7 11 15)
                                ( 2  6 10 14)
                                ( 1  5  9 13)
                                ( 0  4  8 12))))
(defparameter kr4 (make-array '(4 4) :initial-contents         ;carr rversible
                              '(( 3  2  1  0)
                                ( 7  6  5  4)
                                (11 10  9  8)
                                (15 14 13 12))))

;;   Autre carr en ordre plus ou moins "naturel", mais non rversible : aprs
;; transformation par la fonction hypermag ci-dessous, il donne un carr semi-
;; magique, dont les sommes des lignes et colonnes sont bien 30, mais dont les
;; diagonales font la somme 34 -- ce qui est, par ailleurs, la somme magique
;; d'un carr naturel d'ordre 4 (par ex. dans la Mlancolie de Drer, voir aussi
;; ci-dessous [SESIANO, op.cit.]).
;$   Andere Quadrate mit mehr oder weniger "natrlicher" Reihenfolge, aber nicht
;$ umkehrbar: Wenn man sie mittels der hypermag-Funktion unten umwandelt,
;$ entstehen semi-magische Quadrate, in welchen die Summe der Reihen und Spalten
;$ zwar 30 ist, aber die Diagonalen die Summe 34 ergeben -- welche
;$ zuflligerweise die magische Summe natrlicher Quadrate 4. Ordnung ist (z. B.
;$ in Drers Melancholia, siehe auch unten [SESIANO, op.cit.]).
;+   Other square in more or less "natural" order, but not reversible: when
;+ transformed by the hypermag function below, it gives a semi-magical square,
;+ of which the sums of lines and columns are indeed 30, but of diagonals
;+ yielding the sum 34--which is, incidentally, the magical sum of a natural
;+ square of order 4 (for instance in Drer's Melancholia, see also below
;+ [SESIANO, op.cit.]).

(defparameter rr1 (make-array '(4 4) :initial-contents         ;non rversible
                              '(( 3  4 11 12)
                                ( 2  5 10 13)
                                ( 1  6  9 14)
                                ( 0  7  8 15))))

;;   Fonction utilitaire, pour l'impression des tableaux LISP sous forme de
;; carrs facilement lisibles.
;$   Hilfsfunktion um LISP-Arrays als leicht lesbare Quadrate abzubilden.
;+   Utility function to print LISP arrays as easily readable squares.

(defun carre (table)
  (dotimes (i (array-dimension table 0) 'ok)
    (dotimes (j (array-dimension table 1) (format t "~2%"))
      (format t "~5@A" (aref table i j)))))

;;   Ralisation d'un carr hypermagique selon la mthode dcrite [ibid.:Fig.2],
;;  partir d'un carr rversible.
;;   Utilisation type, consistant  imprimer un carr de dpart kr<n>, puis le
;; carr hypermagique qui en est dduit par la fonction hypermag, prcd de son
;; analyse (cf. fonction test ci-dessous) :
;$   Verwirklichung eines hypermagischen Quadrats durch die beschriebene Methode
;$ [ibid.:Fig.2], ausgehend von einem umkehrbaren Quadrat.
;$   Beispielanwendung, die zuerst ein Startquadrat ausgibt, gefolgt von einem
;$ mit der hypermag-Funktion aus letzterem erstellten hypermagischen Quadrat mit
;$ seiner Analyse durch die test Funktion (vgl. unten):
;+   Realization of an hypermagical square through the described method [ibid.:
;+ Fig.2], starting from a reversible square.
;+   Example usage, first printing an initial square, then the hypermagical
;+ square composed from the first by the hypermag function, preceded by its
;+ analysis by the test function (cf. below):
;-    (let ((kk kr4)) (carre kk) (carre (test (hypermag kk))))

(defun hypermag (orig)
  (let ((tmp nil                )
        (dbl (make-array '(4 4)))
        (rez (make-array '(4 4))))
    (dotimes (i 4)                    ;crer une copie pour prserver l'original
      (dotimes (j 4)
        (setf (aref dbl i j) (aref orig i j)))) 
    (dotimes (i 4)                        ;travailler sur la copie -- tape <1>
      (setq tmp            (aref dbl 2 i));inversion des 2 dernires lignes, ...
      (setf (aref dbl 2 i) (aref dbl 3 i))
      (setf (aref dbl 3 i) tmp           ))
    (dotimes (j 4)
      (setq tmp            (aref dbl j 2));... puis des 2 dernires colonnes
      (setf (aref dbl j 2) (aref dbl j 3))
      (setf (aref dbl j 3) tmp           ))
    (dotimes (i 4 rez)                    ;travailler sur la copie -- tape <2>
      (dotimes (j 4)
        (setf (aref rez (mod (+ i       (* 2 j)) 4)   ;(i,j) -> ((i+2j),(2i+3j))
                        (mod (+ (* 2 i) (* 3 j)) 4))
              (aref dbl i j))))))

;;   Contruction d'un carr magique d'ordre pair 4<k>, selon une mthode de
;; quadrants symtriques, frquemment prsente dans des textes arabes ds le
;; XIe sicle. La somme magique en est toujours
;$   Konstruktion eines magischen Quadrats 4<k>ter Ordnung mittels einer Methode
;$ symmetrischer Quadrate, oft in arabischen Texten bereits im 11. Jahrhundert
;$ dargestellt. Die magische Summe ist immer
;+   Construction of a magical square of order 4<k>, through a method of
;+ symmetrical quadrants, frequenty presented in arab texts as early as the XIth
;+ Century. The magical sum is always
;-    <S> = (/ (+ (expt (* 4 <k>) 3) (* 4 <k>)) 2)
;- Cf. [SESIANO, Jacques, 2005 : "Les carrs magiques en terre d'islam",
;- <Mathmatiques exotiques>, Dossier Pour la science : 47 : 38 sq.].
;;   Dans la boucle -a-, on prpare le premier quadrant [op.cit.:40,Fig.6a]. On
;; fait usage de la fonction permuter2 dfinie ci-dessus. Il est ventuellement
;; ncessaire de reprendre cette tape tant que le vecteur d'indices permuts
;; <indxp> ne permet pas de marquer exactement 2<k><k> cases rparties  raison
;; de <k> par ligne et <k> par colonne, c'est--dire sans que deux des 2<k><k>
;; couples d'indices des vecteurs <indxv> et <indxp> ne pointent sur une mme
;; case. La mthode utilise rend trs improbable la confection de carrs
;; d'ordre 20 ou suprieur (<k> >= 5). L'approche heuristique de cet algorithme
;; montre l ses limites...
;;   En dernier ressort, aprs un certain nombre de tentatives infructueuses
;; (cf. variable globale <Limite> fixe ci-dessous  3333) la procdure de
;; secours Hilfe est appele en -!- afin de construire le premier quadrant.
;; Cette procdure recourt encore  des permutations alatoires d'indices par
;; permuter2, mais rend toujours un quadrant bien constitu. Elle rduit par
;; contre grandement le nombre de solutions thoriquement possibles.
;;    l'issue de cette tape, quoi qu'il en soit, le premier quadrant est
;; constitu de 2<k><k> cases marques avec t, les autres restant  nil.
;;   En -b- on rabat ce premier quadrant symtriquement sur ses voisins [ibid.:
;; Fig.6b].
;;   En -c- on inscrit leur numro d'ordre de carr naturel uniquement dans les
;; cases marques t, puis en -d- on "rebrousse chemin", encore en mode de carr
;; naturel, en marquant cette fois les cases restes  nil [ibid.:Fig.6c].
;;   Des appels  la fonction d'impression carre, dfinie ci-dessus, et  prins
;; affichent des tapes intermdiaires de l'algorithme. On peut les supprimer.
;$   In der -a- -Schleife wird der erste Quadrant vorbereitet
;$ [op.cit.:40,Fig.6a]. Benutzt wird die oben definierte Funktion permuter2. Es
;$ ist eventuell ntig, diesen Schritt neu zu berechnen solange der Vektor des
;$ permutierten Indexes <indxp> nicht erlaubt, genau 2<k><k> Arrayfelder
;$ verteilt mit einer Rate von <k> pro Reihe und <k> pro Spalte zu markieren,
;$ also ohne dass zwei der 2<k><k> Indexkonstruktionen auf dasselbe Feld zeigen.
;$ Diese Methode macht die Konstruktion von Quadraten 24. Ordnung oder hher
;$ (<k> >= 6) relativ unwahrscheinlich. Der heuristische Ansatz dieses
;$ Algorithmus zeigt hier seine Grenzen...
;$   Als letztes Mittel wird nach einer bestimmten Anzahl fruchtloser Versuche
;$ (vgl. die unten auf 3333 gesetzte globale Variable <limite>) die "Hilfe"-
;$ Rettungsprozedur aufgerufen, welche den ersten Quadranten konstruiert.
;$ Diese Prozedur verwendet auch die zuflligen Index-Permutationen durch
;$ permuter2, aber fhrt immer zu einem zufriedenstellenden Quadranten.
;$ Andererseits verringert sie die Anzahl theoretisch mglicher Lsungen
;$ drastisch.
;$   Nach diesem Schritt besteht der erste Quadrant auf jeden fall aus 2<k><k>
;$ mit t markierten Feldern, die anderen bleiben nil.
;$   In -b- wird dieser erste Quadrant symmetrisch auf seine Nachbarn gefaltet
;$ [ibid.:Fig.6b]. 
;$   In -c- wird ihre Ordnungszahl im natrlichen Quadrat nur in die mit t
;$ markierten Felder geschrieben. Dann "kehrt" der Algorithmus in -d-  "zurck"
;$ um, wieder in der Reihenfolge eines natrlichen Quadrats, aber diesmal in die
;$ freien mit nil markierten Felder ihre Zahlen zu schreiben [ibid.:Fig.6c].
;$   Aufrufe der oben definierten Funktionen carre und prins geben die
;$ Zwischenstufen des Algorithmus aus. Sie knnen unterdrckt werden.
;+    In the -a- loop, the first quadrant is prepared [op.cit.:40,Fig.6a]. The
;+ permuter2 function defined above is used. It is eventually necessary to
;+ recompute this stage as long as the vector of permuted indexes <indxp> does
;+ not allow to mark exactly 2<k><k> array cases distributed at the rate of <k>
;+ per line and <k> per column, that is to say without two of the 2<k><k> index
;+ couples pointing at the same case. This method makes quite unlikely the
;+ construction of squares of order 24 or greater (<k> >= 6). The heuristic
;+ approach of this algorithm thus shows its limits...
;+   In last resort, after a certain number of fruitless trials (cf. global
;+ variable <Limite> set below to 3333) the Hilfe rescue procedure is called to
;+ construct the first quadrant. This procedure also uses random index
;+ permutations by permuter2, but always results in a satisfactory quadrant. On
;+ the other hand, it greatly reduces the number of theoretically possible
;+ solutions.
;+   After this stage, at all events, the first quandrant consists in 2<k><k>
;+ cases marked with t, the others remaining with nil.
;+   In -b- this first quadrant is symmetrically folded up on its neighbors
;+ [ibid.:Fig.6b].
;+   In -c- their natural square order number is inscribed only in the cases
;+ marked t. Then, in -d-, the algorithm "returns back", again in the order of a
;+ natural square, but this time inscribing their numbers in the cases remaining
;+ free with nil [ibid.:Fig.6c].
;+   Calls to the carre function defined above, and to prins, print out
;+ intermediate stages of the algorithm. They can be suppressed.
;-   Exemple d'appel : / Beispielaufruf: / Example call:
;-    (carre (mag4k 4)) => carre magique, ordre 16 ; somme magique = 2056 [...]
;!
;! SANS la procdure de secours Hilfe
;!    MCL 5.1 < 2009 :
;!        (mag4k 5) = limite utile
;!        (mag4k 6) jamais obtenu de rsultat !
;!    LispWorks Personal 5.1.1 (2009, 2.6 GHz Intel Core 2 Duo)
;!        (mag4k 4) = limite utile !
;!        (mag4k 5) peut prendre jusqu' 5 minutes !
;!        Pour rcuprer de la place mmoire...
;!            voir avant:  (room)
;!            en faire :   (mark-and-sweep 3)
;!            voir aprs : (room)
;!        Pour voir la mmoire bouffe par une valuation de (mag4k 4) :
;!         (let ((deb (total-allocation))) (mag4k 4) (- (total-allocation) deb))
;! AVEC procdure Hilfe (05/2009) : appele systmatiquement  partir de <k>=5
;! aprs <Limite> tentatives infructueuses -- 3333 prennent env. (<k>**2)+5 secs

(defparameter Limite 3333)

(defun mag4k (k &aux rez indxp)
  (let* ((4k    (* 4 k))
         (2k    (* 2 k))
         (2k2   (* 2 k k))
         (table (make-array (list 4k 4k)))
         (indxs (dotimes (i 2k rez) (dotimes (n k) (push i rez)))))
    (prins indxs) ;debug : liste d'indices ordonns
    ;(carre table)(prins "----") ;debug : table vide
    (do ((tot   0)                                                          ;-a-
         (cmpt   0 (1+ cmpt))
         (indxv (make-array 2k2 :initial-contents indxs)))
        ((= tot 2k2))
      (dotimes (i 2k) (dotimes (j 2k) (setf (aref table i j) nil)));RZ quadrant
      ;(format t "~12D " cmpt)
      (if (< cmpt Limite)
          (setq indxp (make-array 2k2 :initial-contents (permuter2 indxs)))
        (setq indxp (make-array 2k2 :initial-contents (Hilfe k))))          ;-!-
      ;(prins indxp) ;debug : vecteur d'indices permuts
      (dotimes (i 2k2)
        (setf (aref table (aref indxv i) (aref indxp i)) t))
      (setq tot 0)
      (dotimes (i 2k)
        (dotimes (j 2k) (if (aref table i j) (incf tot)))))
    (carre table)(prins "----") ;debug : premier quadrant prt
    (setq indxp (- 4k 1))                                                   ;-b-
    (dotimes (i 2k)
      (dotimes (j 2k)
        (setf (aref table i (- indxp j)) (aref table i j))))
    ;(carre table)(prins "----") ;debug : premier quadrant rabattu -> demi table
    (dotimes (i 2k)
      (dotimes (j 4k)
        (setf (aref table (- indxp i) j) (aref table i j))))
    (carre table)(prins "----") ;debug : demi table rabattue -> pleine table
    (setq rez 0)                                                            ;-c-
    (dotimes (i 4k)
      (dotimes (j 4k)
        (incf rez)
        (if (aref table i j) (setf (aref table i j) rez))))
    (carre table)(prins "----") ;debug : premire numrotation, ordre normal
    (setq rez 0)                                                            ;-d-
    (dotimes (i 4k)
      (dotimes (j 4k)
        (incf rez)
        (if (not (aref table (- indxp i) (- indxp j)))
          (setf (aref table (- indxp i) (- indxp j)) rez))))
    ;(carre table)(prins "----") ;debug : 2me numrotation, ordre inverse = FIN
    (prins "Carre magique, ordre " 4k
           " ; somme magique = " (/ (+ (expt 4k 3) 4k) 2)) (prins "")
    table))

;;   Procdure de secours appele par mag4k en dernier recours, aprs <Limite>
;; tentatives infructueuses de constituer le premier quadrant. Cette procdure
;; permute aussi alatoirement des indices, mais aboutit toujours  un rsultat
;; satisfaisant. Le nombre de quadrants possibles est cependant rduit par
;; rapport  toutes les solutions thoriquement accessibles par la procdure
;; plus gnrale d'abord tente dans mag4k. La rduction est de l'ordre de 30%
;; pour <k>=5, par exemple. Elle serait mme de plus en plus drastique pour des
;; valeurs infrieures de <k>, mais il n'est heureusement jamais ncessaire dans
;; ces cas de recourir  cette procdure restreinte.
;$   Rettungsprozedur, die als letztes Mittel von mag4k nach <Limite>
;$ fruchtlosen Versuchen, den ersten Quadranten zu konstruieren, aufgerufen
;$ wird. Diese Prozedur permutiert Indexe auch zufllig, aber erzeugt immer ein
;$ befriedigendes Ergebnis. Die Anzahl mglicher Quadranten ist jedoch, im
;$ Vergleich zu allen theoretisch durch die zuerst in mag4k versuchte
;$ generellere Prozedur erreichbaren, verringert. Die Reduktion liegt im Bereich
;$ von z. B. 30% fr <k>=5. Sie wrde noch drastischer fr kleinere Werte fr
;$ <k> ausfallen, aber es ist in diesen Fllen glcklicherweise niemals ntig,
;$ auf diese beschrnkte Prozedur zurckzugreifen.
;+   Rescue procedure called in last resort by mag4k, after <Limite> fruitless
;+ trials to construct the first quadrant. This procedure also randomly permutes
;+ indexes, but always yields a satisfactory result. The number of possible
;+ quadrants is however reduced relatively to all the solutions theoretically
;+ accessible by the more general procedure tried first in mag4k. The reduction
;+ is in the order of 30% for <k>=5, for instance. It would be even more drastic
;+ for inferior values of <k>, but it is fortunately never necessary to resort
;+ to this restricted procedure in those cases.

(defun Hilfe (k)
  (format t "~%; RETTUNG!!!~2%")
  (let (ser (rez '(x)))
    ;(dbg rez)
    (rplacd rez nil)     ;dtruire trace d'une utilisation prcdente de <rez> !
    ;(dbg rez)
    (dotimes (n (* 2 k))
      (push n ser))
    ;(dbg ser)
    (dotimes (n k (cdr rez))
      (nconc rez (permuter2 ser))
      ;(dbg rez)
      )))

;;   Construction de carrs magiques d'ordre impair <k>, par une mthode de
;; disposition des entiers naturels le long de diagonales brises -- comme
;; "enroules"  la manire des rayures d'une enseigne de barbier... La somme
;; magique en est toujours
;$   Konstruktion eines magischen Quadrats ungerader <k>ter Ordnung durch
;$ Ausbreiten der natrlichen Ganzzahlen entlang der gebrochenen Diagonalen --
;$ "Wicklung" wie die Streifen auf dem Schild eines Friseursalons... Ihre
;$ magische Summe ist immer
;+   Construction of magical squares of odd order <k>, by a method laying out
;+ the natural integers along broken diagonals--"wraparound" as the stripes of a
;+ barber shop sign... Their magic sum is always
;-    <S> = (/ (+ (expt <k> 3) <k>) 2) [SESIANO, op.cit.]
;;   L'une des quatre cases contigus  la case centrale est choisie au hasard
;; comme point de dpart pour le nombre 1. Les carrs obtenus sont toujours bien
;; magiques pour les lignes et colonnes, mais ont une imperfection dans une ou
;; deux de leurs diagonales principales, sauf si la case initiale choisie pour
;; le nombre 1 est exactement celle situe sous la case centrale. Ceci se
;; produit statistiquement une fois sur quatre en utilisant la fonction. Les
;; carrs  une ou deux diagonales dfectueuses sont appels semi-magiques
;; [BOYER, Christian, 2008 : "nigmes sur les carrs magiques", Jeux math',
;; <Dossier Pour la science> : 59 : 22 sq.].
;;   Des appels  la fonction d'impression carre, dfinie ci-dessus, et  prins
;; affichent des tapes intermdiaires de l'algorithme. On peut les supprimer.
;$   Eines der zum zentralen Feld benachbarten vier Felder wird zufllig als
;$ Startpunkt fr die Nummer 1 ausgewhlt. Die erhaltenen Quadrate sind immer
;$ korrekt magisch in Bezug auf Reihen und Spalten, haben allerdings Fehler in
;$ einem oder zwei der Hauptdiagonalen, auer wenn das Startfeld fr Nummer 1
;$ genau unter dem zentralen Feld liegt. Dies passiert statistisch bei jedem
;$ vierten Quadrat. Die Quadrate mit einer oder zwei fehlerhaften Diagonalen
;$ werden semi-magisch genannt [BOYER, Christian, 2008: "nigmes sur les carrs
;$ magiques", Jeux math', <Dossier Pour la science>: 59: 22 sq.].
;$   Aufrufe der oben definierten Funktionen carre und prins geben die
;$ Zwischenschritte des Algorithmus aus. Diese knnen unterdrckt werden.
;+   One of the four cases contiguous to the central case is randomly chosen as
;+ starting point for number 1. The squares obtained are always correctly
;+ magical for lines and columns, but have an imperfection in one or two of
;+ their main diagonals, except if the initial case chosen for number 1 is
;+ exactly the one located below the central case. This statistically happens
;+ for one square out of four. The squares possessing one or two faulty
;+ diagonals are called semi-magical [BOYER, Christian, 2008 : "nigmes sur les
;+ carrs magiques", Jeux math', <Dossier Pour la science> : 59 : 22 sq.].
;+   Calls to the carre function defined above, and to prins, print out
;+ intermediate stages of the algorithm. They can be suppressed.
;-   Exemple d'appel : / Aufrufbeispiel: / Example call:
;-    (carre (test (magBarbier 5)))
;-         => Carre magique, ordre 5 ; somme magique = 65 [...]

(defun magBarbier (k)
  (let ((milieu (/ (1- k) 2))
        ligne colon table)
    (setq table (list (1- milieu) (if (= 1 (random 2)) milieu (- milieu 2))))
    (prins table)
    (when (= 1 (random 2)) (setq table (reverse table)))
    (prins table)
    (setq ligne ( car table))
    (setq colon (cadr table))
    (setq table (make-array (list k k) :initial-element nil))
    ;(carre table)
    (prins "Carre magique, ordre " k
           " ; somme magique = " (/ (+ (expt k 3) k) 2)) (prins "")
    (dotimes (n (* k k) table)
      (setq ligne (mod (1+ ligne) k))
      (setq colon (mod (1+ colon) k))
      (when (aref table ligne colon)
        (setq ligne (mod (1+ ligne) k))
        (setq colon (mod (1- colon) k)))
      (setf (aref table ligne colon) (1+ n)))))

;;   Procdure de test d'un carr magique, affichant les sommes des lignes, des
;; colonnes, puis de toutes les diagonales, principales et brises.
;;   N.B. : le commentaire imprim ("Somme magique =...") n'est valable que pour
;; des carrs dont la somme magique est celle de carrs naturels de mme ordre
;$   Testfunktion fr magische Quadrate, die die Summe der Reihen, Spalten und
;$ sowohl der Haupt- wie auch der gebrochenen Diagonalen ausgibt.
;$   N. B.: der ausgegebene Kommentar ("Somme magique =...") ist nur fr
;$ Quadrate mit derselben magischen Summe wie natrliche Quadrate der gleichen
;$ Ordnung gltig.
;+   Magical square test procedure, printing the sums of lines, of columns, and
;+ of all diagonals, main as well as broken.
;+   N.B.: the printed comment ("Somme magique =...") is valid only for squares
;+ with the same magical sum as natural squares of identical order
;-    <S> = (/ (+ (expt <k> 3) <k>) 2), cf. magBarbier ci-dessus / oben / above
;; En particulier, il n'est PAS pertinent pour les carrs hypermagiques crs
;; par la fonction hypermag ci-dessus. Par contre, cette fonction de test met en
;; valeur les sommes magiques de toutes les diagonales des carrs hypermagiques.
;$ Im besonderen ist er NICHT fr mit der obigen hypermag-Funktion erstellte
;$ hypermagische Quadrate gltig. Dennoch zeigt diese Testfunktion die magische
;$ Summe aller Diagonalen eines hypermagischen Quadrats.
;+ In particular, it is NOT valid for hypermagical squares created by the above
;+ hypermag function. However, this test function highlights the magical sums of
;+ all diagonals of hypermagical squares.
;-   Exemples d'appel : / Aufrufbeispiele: / Example calls:
;-    (carre (test (hypermag kr3)))
;-    (carre (test (hypermag rr1)))
;-    (carre (test (mag4k 5)))
;-    (carre (test (magBarbier 9)))

(defun test (carre)
  (let* ((k (array-dimension carre 0))
         (s (/ (+ (expt k 3) k) 2))
         rez)
    (prins "Somme magique = " s)
    (prins "Sommes des lignes :")
    (dotimes (i k)
      (setq rez 0)
      (dotimes (j k)
        (incf rez (aref carre i j)))
      (prins i " = " rez))
    (prins "Sommes des colonnes")
    (dotimes (j k)
      (setq rez 0)
      (dotimes (i k)
        (incf rez (aref carre i j)))
      (prins j " = " rez))
    (prins "Sommes des diagonales NO-SE :")
    (dotimes (i k)
      (setq rez 0)
      (dotimes (j k)
        (incf rez (aref carre j (mod (+ i j) k))))
      (prins i " = " rez))
    (prins "Sommes des diagonales NE-SO :")
    (dotimes (i k)
      (setq rez 0)
      (dotimes (j k)
        (incf rez (aref carre j (mod (- k j i 1) k))))
      (prins (- k i 1) " = " rez))
    carre))

;;   La distance de Levenshtein mesure la similarit entre deux squences. Elle
;; est gale au nombre minimal d'lments qu'il faut supprimer, insrer ou
;; remplacer pour passer d'une squence  l'autre. Dans la fonction suivante,
;; l'une et/ou l'autre des deux squences <str1> et <str2> peuvent tre soit des
;; listes, des chanes de caractres ou des vecteurs (tableaux  une dimension).
;; Cf. http://fr.wikipedia.org/wiki/Distance_de_Levenshtein.
;$   Die Levenshtein-Distanz zwischen zwei Zeichenketten ist die minimale Anzahl
;$ von Einfge-, Lsch- und Ersetz-Operationen um eine Zeichenkette in eine
;$ andere umzuwandeln. In der folgenden Funktion knnen die eine und/oder die
;$ andere der beiden Folgen <str1> und <str2>  Listen, Zeichenketten oder
;$ Vektoren (eindimensionale Arrays) sein.
;$ Vgl. http://de.wikipedia.org/wiki/Levenshtein-Distanz.
;+   The Levenshtein Distance between two sequences is defined as the minimum
;+ number of edits needed to transform one sequence into the other, with the
;+ allowable edit operations being insertion, deletion, or substitution of a
;+ single element. In the following function, one and/or the other of the two
;+ sequences <str1> and <str2> can be either lists, character strings or vectors
;+ (one dimensional arrays).
;+ Cf. http://en.wikipedia.org/wiki/Levenshtein_distance.
;- Exemples : / Beispiele: / Examples:
;-    (Levenshtein '(n i c h e) '(c h i e n s)) => 5
;-    (Levenshtein "niche" "chiens") => 5
;-    (Levenshtein '#(7 5 3 2 4) '#(3 2 5 4 7 6)) => 5
;-    (Levenshtein '(60 64 67 69 70) '(67 69 64 70 60 65)) => 5      ;ex. midi
;-    (Levenshtein '((0 . 3) (4 . 3) (7 . 3) (9 . 3) (10 . 3))       ;ex. NumNot
;-                 '((7 . 3) (9 . 3) (4 . 3) (10 . 3) (0 . 3) (5 . 3))) => 5
;-    (Levenshtein '(n i c h e) '#(c h i e n s)) => 5
;-    (Levenshtein "niche" '(#\c #\h #\i #\e #\n #\s)) => 5
;-    (Levenshtein nil '#(n i c h e)) = (Levenshtein '(n i c h e) "") => 5
;; Si <S1> et <S2> sont des squences quelconques :
;$ Falls <S1> und <S2> beliebige Sequences sint:
;+ If <S1> and <S2> are any sequences:
;-    (Levenshtein <S1> <S1>) => 0
;-    (Levenshtein <S1> (reverse <S1>))
;-        => (if (evenp (length <S1>)) (length <S1>) (1- (length <S1>)))
;-    (Levenshtein <S1> <S2>) = (Levenshtein <S2> <S1>)
;; Si <L> est une liste quelconque :
;$ Falls <L> eine beliebige Liste ist:
;+ If <L> is any list:
;-    (Levenshtein (cons 'xx <L>) <L>) => 1
;-    (Levenshtein (croitre1 <L> (+ (length <L>) <n>)) <L>) => <n>
;; Si <L> est une liste sans doublons, et
;$ Falls <L> eine Liste ohne Duplikate ist, und
;+ If <L> is a list without duplicates, and
;-    <n> >= 0
;-    <m> = (mod <n> (length <L>)) :
;-    (Levenshtein <L> (rotad2 <L> <n>)) = (Levenshtein <L> (rotag2 <L> <n>))
;-        => (* 2 (min <m> (- (length <L>) <m>)))
;-    0 <= (Levenshtein <L> (permuter2 <L>)) <= (length <L>)
;;        par ex. (pour une meilleure lisibilit, ritrer avec <RETURN>) :
;$        zum Beispiel (bessere lesbarkeit wenn mit <RETURN> wiederholt):
;+        for instance (reiterate with <RETURN> for a better readability):
;-            - (pasapas (let ((perm (permuter2 a)))      ;ou pressebouton ou cc
;-                         (print a)                                      ;ou cc
;-                         (print perm)
;-                         (format t "~&Levenshtein Distanz = ~D~%"
;-                                   (Levenshtein a perm))))              ;ou cc
;-            - (pasapas (let ((perm (permuter2 gg)))     ;ou pressebouton ou cc
;-                         (print gg)                                     ;ou cc
;-                         (print perm)
;-                         (format t "~&Levenshtein Distanz = ~D~%"
;-                                   (Levenshtein gg perm))))             ;ou cc
;-            - (length (eval (append '(concatenate (quote string)) ph))) => 56
;;              ... mais trs infrieur  56 :
;$              ... aber viel weniger als 56:
;+              ... but much less than 56:
;-              (pasapas (let ((perm (permuter2 ph)))           ;ou pressebouton
;-                        (apply #'prins ph)
;-                        (apply #'prins perm)
;-                        (format t "~&Levenshtein Distanz = ~D~%"
;-                          (Levenshtein
;-                            (eval
;-                              (append '(concatenate (quote string)) ph))
;-                            (eval
;-                              (append '(concatenate (quote string)) perm))))))

(defun Levenshtein (str1 str2)      ;chanes de caractres ou listes ou vecteurs
  (let* ((lstr1 (length str1))
         (lstr2 (length str2))
         (d (make-array (list (1+ lstr1) (1+ lstr2)))))
    (do ((i 0 (1+ i))) ((> i lstr1))
      (setf (aref d i 0) i))
    (do ((j 0 (1+ j))) ((> j lstr2))
      (setf (aref d 0 j) j))
    (do ((i 1 (1+ i))) ((> i lstr1))
      (do ((j 1 (1+ j))) ((> j lstr2))
        (setf (aref d i j)
              (min (+ (aref d (1- i) j     )
                      1)
                   (+ (aref d i      (1- j))
                      1)
                   (+ (aref d (1- i) (1- j))
                      (if (equal (elt str1 (1- i)) (elt str2 (1- j))) 0 1))))))
;    (do ((i 0 (1+ i))) ((> i lstr1))       ;impression de la matrice rsultante
;      (format t "~%")
;      (do ((j 0 (1+ j))) ((> j lstr2))
;        (format t "~3D" (aref d i j))))
    (aref d lstr1 lstr2)))                              ;Distance de Levenshtein
