;===============================================================================
;;; 17.2 --> MACRO-CARACTRES : exemple d'application  un codage des hauteurs,
;;;          avec quelques fonctions esquissant un environnement
;$$ 17.2 --> MACRO-ZEICHEN: auf ein Tonhhen-Codierungs-System angewendet, mit
;$$          ein paar Funktionen, die eine Umgebung umreissen         [d..H.B.]
;++ 17.2 --> MACRO-CHARACTERS: Example Applied to a Pitch Coding System, With a
;++          Few Functions Sketching an Environment
;===============================================================================

;;; Codage de HAUTEURs, conversions en MIDINOTEs et frquences, et vice-versa
;$$ Kodierung der Tonhhen (HAUTEURs), Umrechnungen in MIDINOTEn und Frequenzen
;$$ und vice-versa
;++ Pitch (HAUTEURs) Coding, Conversions to MIDINOTEs and Frequencies and Vice
;++ Versa

;;; Dfinitions
;$$ Definitionen
;++ Definitions

;;   Les lments non dfinis formellement ici le seront, par l'exemple, dans
;; les premires lignes du code LISP ci-dessous.
;$   Elemente, die hier nicht ausdrcklich definiert wurden, werden im Beispiel
;$ in den ersten Zeilen des LISP-Codes unten erklrt.
;+   Elements not formally defined here will be presented through the example,
;+ in the first lines of the LISP code below.
;-    HAUTEUR := (NOM . OCTAVE)
;;    NOM     := un lment de la liste NOMS_HAUT -- cf. ci-dessous
;;    OCTAVE  := un nombre entier -- ngatif, zro ou positif
;; Exemple :
;;    (c+ . -3) est une HAUTEUR, dont le NOM est c+, et dont l'OCTAVE est -3
;;   Le nombre de degrs par OCTAVE est dfini par le nombre d'lments de la
;; liste NOMS_HAUT des NOMs (symboles) utiliss pour les dsigner.
;;   La correpondance entre HAUTEURs, MIDINOTEs (:= notes MIDI) et frquences
;; est tablie comme suit :
;;    ((car NOMS_HAUT) . 0) := MIDINOTE numro HAUT_REF := frquence FREQ_REF
;; Par exemple : un environnement comportant 24 degrs ou notes par OCTAVE
;; peut-tre dfini par une liste NOMS_HAUT de 24 symboles (NOMSs) commenant
;; par le NOM a :
;$    NOM     := ein Element der Liste NOMS_HAUT -- vgl. unten
;$    OCTAVE  := ein Integer -- negativ, null oder positiv
;$ Beispiel:
;$    (c+ . 3) ist eine HAUTEUR, mit NOM c+ und OCTAVE -3
;$   Die Anzahl der Schritte pro OCTAVE ist durch die Anzahl der Elemente in der
;$ Liste der NOMs (Symbolen) NOMS_HAUT definiert, die benutzt wird um die 
;$ Schritte zu benennen.
;$   Die Verbindung zwischen HAUTEURS, MIDINOTEs (:= MIDI-Tonhhen) und
;$ Frequenzen ist folgendermaen festgelegt:
;$    ((car NOMS_HAUT) . 0) := MIDINOTE Nummer HAUT_REF := Frequenz FREQ_REF
;$ Beispiel: eine Umgebung mit 24 Schritten oder Noten pro Oktave kann durch
;$ eine NOMS_HAUT-Liste mit 24 Symbolen (NOMs) beginnend mit dem NOM a bestimmt
;$ werden:
;+    NOM     := an element of the NOMS_HAUT list--cf. below
;+    OCTAVE  := an integer number--negative, zero or positive
;+ Example:
;+    (c+ . 3) is a HAUTEUR, of NOM c+ and OCTAVE -3
;+   The number of steps per OCTAVE is defined by the number of elements of the
;+ NOMS_HAUT list of NOMs (symbols) used to designate them.
;+   The correspondence between HAUTEURs, MIDINOTEs (:= MIDI pitches) and
;+ frequencies is established as follows:
;+    ((car NOMS_HAUT) . 0) := MIDINOTE number HAUT_REF := frequency FREQ_REF
;+ For instance: an environment counting 24 steps or notes per OCTAVE can be
;+ defined by a NOMS_HAUT list of 24 symbols (NOMs) starting with the NOM a:
;-    (a a+ a* a% b b+ c c+ c* c% d d+ d* d% e e+ f f+ f* f% g g+ g* g%)
;- Par consquent : / Demnach: / Consequently:
;-
;-    ...
;-    (g% . -3)  :=  HAUT_REF - 49
;-    (a  . -2)  :=  HAUT_REF - 48
;-    ...
;-    (g  . -1)  :=  HAUT_REF - 4
;-    (g+ . -1)  :=  HAUT_REF - 3
;-    (g* . -1)  :=  HAUT_REF - 2
;-    (g% . -1)  :=  HAUT_REF - 1
;-    (a  .  0)  :=  HAUT_REF
;-    (a+ .  0)  :=  HAUT_REF + 1
;-    (a* .  0)  :=  HAUT_REF + 2
;-    (a% .  0)  :=  HAUT_REF + 4
;-    (b  .  0)  :=  HAUT_REF + 5
;-    ...
;-    (a  .  1)  :=  HAUT_REF + 24
;-    ...
;-    (a  .  2)  :=  HAUT_REF + 48
;-    ...
;-
;;   Les OCTAVEs vont donc du premier au dernier lment de la liste NOMS_HAUT.
;;   Pour modifier les NOMs de degrs, le premier NOM dbutant chaque OCTAVE, et
;; HAUT/OCTAV (:= nombre de degrs par OCTAVE), il faut et il suffit de
;; redfinir la liste NOMS_HAUT.
;;   N.B. : pour utiliser le macro-caractre "" ci-dessous, ne pas inclure le
;; caractre "-" ni de chiffres dans les NOMs de NOMS_HAUT -- car ils sont
;; ncessaires au codage des OCTAVEs -- ni le caractre "#" -- qui est dj un
;; macro-caractre important en Common LISP.
;$   Die OCTAVEn gehen daher vom ersten bis zum letzten Element der NOMS_HAUT-
;$ Liste.
;$   Um die NOMs der Schritte, den ersten NOM jeder OCTAVE und HAUT/OCTAV
;$ (:= Anzahl der Schritte pro OCTAVE) zu verndern, muss man nur die NOMS_HAUT-
;$ Liste neu definieren.
;$   N.B.: um das Macrozeichen "" unten einsetzen zu knnen, darf man weder das
;$ "-"-Zeichen, noch Ziffern -- diese werden zur Kodierung der OCTAVEn bentigt
;$ -- oder das "#"-Zeichen -- selbst schon ein wichtiges Common LISP-
;$ Macrozeichen -- in den NOMs der NOMS_HAUT verwenden.
;+   The OCTAVEs thus go from the first to the last element of the NOMS_HAUT
;+ list.
;+   To modify the NOMs of steps, the first NOM starting each OCTAVE, and
;+ HAUT/OCTAV (:= number of steps per OCTAVE) one must only redefine the
;+ NOMS_HAUT list.
;+   N.B.: to use the "" macro-character below, do not include the character
;+ "-" or digits in the NOMs of NOMS_HAUT--because they are required for the
;+ coding of OCTAVEs--nor the "#" character--which is already an important
;+ Common LISP macro-character.

;;   Le macro-caractre "" est dfini pour le codage abrg des HAUTEURs.
;;   Les fonctions suivantes permettent toutes les conversions ncessaires :
;;    hautp <ht>      => numro de MIDINOTE correspondant  la HAUTEUR <ht>, ou
;;                       nil si l'argument <ht> n'est pas une HAUTEUR
;;    midin>haut <mn> => HAUTEUR correspondant au numro de MIDINOTE <mn>, ou
;;                       nil si impossible
;;    haut>freq <ht>  => conversion de HAUTEUR <ht>  frquence correspondante
;;    freq>midin <f>  => conversion de frquence <f>  la plus proche MIDINOTE
;;   En outre, freq>haut et midin>freq compltent ces fonctions de conversion.
;;   Une fonction de transposition est dfinie :
;;    transpoz4 <liste_de_HAUTEURs> <intervalle_chromatique_de_transposition>
;$   Das Macrozeichen "" ist definiert zur abgekrzten Kodierung der HAUTEURs.
;$   Die folgenden Funktionen ermglichen die ntigen Konvertierungen:
;$    hautp <ht>      => MIDINOTE-Nummer, die dem <ht> HAUTEUR entspricht, oder
;$                       nil falls das <ht>-Argument kein HAUTEUR ist
;$    midin>haut <mn> => HAUTER, der der MIDINOTE-Nummer <mn> entspricht, oder 
;$                       nil falls unmglich
;$    haut>freq <ht>  => Konvertierung von HAUTEUR <ht> zur entsprechenden
;$                       Frequenz
;$    freq>midin <f>  => Konvertierung der Frequenz <f> zum nchstgelegenen
;$                       HAUTEUR
;$   Auerdem vervollstndigen freq>haut und midin>freq diese
;$ Konvertierungsfunktionen.
;$   Eine Transpositionsfunktion ist folgendermaen definiert:
;$    transpoz4 <Liste_der_HAUTEURs> <chromatisches_Transpositionsintervall>
;+   The macro-character "" is defined for the abbreviated coding of HAUTEURs.
;+   The following functions allow all necessary conversions:
;+    hautp <ht>      => MIDINOTE number corresponding to the <ht> HAUTEUR, or
;+                       nil if the <ht> argument is not a HAUTEUR
;+    midin>haut <mn> => HAUTEUR corresponding to the <nm> MIDINOTE, or nil if
;+                       impossible
;+    haut>freq <ht>  => conversion from HAUTEUR <ht> to corresponding frequency
;+    freq>midin <f>  => conversion from frequency <f> to the nearest MIDINOTE
;+   Moreover, freq>haut and midin>freq complete these conversion function.
;+   A transposition function is defined:
;+    transpoz4 <list_of_HAUTEURs> <chromatic_transposition_interval>

;;;========= Les trois variables suivantes dfinissent l'environnement =========
;$$=========== Die drei folgenden Variablen definieren die Umgebung ============
;++=========== The Three Following Variables Define the Environment ============

;;   Dans tous les exemples ci-dessous, il s'agit des douze notes tempres
;; "normales", au diapason standard, en notation  l'anglo-saxone. On cre le
;; package spcial MIDIN afin de regrouper  part -- et prserver -- ces
;; variables globales.
;$   Alle Beispiele unten beziehen sich auf die zwlf "normalen" temperierten
;$ Tonhhen, auf den Standard-Kammerton, in angelschsischer Notierung. Ein
;$ spezielles MIDIN-Paket wird erstellt um diese globalen Variablen zu isolieren
;$ -- und zu speichern.
;+   All the examples below concern the twelve "normal" tempered notes, to the
;+ standard tuning pitch, in Anglo-Saxon notation. A special MIDIN package is
;+ created in order to group apart--and preserve-- these global variables.

(defpackage "MIDIN")                   ;majuscules ou midin ncessaires en MCL
                                       ;minuscules "midin" suffisent sous UNIX

(defparameter MIDIN::nm-ht '(          ;dfinition de NOMS_HAUT comportant 12
        a a+                           ;NOMs (symboles)
        b
        c c+
        d d+
        e
        f f+
        g g+
                    ))
(defparameter MIDIN::h-rf  57  )       ;dfinition de HAUT_REF := (a . 0)
(defparameter MIDIN::f-rf 220.0)       ;dfinition de FREQ_REF := (a . 0)

;;;================= Rien ne devrait tre modifi ci-dessous... ================
;$$================= Unterhalb sollte nichts verndert werden... ===============
;++==================== Nothing Should Be Modified Below... ====================

(defparameter MIDIN::h/oc (list-length MIDIN::nm-ht)) ;HAUT/OCTAV sera un entier
(when (not (integerp MIDIN::h-rf))                    ;HAUT_REF doit tre entier
  (error "HAUT_REF : pas un entier." MIDIN::h-rf))

;;   Macro-caractre de codage de HAUTEURs : "". Permet d'crire une HAUTEUR
;; sous la forme abrge <n><o>, o
;$   Makrozeichen zur Kodierung der HAUTEUR: "". Ermglicht ein HAUTEUR in der
;$ Kurzschreibweise <n><o> anzugeben, wobei
;+   HAUTEUR coding macro-character: "". Allows to write a HAUTEUR in the
;+ abbreviated form <n><o>, where
;-    <n> := NOM
;-    <o> := OCTAVE
;; Exemples, pour le NOMS_HAUT chromatique normal de 12 NOMs ci-dessus :
;$ Beispiele fr die normalen 12 NOMs der chromatischen NOMS_HAUT oben:
;+ Examples, for the normal 12 NOMs chromatic NOMS_HAUT above:
;-    'a0   => (A . 0)
;-    'g+-2 => (G+ . -2)
;-    'b2   => (B . 2)
;-    'w2   => Error: macro-caractere "" : W2 : mauvaise HAUTEUR.

(defun tilde-notes (flux char) (declare (ignore char)) ;viter Warning... en MCL
       (prog (skyelu limite long nom octave)
         (multiple-value-setq (skyelu long) (explode (read flux t nil t)))
         (setq limite 0)
         (do ((rez (member (nth limite skyelu) '(- 0 1 2 3 4 5 6 7 8 9))
                   (member (nth limite skyelu) '(- 0 1 2 3 4 5 6 7 8 9))))
             (rez)
           (incf limite)
           (when (> limite long) (go erreur)))
         (when (zerop limite) (go erreur))
         (setq nom (implode (firstn2 skyelu limite)))
         (when (not (member nom MIDIN::nm-ht)) (go erreur))
         (setq octave (implode (lastn2 skyelu (- long limite))))
         (when (not (integerp octave)) (go erreur))
         (return (cons nom octave))
  erreur (error "macro-caractere \"\" : ~A : ~
                 mauvaise HAUTEUR." (implode skyelu))))
(set-macro-character #\ #'tilde-notes)

;;; Fonctions de conversions lmentaires de HAUTEURs
;$$ Elementare HAUTEURs-Konvertierungsfunktionen
;++ Elementary HAUTEURs Conversions Functions

;;    hautp <ht> => numro de MIDINOTE correspondant  la HAUTEUR <ht>, ou nil
;;                  si l'argument <ht> n'est pas une HAUTEUR
;$    hautp <ht> => MIDINOTE-Nummer, die dem HAUTEUR <ht> entspricht, oder nil
;$                  falls das <ht>-Argument kein HAUTEUR ist
;+    hautp <ht> => MIDINOTE number corresponding to the <ht> HAUTEUR, or nil if
;+                  the <ht> argument is not a HAUTEUR
;- Exemple : / Beispiel: / Example:
;-    (hautp 'a0) => 57
;; en vertu des dfinitions ci-dessus.
;$ entsprechend der obigen Definition.
;+ according to the above definitions.

(defun hautp (hauteur)
  (and (consp hauteur)
       (let* ((nom  (member (car hauteur) MIDIN::nm-ht))
              (lng (list-length nom))
              (octv (cdr hauteur)))
         (and nom
              (integerp octv)
              (+ MIDIN::h-rf
                 (if (minusp octv)
                   (- (* MIDIN::h/oc (1+ octv)) lng)
                 (+ (- MIDIN::h/oc lng) (* MIDIN::h/oc octv))))))))

;;    midin>haut <mn> => HAUTEUR correspondant au numro de MIDINOTE <mn>, ou
;;                       nil si impossible
;$    midin>haut <mn> => HAUTEUR, welcher der MIDINOTE <nm> entspricht oder nil
;$                       falls nicht mglich
;+    midin>haut <mn> => HAUTEUR corresponding to the <nm> MIDINOTE, or nil if
;+                       impossible
;- Exemple : / Beispiel: / Example:
;-    (mapcar #'midin>haut '(62 -4 68)) => ((D . 0) NIL (G+ . 0))

(defun midin>haut (mn)
  (and (integerp mn)
       (not (minusp mn))
       (< mn 128)
       (let ((a (- mn MIDIN::h-rf)))
         (cons (nth (mod a MIDIN::h/oc) MIDIN::nm-ht)
               (floor (/ a MIDIN::h/oc))))))

;;    midin>freq <mn> => conversion de MIDINOTE <mn>  frquence correspondante
;$    midin>freq <mn> => Konvertierung der MIDINOTE <mn> zur entsprechenden
;$                       Frequenz
;+    midin>freq <mn> => conversion of MIDINOTE <mn> to corresponding frequency
;- Exemple : / Beispiel: / Example:
;-    (midin>freq 60) => 261.6255653005986
;; ce qui est la frquence du Do central.
;$ was der Frequenz des mittleren C entspricht.
;+ which is the frequency of Middle C.

(defun midin>freq (mn)
  (* MIDIN::f-rf (expt 2. (/ (- mn MIDIN::h-rf) MIDIN::h/oc))))

;;    freq>midin <f> => conversion de frquence <f>  la plus proche MIDINOTE
;$    freq>midin <f> => Konvertierung der Frequenz <f> zur nchsten MIDINOTE
;+    freq>midin <f>  => conversion from frequency <f> to the nearest MIDINOTE
;- Exemple : / Beispiel: / Example:
;-    (freq>midin 261.5) => 60
;; ce qui correspond au Do central.
;$ was dem mittleren C entspricht.
;+ which corresponds to Middle C.

(defun freq>midin (f)
  (+ MIDIN::h-rf (round (* MIDIN::h/oc (log (/ f MIDIN::f-rf) 2.)))))

;;    haut>freq <ht> => conversion de HAUTEUR <ht>  frquence correspondante
;$    haut>freq <ht> => Konvertierung der HAUTEUR <ht> zur entsprechenden
;$                      Frequenz
;+    haut>freq <ht>  => conversion from HAUTEUR <ht> to corresponding frequency
;- Exemple : / Beispiel: / Example:
;-    (haut>freq 'c0) => 261.6255653005986
;; ce qui est la frquence du Do central.
;$ was der Frequenz des mittleren C entspricht.
;+ which is the frequency of Middle C.

(defun haut>freq (h)
  (midin>freq (hautp h)))

;;    freq>haut <f> => conversion de frquence <f>  la plus proche HAUTEUR
;$    freq>haut <f> => Konvertierung der Frequenz <f> zur nchsten HAUTEUR
;+    freq>haut <f> => conversion of frequency <f> to nearest HAUTEUR
;- Exemple : / Beispiel: / Example:
;-    (freq>haut 65.4) => (C . -2)
;; ce qui est la note situe deux octaves sous le Do central.
;$ was der Note zwei Oktaven unter dem mittleren C entspricht.
;+ which is the note located two octaves below Middle C.

(defun freq>haut (f)
  (midin>haut (freq>midin f)))

;;; Utilitaires
;$$ Hilfsfunktionen
;++ Utilities

;;   Fonction de transposition de <n> intervalles chromatiques d'une liste <hts>
;; de HAUTEURs  -- il y a, par dfinition, HAUT/OCTAV intervalles chromatiques
;; par OCTAVE.
;$   Transpositionsfunktion einer HAUTEURs-Liste <hts> um <n> chromatische
;$ Intervalle -- es gibt, laut Definition, HAUT/OCTAV chromatische Intervalle
;$ pro Oktave.
;+   Transposing function of a <hts> list of HAUTEURs by <n> chromatic intervals
;+ --by definition, there are HAUT/OCTAV chromatic intervals per OCTAVE.
;- Exemple : / Beispiel: / Example:
;-    (transpoz4 '(c0 g+-2 f+1 d0 d+-1) 25)
;-        => ((C+ . 2) (A . 1) (G . 3) (D+ . 2) (E . 1))

(defun transpoz4 (hts n)
  (mapcar #'tr-aux hts (ncirclist (list n))))

(defun tr-aux (orig n)
  (midin>haut (+ (hautp orig) n)))

;;   Utilitaires de la fonction tilde-notes ci-dessus, pour la manipulation de
;; symboles par dcomposition de leur print-name. Cf. [Wertz:80-83] et les
;; primitives Le_Lisp explodech et implodech.
;;   Les quatre utilitaires ci-dessous sont une survivance de la premire
;; version du prsent fichier en Le_Lisp. En Common LISP, on pourrait simplement
;; traiter la chane de caractres lue comme une squence, l'explorer au moyen
;; de la primitive elt, et la scinder avec la primitive subseq  -- cf.
;; [CLtL2:392-393].
;$   Hilfsfunktionen fr die tilde-notes-Funktion oben zur Manipulation eines
;$ Symbols durch Zerlegung seines print-name. Vgl. [Wertz:80-83] und die
;$ Le_Lisp-Primitives explodech und implodech.
;$   Die vier Hilfsfunktionen unten sind ein berbleibsel aus der ersten Version
;$ der jetzigen Datei fr Le_Lisp. In Common LISP knte man die Zeichenketten
;$ einfach als Sequenz betrachten, sie mit dem elt-Primitive abtasten und mit
;$ dem subseq-Primitive aufteilen -- Vgl. [CLtL2:392-393].
;+   Utilities for the tilde-notes function above, for the manipulation of
;+ symbols through decomposition of their print-name. Cf. [Wertz:80-83] and the
;+ Le_Lisp primitives explodech and implodech.
;+   The four utilities below are a survival of the first Le_Lisp version of the
;+ present file. In Common LISP, one could simply process the characters read as
;+ a sequence, explore it with the elt primitive, and split it up with the
;+ subseq primitive--cf. [CLtL2:392-393].

;;    Conversion du symbole <symb> => liste de symboles mono-caractres
;$    Umwandlung des Symbols <symb> => Liste aus Einzelzeichen-Symbolen
;+    Conversion of symbol <symb> => list of mono-character symbols
;- Exemple : / Beispiel: / Example:
;-    (explode 'MammaMia) => (M A M M A M I A)

(defun explode (symb)
  (if (symbolp symb)
    (do* ((arg (string symb))
          (lng (length arg))
          (rez () (push (read-from-string
                         (string (elt arg indx)))
                        rez))
          (indx (1- lng) (1- indx)))
         ((minusp indx) (values rez lng)))
    (error "explode : ~A : pas un symbole." symb)))

;;    Conversion de la <liste> de symboles mono-caractres => symbole
;$    Umwandlung der Liste <liste> aus Einzelzeichen-Symbolen => Symbol
;+    Conversion of the <liste> list of mono-character symbols => symbol
;- Exemple : / Beispiel: / Example:
;-    (implode '(P I Z Z A + 1 2)) => PIZZA+12

(defun implode (liste)
  (let ((long (list-length liste)))
    (if (< long 2) (car liste)
    (read-from-string (reduce #'string-appendXX liste)))))

(defun string-appendXX (a z)
  (concatenate 'string (to-string a) (to-string z)))

(defun to-string (arg)
  (cond ((stringp arg) arg)
        ((symbolp arg) (string arg))
        ((integerp arg) (string (elt "0123456789" arg)))
        (t (error "implode : ~A : pas un symbole ou entier." arg))))

;;   Chargement de firstn2 et lastn2 (ped.08), puis de ncirclist (ped.15). Le
;; path common-lisp::*path-ped* est prsum dj dfini dans l'environnement --
;; cf. fichiers ~dlo/.clinit.cl (UNIX) ou .../cl.d/ped.d/+clinit.cl (Mac OS X).
;$   Laden von firstn2 und lastn2 (ped.08), danach von ncirclist (ped.15). Es
;$ wird vorausgesetzt, dass der Pfad common-lisp::*path-ped*  schon in der
;$ Umgebung definiert wurde -- Vgl. die Dateien ~dlo/.clinit.cl (UNIX) oder
;$ .../cl.d/ped.d/+clinit.cl (Mac OS X).
;+   Loading of firstn2 and lastn2 (ped.08), then of ncirclist (ped.15). The
;+ common-lisp::*path-ped* path is presumed already defined in the environment--
;+ cf. files ~dlo/.clinit.cl (UNIX) or .../cl.d/ped.d/+clinit.cl (Mac OS X).

(when (and (load (merge-pathnames "ped.08.lisp" common-lisp::*PathLocPed*))
           (load (merge-pathnames "ped.15.lisp" common-lisp::*PathLocPed*)))
  (format t "~&;Charges : firstn2 et lastn2 (ped.08), ncirclist (ped.15).~%"))

;;   Deux motifs, pour l'exemple...
;$   Zwei Motive als Beispiele...
;+   Two motifs, as examples...

(format t ";mt1 := ~A~%" (setq mt1 '(c-1 f+1 g+-2 a0 c+-3 g0 f2)))
(format t ";mt2 := ~A~%" (setq mt2 '(c0 g+-2 f+1 d0 d+-1)))
