;;;=============================================================================
;;; 18.2 - Annexe C --> FILTRES LABORS (pattern matching) : liaison d'lments
;;;                     d'assertions    des  variables  de  filtres,  calcul de
;;;                     contraintes et d'lments de filtres
;;;
;;;               b. -> FONCTIONS AUXILIAIRES DE fX
;;;=============================================================================

;;   Utiles pour la mise au point.

(defparameter bugbug nil)                               ;t => DEBUG ; nil => non
(defun bug () (setq bugbug (not bugbug)))

;;   Fonctions effectuant les valuations commandes par les caractres spciaux
;; ^ et @.
;;   On ne fait aucune vrification  priori de la validit des arguments
;; impliqus dans les valuations commandes par ces caractres. Les erreurs
;; LISP standard sont dclenches par des incompatibilits  ce niveau.

(defun chapeau (expr obj ali)                                                 ;^
  (progv (mapcar #'car ali) (mapcar #'cdr ali)
    (when bugbug (dbg expr chapeau)
                 (dbg ali))
    (eval (subst `(quote ,obj) '& expr))))

(defun escargot (expr ali)                                                    ;@
  (progv (mapcar #'car ali) (mapcar #'cdr ali)
    (when bugbug (dbg expr escargot))
    (eval expr)))

;;   Fonctions grant les squences d'lments d'assertions commandes par les
;; caractres spciaux $ et ~.
;; N.B. : *A-LISTE* est dclare de type special par f3 dans ped.18.2-AnxC.c.
;; Cette variable spciale contiendra les variables de filtre et leurs liaisons,
;; en cours de filtrage.

(defun SEq1n (fi as)                                  ;$ sans variable de filtre
  (when bugbug (dbg fi SEq1n)
               (dbg as SEq1n))
  (cond
   (as (pop as)
       (if (fX (cdr fi) as) t
           (SEq1n fi as)))
   (t nil)))

(defun SEqv1n (va fi as)                              ;$ avec variable de filtre
  (declare (special *A-LISTE*))
  (when bugbug (dbg fi SEqv1n)
               (dbg as SEqv1n)
               (dbg va SEqv1n))
  (cond
   (as (Anconc (assoc va *A-LISTE*) (pop as))
       (if (fX (cdr fi) as) t
           (SEqv1n va fi as)))
   (t nil)))

(defun SEq0n (fi as)                                  ;~ sans variable de filtre
  (when bugbug (dbg fi SEq0n)
               (dbg as SEq0n))
  (cond
   (as (if (fX (cdr fi) as) t
           (progn (pop as)
                  (SEq0n fi as))))
   (t (fX (cdr fi) as))))

(defun SEqv0n (va fi as)                              ;~ avec variable de filtre
  (declare (special *A-LISTE*))
  (when bugbug (dbg fi SEqv0n)
               (dbg as SEqv0n)
               (dbg va SEqv0n)
               (dbg *A-LISTE* SEqv0n))
  (cond
   (as (if (fX (cdr fi) as) t
           (progn (Anconc (assoc va *A-LISTE*) (pop as))
                  (SEqv0n va fi as))))
   (t (fX (cdr fi) as))))

;;   Fonctions grant les listes d'alternatives de dfinition d'un lment de
;; filtre, commandes par le caractre spcial ?.  La fonction ORx1 permet,
;; parmi ces alternatives, d'insrer ventuellement des squences de plusieurs
;; lments de filtre.

(defun ORx1 (li fi as)                                     ;?_ : cas particulier
  (when bugbug (dbg li ORx1) (dbg fi ORx1) (dbg as ORx1))
  (let ((prem))
    (cond
     ((endp li) nil)
     ((fX (append (if (listep (setq prem (car li))) prem (list prem))
                  (cdr fi))
          as))
     (t (ORx1 (cdr li) fi as)))))

(defun ORx1n (li fi as)                              ;?$ sans variable de filtre
  (when bugbug (dbg li ORx1n) (dbg fi ORx1n) (dbg as ORx1n))
  (when (alternative li (list (pop as)))
    (or (fX (cdr fi) as)
        (ORx1n li fi as))))

(defun ORxv1n (va li fi as)                          ;?$ avec variable de filtre
  (declare (special *A-LISTE*))
  (when bugbug (dbg li ORxv1n) (dbg fi ORxv1n) (dbg as ORxv1n))
  (let ((prem (pop as)))
    (Anconc (assoc va *A-LISTE*) prem)
    (when (alternative li (list prem))
      (or (fX (cdr fi) as)
          (ORxv1n va li fi as)))))

(defun ORx01 (li fi as)                              ;?% sans variable de filtre
  (when bugbug (dbg li ORx01) (dbg fi ORx01) (dbg as ORx01))
  (cond
   ((fX (cdr fi) as))
   ((alternative li (list (pop as)))
    (fX (cdr fi) as))
   (t nil)))

(defun ORxv01 (va li fi as)                          ;?% avec variable de filtre
  (declare (special *A-LISTE*))
  (when bugbug (dbg li ORxv01) (dbg fi ORxv01) (dbg as ORxv01))
  (cond
   ((fX (cdr fi) as))
   (t (Anconc (assoc va *A-LISTE*) (car as))
      (when (alternative li (list (pop as)))
        (fX (cdr fi) as)))))

(defun ORx0n (li fi as)                              ;?~ sans variable de filtre
  (when bugbug (dbg li ORx0n) (dbg fi ORx0n) (dbg as ORx0n))
  (cond
   ((fX (cdr fi) as))
   ((alternative li (list (pop as)))
    (or (fX (cdr fi) as)
        (ORx0n li fi as)))
   (t nil)))

(defun ORxv0n (va li fi as)                          ;?~ avec variable de filtre
  (declare (special *A-LISTE*))
  (when bugbug (dbg li ORxv0n) (dbg fi ORxv0n) (dbg as ORxv0n))
  (cond
   ((fX (cdr fi) as))
   (t (Anconc (assoc va *A-LISTE*) (car as))
      (when (alternative li (list (pop as)))
        (or (fX (cdr fi) as)
            (ORxv0n va li fi as))))))

;;   Fonctions auxiliaires des ORx... ci-dessus.

(defun alternative (lis boudas)
  (when bugbug (dbg lis alternative) (dbg boudas alternative))
  (cond
   ((endp lis) nil)
   ((fX (list (pop lis)) boudas))
   (t (alternative lis boudas))))

(defun listep (arg)                                     ;dtecte une VRAIE liste
  (and (listp arg) (listp (cdr arg))))

;;   Fonctions destines  la gestion de la A-liste *A-LISTE*, contenant les
;; variables de filtre et leurs liaisons.

;;   La macro-fonction suivante sert  dtruire, si elles existent dj dans la
;; A-liste *A-LISTE*, une variable de filtre et sa liaison.
;;   Elle est utilise par fX chaque fois qu'elle rencontre une variable de
;; filtre, afin de prendre en compte ses propres appels rcursifs  elle-mme,
;; qui impliquent de re-crer sans cesse des liaisons pour des variables qui ont
;; ventuellement dj t lies  d'autres valeurs lors de tentatives
;; antrieures -- et donc infructueuses.

(defmacro delier (var)
  `(let ((x (assoc ,var *A-LISTE*)))
     (when x (setq *A-LISTE* (remove x *A-LISTE* :test #'equal)))))

;;   Lorsqu'elle a dli une variable de filtre, la fonction fX la (re)cre dans
;; la A-liste *A-LISTE*, d'abord lie  la valeur de la variable spciale
;; *rien*. La valeur par dfaut de cette variable spciale *rien* est *rien*,
;; mais peut-tre modifie au moyen du mot-clef :rien de la fonction f3 -- cf.
;; fichier ped.18.2-AnxC.c.
;;   Anconc est ensuite utilise pour initialiser la variable en crant une
;; premire liaison vritable avec un lment d'assertion, ou pour poursuivre la
;; liste des lments d'assertion devant tre lis  la variable, dans le cas o
;; celle-ci est dj en cours de liaison dans la A-liste *A-LISTE*.

(defun Anconc (varliee nqueue)
  (declare (special *rien*))
  (let ((z1 (cdr varliee))) (cond
       ((eq *rien* z1)
        (nconc varliee
               (if (atom nqueue) nqueue
                   (list nqueue))))
       (t (nconc varliee
                 (if (atom z1) (cons z1 (list nqueue))
                     (list nqueue)))))))

;;   Xassoc permet la consultation de la A-liste *A-LISTE* en rendant la valeur
;; lie  une variable de filtre. Les primitives (cdr (assoc...)) ne suffisent
;; pas en l'occurrence, car elles ne permettent pas de distinguer l'absence
;; d'une variable dans la A-liste d'une liaison de cette variable  la valeur
;; particulire nil. Ici, l'absence de la variable est signale par la valeur
;; spcifique *undef*. Par ailleurs, la valeur de la variable spciale *rien* a
;; la signification dcrite ci-dessous.
;;   La combinaison des appels de fX, de Anconc et de Xassoc permet de
;; distinguer tous les cas de liaisons possibles, selon la valeur rendue par
;; l'appel
;;    Xassoc <vari> <A-liste> =>
;;        *undef* => la <vari> n'est pas dans la <A-liste>, i.e. : n'existe pas
;;                   -- du moins au moment de l'appel
;;        (symbol-value '*rien*)
;;                => la <vari> existe, mais elle est n'est lie  rien, i.e. :
;;                    zro lments de l'assertion -- du moins au moment de
;;                   l'appel
;;        <autre_valeur> => la <vari> existe, et est lie  cette <autre_valeur>
;;                          -- y compris ventuellement nil

(defun Xassoc (vari a-liste)
  (labels ((Xssc (ali) (cond
       ((endp ali) '*undef*)
       ((eq vari (caar ali)) (cdar ali))
       (t (Xssc (cdr ali))))))
    (Xssc a-liste)))
