;===============================================================================
;;; 19.1a --> PROGRAMMATION OBJET EN CLOS : mthodes gnriques, primitive
;;;           defmethod
;$$ 19.1a --> OBJEKTORIENTIERTE PROGRAMMIERUNG MIT CLOS: Generische Methoden,
;$$           defmethod-Primitive                                     [d..H.B.]
;++ 19.1a --> CLOS OBJECT PROGRAMMING: Generic Methods, defmethod Primitive
;===============================================================================

;;   Exemple de mthode gnrique, remplaant, en Common Lisp Object System
;; (CLOS), le mcanisme plus courant d'envoi de messages  des objets. Cf.
;; [Masini et al.:190].
;$   Beispiel einer generischen Methode, die im Common Lisp Object System (CLOS)
;$ den gebruchlicheren Mechanismus des Sendens von Nachrichten an Objekte
;$ ersetzt. Vgl. [Masini et al.:190].
;+   Example of generic method replacing, in Common LISP Object System (CLOS),
;+ the more usual mechanism of messages sent to objects. Cf. [Masini et al.:
;+ 190].

;;   Quatre dfinitions d'une mthode plus, composant une mthode gnrique :
;;    plus <a1> &rest <reste>
;;        => a. addition      des  arguments, si <a1> est un nombre, ou
;;           b. concatnation  "   "   "   "   "   "   "   " string, ou
;;           c. "union"        "   "   "   "   "   "   "   " cons,   ou
;;           d. erreur dans les autres cas
;$   Vier Definitionen einer plus-Methode, die eine generische Methode bilden: 
;$    plus <a1> &rest <reste>
;$        => a. Addition      der Argumente, falls <a1> eine Zahl      ist, oder
;$           b. Verkettung     "   "   "   "   "   "  " " Zeichenkette ist, oder
;$           c. "Vereinigung"  "   "   "   "   "   "  " ein  cons      ist, oder
;$           d. Fehler in anderen Fllen
;+   Four definitions of a plus method, constituting a generic method:
;+    plus <a1> &rest <reste>
;+        => a. addition      of arguments, if <a1> is a number, or
;+        => b. concatenation "   "   "   "   "   "   "  string, or
;+        => c. "union"       "   "   "   "   "   "   "  cons,   or
;+        => d. error in other cases

(defmethod plus ((a1 number) &rest reste)                                    ;a.
  (apply #'+ (cons a1 reste)))
(defmethod plus ((a1 string) &rest reste)                                    ;b.
  (apply #'concatenate (cons 'string (cons a1 reste))))
(defmethod plus ((a1 cons) &rest reste)                                      ;c.
  (apply #'union-ensemble (cons a1 reste)))
(defmethod plus ((a1 t) &rest reste) (declare (ignore reste))                ;d.
  (error "J'connais pas ~A, M'sieu' !!!~%" a1))

;;   En l'occurrence, bien entendu, une fonction normale, du genre
;$   Im vorliegenden Fall wrde natrlich auch eine normale Funktion wie
;+   In the present case, of course, a normal function such as
;-    (defun plus (a1 ...)
;-      (cond ((numberp a1) ... etc. )                                       ;a.
;-            ((stringp a1) ... etc. )                                       ;b.
;-            ((consp   a1) ... etc. )                                       ;c.
;-            (t (error     ... etc.))))                                     ;d.
;; donnerait les mmes rsultats.
;$ zum selben Ergebnis fhren.
;+ would lead to the same results.

;;   La fonction suivante, pour l'"union" de listes, peut tre compare  des
;; exemples du fichier ped.11.2b : les fonctions inter (intersection) et
;; antidbl (anti-doublons).
;$   Die folgende Funktion, fr die "Vereinigung" von Listen, ist vergleichbar
;$ mit den Beispielen in Datei ped.11.2b: Funktionen inter (Schnittmenge) und
;$ antidbl (Anti-Duplikate).
;+   The following function, for the "union" of lists, can be compared to
;+ examples of file ped.11.2b: functions inter (intersection) and antidbl
;+ (suppression of duplicates).

(defun union-ensemble (&rest tout)
  (remove-duplicates (apply #'append tout) :test #'equal :from-end t))

;;   Exemples d'appels de la mthode gnrique plus : la mthode concrtement
;; value, choisie parmi les quatre dfinitions connues, est chaque fois
;; dtermine par le type du premier argument d'appel <a1>.
;$   Beispiele fr Aufrufe der generischen Methode plus: die im Endeffekt
;$ ausgefhrte Methode wird jedes Mal anhand des Typs des ersten Aufrufarguments
;$ <a1> aus den vier bekannten Definitionen ausgewhlt.
;+   Examples of calls to generic method plus: the method concretely evaluated,
;+ chosen among the four known definitions, is each time determined by the type
;+ of the first calling argument <a1>.

(dolist (lordre '(

      (plus 3 4.4 3/17 pi)                                                   ;a.
      (plus "Cf. " "[Masi" "ni et " "al.:190" " sq." "]")                    ;b.
      (plus '(a b c) '(c d j x a) '(h f e w a b b b c c c) '(x p a c b f))   ;c.
      (plus 'zut 45.8 "abf")                                                 ;d.

            ))
  (format t "~%~A" lordre)
  (format t "~%~4T=> ~A~%" (eval lordre)))
