;===============================================================================
;;; 19.2a --> PROGRAMMATION  OBJET  EN  CLOS  : exemple classique du robot
;;;           manipulateur -- version simple, cf. [Abdulrab:126-129]
;$$ 19.2a --> OBJEKTORIENTIERTE PROGRAMIERUNG MIT CLOS: Klassisches
;$$           Robotermanipulator-Beispiel -- Einfache Version, Vgl.
;$$           [Abdulrab:126-129]                                      [d..H.B.]
;++ 19.2a --> CLOS OBJECT PROGRAMMING: Classical Manipulating Robot Example--
;++           Simple Version, cf. [Abdulrab:126-129]
;===============================================================================

;;; Classes
;$$ Klassen
;++ Classes

(defclass objet
  () 
  ((nom   :initarg :nom))
  (:default-initargs :nom nil))

(defclass robot
  (objet)
  ())

(defclass table
  (objet)
  ())

(defclass objet-a-deplacer
  (objet)
  ((sur   :initarg :sur  )
   (sous  :initarg :sous )
   (ename :initarg :ename)
   (eposi :initarg :eposi))
  (:default-initargs :sur            nil
                     :sous           nil
                     :ename        "???"                              ;interface
                     :eposi '*nullepart*))                            ;interface

;;; Cration d'instances pour l'exemple : *robot* et *table*
;$$ Erzeugen von Instanzen fr das Beispiel: *robot* und *table*
;++ Creation of Instances for the Example: *robot* and *table*

(defvar *robot* (make-instance 'robot :nom '*robot*))
(defvar *table* (make-instance 'table :nom '*table*))

;;; Mthode principale
;$$ Hauptmethode
;++ Main Method

(defmethod pose-sur ((act robot) (obj objet) (sup objet))
  (cond ((eq obj sup)
          (format t "~&~2TEh ! Je ne peux pas poser ~A sur lui-meme.~%"
                    (slot-value sup 'nom)))
        ((equal 'table (type-of obj))
          (format t "~&~2TJe ne peux pas poser une table sur ~A~%"
                    (slot-value sup 'nom)))
        ((equal 'table (type-of sup))
         (if (equal sup (slot-value obj 'sur))            ;clause supplmentaire
             (format t "~&~2T~A est deja sur ~A.~%"
                    (slot-value obj 'nom)
                    (slot-value sup 'nom))
          (sur-table obj)))
        ((non-place? sup)
          (format t "~&~2TUne chose a la fois : ~
                    je ne vois pas ~A sur la table.~%"
                    (slot-value sup 'nom)))
        ((equal sup (slot-value obj 'sur))                ;clause supplmentaire
          (format t "~&~2T~A est deja sur ~A.~%"
                    (slot-value obj 'nom)
                    (slot-value sup 'nom)))
        (t (libere obj)
           (libere sup)
           (bouge obj)
           (ajoute-sur obj sup)))
  (format t "~&~4TA vot' se'vice M'sieu...~%"))

;;   Macro-fonction interface pour la mthode prcdente, plus proche d'un
;; langage naturel. Elle ne peut, cependant, s'adresser qu'au seul <robot>
;; *robot*. Elle permet d'abrger la rdaction des ordres :
;;    (pose <objet> sur <support>) => (pose-sur *robot* <objet> <support>)
;; N.B. : le second argment n'est pas utilis. Il permet seulement de donner 
;; la commande l'allure d'une phrase "normale".
;$   Schnittstellen-Makrofunktion fr die vorangegangene Methode, hnlich
;$ natrlicher Sprache. Jedoch kann sie nur den *robot* <robot> ansprechen.
;$ Sie erlaubt die Abkrzung der Befehle:
;$    (pose <objet> auf <support>) => (pose-sur *robot* <objet> <support>)
;$ N.B.: das zweite Argument wird nicht verwendet. Es macht die Befehle nur
;$ "normalen" Stzen hnlicher. 
;+   Interface macro-function for the preceding method, similar to a natural
;+ language. However, it can only address the *robot* <robot>. It allows to
;+ abreviate the orders:
;+    (pose <objet> on <support>) => (pose-sur *robot* <objet> <support>)
;+ N.B.: the second argument is not used. It only makes the orders more similar
;+ to "normal" sentences.

(defmacro pose (depl x supp)
  (declare (ignore x))
  `(pose-sur *robot* ,depl ,supp))

;;; Chargement du fichier grant les interfaces
;$$ Laden der Datei, die die Schnittstellen verwaltet
;++ Loading of the File Controlling the Interfaces

(load (merge-pathnames "ped.19.2x.ecran.lisp" common-lisp::*path-ped*))
(format t "~&;  Chargement de ped.19.2x.ecran.~%")

;;; Autres mthodes
;$$ Andere Methoden
;++ Other Methods

(defmethod sur-table ((obj objet-a-deplacer))
  (libere obj)
  (bouge obj)
  (ajoute-sur obj *table*))

(defmethod libere ((obj objet-a-deplacer))
  (let ((sub (slot-value obj 'sous)))
    (and sub (sur-table sub))))

(defmethod bouge ((obj objet-a-deplacer))
  (let ((sup (slot-value obj 'sur)))
    (or    (non-place? obj)
        (equal 'table (type-of sup))
        (reinitialize-instance sup :sous nil))
    (reinitialize-instance obj :sur nil)
    (JePrends obj)))

(defmethod ajoute-sur ((obj objet-a-deplacer) (sup objet))
  (reinitialize-instance obj :sur sup)
  (or (equal 'table (type-of sup))
      (reinitialize-instance sup :sous obj))
  (JePose obj sup))

(defmethod non-place? ((obj objet-a-deplacer))
  (not (slot-value obj 'sur)))

;;; Cration d'instances d'objets pour l'exemple : *objet0*  *objet9*
;$$ Erstellen einer Instanz fr das Beispiel: *objet0* bis *objet9*
;++ Creation of Instances for the Example: *objet0* to *objet9*

(dotimes (i 10)
  (let ((symbo (read-from-string (format nil "*objet~A*" i))))
    (set symbo (make-instance 'objet-a-deplacer :nom   symbo
                                                :ename (format nil "o~A " i)))))

;;; Sance automatique de manipulations d'<objets> -- cf. ped.19.2a.seance
;$$ Automatische Sitzung mit <objects> Manipulationen -- Vgl. ped.19.2a.seance
;++ Automatic Session of <objets> Manipulations--cf. ped.19.2a.seance

;;   ...  Sance <1> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;$   ... Sitzung <1> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;+   ... Session <1> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...

;;   Fonction utilitaire, pour connatre la situation d'un objet.
;$   Hilfsfunktion um die Lage eines Objekts zu erfahren.
;+   Utility function, to state the situation of an object.
    
(defun ou-est? (obj)
  (let ((nommm (slot-value obj 'nom))
        (desus (slot-value obj 'sur))
        (desou (slot-value obj 'sous)))
    (cond ((null desus)
            (format t "~A n'est pas positionne.~%" nommm))
          (t (format t "~A est sur ~A"  nommm (slot-value desus 'nom))
             (if desou (format t ", et sous ~A.~%" (slot-value desou 'nom))
             (format t ".~%"))))))
