;===============================================================================
;;; 19.2b --> PROGRAMMATION  OBJET  EN  CLOS : exemple classique du robot
;;;           manipulateur -- version plus complexe, cf. [Abdulrab:129-131]
;$$ 19.2b --> OBJEKTORIENTIERTE PROGRAMIERUNG MIT CLOS: Klassisches
;$$           Robotermanipulator-Beispiel Komplexere Version, Vgl.
;$$           [Abdulrab:129-131]                                      [d..H.B.]
;++ 19.2b --> CLOS OBJECT PROGRAMMING: Classical Manipulating Robot Example--
;++           More Complex Version, cf. [Abdulrab:129-131]
;===============================================================================

;;   Le prsent fichier complte ped.19.2a, afin d'y adjoindre des capacits
;; supplmentaires.
;$   Die aktuelle Datei vervollstndigt ped.19.2a um zustzliche Fhigkeiten
;$ hinzuzufgen.

;+   The present file completes ped.19.2a in order to integrate additional
;+ capabilities.

(defclass plaque
    (objet-a-deplacer)
    ())

(defclass livre
    (objet-a-deplacer)
    ())

(defmethod sur-table ((obj plaque))
    (bouge obj)
    (ajoute-sur obj *table*))

(defmethod bouge :after ((obj plaque))                        ;dmon after bouge
    (let ((ya (elements-au-dessus obj)))
      (when ya (BougeAussi ya))))
                                        
(defmethod elements-au-dessus ((obj objet-a-deplacer))        ;mthode rcursive
    (let ((sub (slot-value obj 'sous)))
    (if (not sub) ()
    (cons sub (elements-au-dessus sub)))))

(defmethod ajoute-sur :after ((obj plaque) (sup objet))  ;dmon after ajoute-sur
    (let ((ya (elements-au-dessus obj)))
      (when ya (PoseAussi ya))))

(defmethod libere ((obj livre))
    (dolist (element (reverse (elements-au-dessus obj)))
        (bouge element)                        ;<- manque chez Abdulrab : bug...
        (ajoute-sur element *table*)))

;;; Sance automatique de manipulations d'<objets> -- cf. ped.19.2b.seance
;$$ Automatische Sitzung mit <objects>-Manipulationen -- Vgl. ped.19.2b.seance
;++ Automatic Session of <objets> Manipulations--cf. ped.19.2b.seance

;;   Le comportement de *robot* n'est pas modifi.
;;   N.B. : on part (ventuellement) de la situation de la fin de la Sance <1>
;; de manipulations, telle qu'excute  partir du fichier ped.19.2a.seance.
;$   Das Verhalten des *robot* wird nicht verndert.
;$   N. B.: man startet (mglicherweise) mit der Lage am Ende der Sitzung <1>
;$ der Manipulationen, wie sie von Datei ped.19.2a.seance ausgefhrt wurden.
;+   The behavior of *robot* is not modified.
;+   N.B.: one (eventually) starts from the situation of the end of Session <1>
;+ of Manipulations, as performed from file ped.19.2a.seance.

;;   ...  Sance <2> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;$   ... Sitzung <2> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;+   ... Session <2> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...

;;; Cration d'instances de <livres> pour l'exemple : *livre0*  *livre9*
;$$ Erstellen von Instanzen des <livres> fr das Beispiel: *livre0* bis *livre9*
;++ Creation of Instances of <livres> for the Example: *livre0* to *livre9*

(dotimes (i 10)
  (let ((symbo (read-from-string (format nil "*livre~A*" i))))
    (set symbo (make-instance 'objet-a-deplacer :nom   symbo
                                                :ename (format nil "L~A " i)))))

;;; Sance automatique de manipulations de <livres> -- cf. ped.19.2b.seance
;$$ Automatische Sitzung mit <livres> Manipulationen -- Vgl. ped.19.2b.seance
;++ Automatic Session of <livres> Manipulations--cf. ped.19.2b.seance

;;   Tout ce qui tait acquis pour des <objets> fonctionne maintenant aussi bien
;; pour des <livres>.
;$   Alle Funktionen, die fr <objets> erarbeitet wurde, funktionieren jetzt
;$ auch fr <livres>.
;+   All functions acquired for <objets> now also work for <livres> as well.

;;   ...  Sance <3> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;$   ... Sitzung <3> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;+   ... Session <3> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...

;;; Cration d'instances de <plaques> pour l'exemple : *plaqueA* et *plaqueB*
;$$ Erstellen von Instanzen der <plaques> fr das Beispiel: *plaqueA* und
;$$ *plaqueB*
;++ Creation of Instances of <plaques> for the Example: *plaqueA* and *plaqueB*

(defvar *plaqueA* (make-instance 'plaque :nom    '*plaqueA*
                                          :ename "--A"))
(defvar *plaqueB* (make-instance 'plaque :nom    '*plaqueB*
                                          :ename "--B"))

;;; Sance automatique de manipulations de <livres> et de <plaques> -- cf.
;;; ped.19.2b.seance
;$$ Automatische Sitzung mit <livres> und <plaques>-Manipulationen -- Vgl.
;$$ ped.19.2b.seance
;++ Automatic Session of <livres> and <plaques> Manipulations--cf.
;++ ped.19.2b.seance

;;   Tout ce qui tait acquis pour des <objets> et pour des <livres> est
;; complt par le nouveau mode de manipulation des <plaques>. 
;;   N.B. : on part de la situation de la fin de la Sance <3> ci-dessus.
;$   Alle fr <objets> und <livres> erarbeiteten Funktionen werden nun durch
;$ die neuen Manipulationsarten fr <plaques> ergnzt.
;$   N. B.: man geht von der Situation am Ende der Sitzung <3> oben aus.
;+   All functions acquired for <objets> and <livres> are completed by the
;+ new manipulation mode for <plaques>.
;+   N.B.: one starts from the situation of the end of Session <3> above.

;;   ...  Sance <4> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;$   ... Sitzung <4> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;+   ... Session <4> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...

;;; Sance automatique de manipulations d'<objets>, de <livres> et de <plaques>
;;; -- cf. ped.19.2b.seance
;$$ Automatische Sitzung mit <objets>, <livres> und <plaques> Manipulationen --
;$$ Vgl. ped.19.2b.seance
;++ Automatic Session of <objets>, <livres> and <plaques> Manipulations--cf.
;++ ped.19.2b.seance

;;   *Robot* est ainsi capable de manipuler indiffremment <objets>, <livres> et
;; <plaques> dans un mme environnement. 
;;   N.B. : on part de la situation de la fin de la Sance <4> ci-dessus.
;$   *Robot* kann nun gleichgltig  <objets>, <livres> und <plaques> in
;$ derselben Umgebung manipulieren.
;$   N. B.: man geht von der Situation am Ende der Sitzung <4> oben aus.
;+   *Robot* is thus capable of manipulating indifferently <objets>, <livres>
;+ and <plaques> in a same environment.
;+   N.B.: one starts from the situation of the end of Session <4> above.

;;   ...  Sance <5> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;$   ... Sitzung <5> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
;+   ... Session <5> ... ... ... ... ... ... ... ... ... ... ... ... ... ... ...
