;===============================================================================
;;; 17.1 --> MACRO-FONCTIONS : primitive defmacro -- style anti-quote
;$$ 17.1 --> MAKRO-FUNKTIONEN: defmacro-Primitive -- Backquote-Schreibweise
;$$                                                                   [d..H.B.]
;++ 17.1 --> MACRO-FUNCTIONS: defmacro Primitive--Back-Quote Style
;===============================================================================

(defmacro moins1 (n)
  (list '1- n))

(defmacro incr1 (var &optional (val 1))
  (list 'setq var (list '+ var val)))

(defmacro if-ll (test prem &rest reste)
  (list 'if test prem (cons 'progn reste)))

;;   Pour obtenir la documentation-string d'une fonction. Cette utilitaire a
;; dj t dfinie dans le fichier ped.08. Appliquer, par exemple,  firstn2 et
;; lastn2, du mme fichier.
;$   Zum Erhalten des documentation-string einer Funktion. Diese Hilfsfunktion
;$ wurde bereits in Datei ped.08 definiert. Wende sie, z. B., auf firstn2 und
;$ lastn2 in derselben Datei an.
;+   To obtain the documentation string of a function. This utility has already
;+ been defined in file ped.08. Apply, for instance, to firstn2 and lastn2, from
;+ the same file.

(defmacro dof2 (nom)
  (list 'documentation (list 'quote nom) '(quote function)))

;;   Exemple de [Wertz:195].
;$   Beispiel aus [Wertz:195].
;+   Example from [Wertz:195].

(defun 4eme1 (l)
    (cadddr l))

(defmacro 4eme2 (l)
    (list 'cadddr l))

;;   Pour tester la rapidit d'excution des deux fonctions ci-dessus : faire
;$   Um die Ablaufgeschwindigkeit der beiden obigen Funktionen zu messen:
;+   To test the execution speed of the two above functions: do
;-    (time (test<N> a))
;; En tous cas, les rsultats dpendent du systme d'exploitation, de la machine
;; et du Common LISP utiliss...
;$ In jedem Fall hngt das Ergebnis vom Betriebssystem, dem Computer und der
;$ Common LISP-Implementation ab...
;+ In all cases, the results depend on the operating system, the computer and
;+ the Common LISP implementation...

(defun test0 (l)                        ;plus rapide UNIX, MCL et LW : appel
    (dotimes (zut 1000000 'fini)        ;d'une seule primitive
        (cadddr l)))
(defun test1 (l)                        ;moyen UNIX et LW : appel de fonction
    (dotimes (zut 1000000 'fini)        ;appelant la primitive -- le + lent MCL
        (4eme1 l)))
(defun test2 (l)                        ;plus lent UNIX et LW : macro-expansion
    (dotimes (zut 1000000 'fini)        ;suivie d'valuation -- aussi rapide que
        (4eme2 l)))                     ;test0 en MCL

;;   Mmes exemples que ci-dessus, redfinis en style anti-quote : meilleure
;; visibilit de la macro-expansion, mais allure parfois absconse...
;$   Dasselbe Beispiel wie oben, neudefiniert in Backquote-Schreibweise: bessere
;$ Sichtbarkeit der Makro-Erweiterung, aber manchmal schwer verstndlich...
;+   Same examples as above, redefined in back-quote style: better visibility of
;+ the macro-expansion, but sometimes obscure...

(defmacro min1 (n)
  `(1- ,n))

(defmacro incr2 (var &optional (val 1))
  `(setq ,var (+ ,var ,val)))

(defmacro if-leli (test prem &rest reste)
  `(if ,test ,prem ,(cons 'progn reste)))

(defmacro 4eme3 (l)
  `(cadddr ,l))

(defmacro dof3 (nom)
  `(documentation (quote ,nom) 'function))

;;   Diverses macro-fonctions utilitaires, cf. fichiers cl.d/+clinit.cl (MCL ou
;; LispWorks) ou ped.17.2a.clinit ci-contre (UNIX) :
;;    - continu, pasapas, iter, dbg et dof pour tests et debugging
;;    - sh, vi et v pour des interactions avec UNIX
;; Dans ces exemples, l'avantage principal de macro-fonctions est d'amliorer le
;; confort de l'utilisateur, en abrgeant les commandes et en dispensant d'en
;; quoter les arguments.
;$   Verschiedene Hilfs-Makrofunktionen, Vgl. cl.d/+clinit.cl (MCL oder
;$ LispWorks) oder die anschlieenden ped.17.2a.clinit?Dateien (UNIX):
;$    - continu, pasapas, iter, dbg und dof fr Tests und Debugging
;$    - sh, vi und v zur Interaktion mit UNIX
;$ In diesen Beispielen besteht der Hauptvorteil einer Makrofuktion in der
;$ Verbesserung des Bedienkomforts durch das Krzen der Befehle und Ersparen des
;$ Zitierens ihrer Argumente.
;+   Various utility macro-functions, cf. cl.d/+clinit.cl (MCL or LispWorks) or
;+ adjoining ped.17.2a.clinit files (UNIX):
;+    - continu, pasapas, iter, dbg and dof for tests and debugging
;+    - sh, vi and v for interactions with UNIX
;+ In these examples, the main advantage of macro-functions consists in
;+ improving user comfort by shortening the commands and sparing the quotation
;+ of their arguments.

;;   Exemple de macro-fonction rcursive : la fonction suivante adapte des
;; formes comportant un nombre quelconque d'arguments pour les prsenter  une
;; primitive n'acceptant que deux arguments. Par exemple :
;$   Beispiel einer rekursiven Makrofunktion: die folgende Funktion bearbeitet
;$ Ausdrcke mit beliebiger Argumentanzahl um sie an ein Primitive
;$ weiterzureichen, das nur zwei Argumente akzeptiert. Zum Beispiel:
;+   Example of a recursive macro-function: the following function adapts forms
;+ with any number of arguments to transfer them to a primitive accepting only
;+ two arguments. For instance:
;-    (addit 1 2 3 ... <n>) => (+ 1 (+ 2 (+ 3 ... (+ <n-1> <n>) ...)))
;; Noter l'emploi de ",@" dans un anti-quote.
;$ Man beachte die Benutzung von ",@" in einem Backquote.
;+ Note the use of ",@" in an back-quote.

(defmacro addit (&rest liste)
  (if (null (cddr liste)) `(+ ,(car liste) ,(cadr liste))
    `(+ ,(car liste) (addit ,@(cdr liste)))))

;;    partir de la seule primitive cons, et sur la base du mme algorithme, on
;; peut raliser l'expansion suivante, comme quivalente de la primitive list :
;$   Einzig vom Primitive cons ausgehend  und basierend auf demselben
;$ Algorithmus kann man die folgende Erweiterung als Equivalent zum
;$ list-Primitive erzeugen:
;+   Starting with the sole primitive cons, and based on the same algorithm, one
;+ can realize the following expansion as equivalent to the list primitive:
;-    (liste a b c ... <z>) => (cons a (cons b (cons c ... (cons <z> ()) ...)))
;; Mais...  condition, toutefois, de possder en outre une quivalente de
;; append, elle-mme macro-fonction rcursive ralise par une troisime
;; fonction, elle aussi rcursive et n'ayant recours, pour finir, qu' cons !
;;   On peut imaginer r-crire ainsi la quasi totalit des primitives LISP, 
;; partir des seules primitives de manipulations lmentaires de doublets : car,
;; cdr, cons et endp.
;$ Jedoch... bruchte man dann allerdings zustzlich ein Equivalent zu append,
;$ welches wiederum eine rekursive Makrofunktion erzeugt durch eine dritte
;$ Funktion ist, welche auch rekursiv ist und im Endeffekt nur auf cons
;$ zurckgreift!
;$   Es ist daher vorstellbar, alle LISP-Primitives nur auf der Basis der
;$ elementaren cons-Zellen-Manipulations-Primitives zu schreiben: car, cdr, cons
;$ und endp.
;+ But... one then additionally needs, however, an equivalent to append, itself
;+ a recursive macro-function realized by a third function, also recursive and
;+ ultimately resorting only to cons!
;+   It is thus imaginable to rewrite almost all LISP primitives on the basis of
;+ the sole elementary cons cell manipulation primitives: car, cdr, cons and
;+ endp.

(defun enfile2 (lis1 lis2) (declare (notinline enfile2));quiv. append  2 args
  (if (endp (cdr lis1)) (cons (car lis1) lis2)
  (cons (car lis1) (enfile2 (cdr lis1) lis2))))

(defmacro enfileN (&rest lis)                           ;quiv. primitive append
  (if (endp (cddr lis)) `(enfile2 ,(car lis) ,@(cdr lis))
  `(enfile2 ,(car lis) (enfileN ,@(cdr lis)))))

(defmacro liste (&rest lis)                             ;quiv. primitive list
  (if (endp lis) nil
  (enfileN '(cons) (cons (car lis) ()) (cons `(liste ,@(cdr lis)) ()))))

(trace enfile2)                                         ;pour les dmonstrations
